/***********************************************************************************************/
/******** DANGER !! DANGER !! DANGER !! DO NOT EDIT FILE -- IT WAS GENERATED AUTOMATICALLY !!! */
/***********************************************************************************************/

/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <math.h>
#include "sph_common_keywords.h"
#include "sph_ifs_keywords.h"
#include "sph_ifs_tags.h"
#include "sph_ifs_master_detector_flat.h"
#include "sph_master_frame.h"
#include "sph_error.h"
#include "sph_cube.h"
#include "sph_utils.h"

/*-----------------------------------------------------------------------------
  Error Codes
 -----------------------------------------------------------------------------*/

sph_error_code SPH_IFS_MASTER_DETECTOR_FLAT_GENERAL              = SPH_IFS_MASTER_DETECTOR_FLAT_ERR_START + 0;
sph_error_code SPH_IFS_MASTER_DETECTOR_FLAT_NO_VALID_DIT_LIST    = SPH_IFS_MASTER_DETECTOR_FLAT_ERR_START + 1;

sph_error_code SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING    = SPH_IFS_MASTER_DETECTOR_FLAT_ERR_START + 3;
sph_error_code SPH_IFS_MASTER_DETECTOR_FLAT_FRAMES_MISSING       = SPH_IFS_MASTER_DETECTOR_FLAT_ERR_START + 4;

/*-----------------------------------------------------------------------------
 Private function prototypes
 -----------------------------------------------------------------------------*/

static int sph_ifs_master_detector_flat_create_plugin ( cpl_plugin * );
static int sph_ifs_master_detector_flat_exec_plugin ( cpl_plugin * );
static int sph_ifs_master_detector_flat_destroy_plugin ( cpl_plugin * );

/*-----------------------------------------------------------------------------
 Static variables
 -----------------------------------------------------------------------------*/

static const char* sph_ifs_master_detector_flat_help = 
    "RECIPE INPUT FRAMES: \n"
    "-------------------- \n"
    "Tag                           Min,Max   Description\n"
    "IFS_DETECTOR_FLAT_FIELD_RAW     2,500   The input raw flat frames. To be taken with different illumination levels.\n"
    "IFS_MASTER_DARK                 0,  1   The master dark frame. Should have the same DIT and readout mode as\n"
    "                                        the raw frames.\n"
    "IFS_LARGE_SCALE_FLAT            0,  1   A large scale structure flat. If provided the result is divided by this before saved.\n"
    "IFS_PREAMP_FLAT                 0,  1   A preamp correction flat. If provided the result is divided by this before saved.\n"
    "IFS_STATIC_BADPIXELMAP          0,  1   Optional frame of static bad pixels. If provided, this is used in addtion to\n"
    "                                        the badpixels from the master dark.\n"
    "\n"
    "RAW FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Type   Optional  Description\n"
    "\n"
    "DESCRIPTION:\n"
    "------------\n"
    "The detector flat field recipe for IFS is very similar to the instrument\n"
    "flat\n"
    "\n"
    "field recipe for IRDIS. The recipe as described here uses input\n"
    "exposures taken with the\n"
    "\n"
    "narrow band or broad band calibration lamps.\n"
    "\n"
    "Several types of flat fields can be produced – in accordance with the\n"
    "calibration plan\n"
    "\n"
    "and the need to have seperate flat field components to provide maximal\n"
    "time stability and\n"
    "\n"
    "flat fielding accuracy. The recipe can be used to create a preamplifier\n"
    "correction flat\n"
    "\n"
    "(which can be used to remove the stripe structure caused by the pre\n"
    "amplifiers), a large\n"
    "\n"
    "scale flat field which is a smoothed flat field and hence only shows\n"
    "large scale structures,\n"
    "\n"
    "and a normal flat field.\n"
    "\n"
    "These master flats are used in all subsequent recipes that need to\n"
    "remove the pixel to\n"
    "\n"
    "pixel variation in the signal response of the detector. The recipe\n"
    "creates master\n"
    "\n"
    "calibration frames, using the input exposures which should be taken as\n"
    "described in the\n"
    "\n"
    "IFS calibration plan. The usual procedure to create a flat field is as\n"
    "follows.\n"
    "\n"
    "All raw frames are read in and dark subtracted.\n"
    "\n"
    "The frames are then corrected for the pre-amplifier variations.\n"
    "\n"
    "In case that a pre-amplifier correction flat was provided that frame is\n"
    "used,\n"
    "\n"
    "otherwise the correction is calculated directly from the raw frames\n"
    "(note that it\n"
    "\n"
    "is currently not possible to skip the pre amplifier correction\n"
    "altogether).\n"
    "\n"
    "This correction is a division operation rather than a subtraction.\n"
    "\n"
    "After this correction, the mean pixel value across the image is\n"
    "determined for\n"
    "\n"
    "all exposures. For every pixel $p=(x,y)$, a set of $m_i$,$v_i(x,y)$ data\n"
    "pairs\n"
    "\n"
    "are stored with $m_i$ being the mean value of exposure $i$ described\n"
    "above,\n"
    "\n"
    "and $v_i(x,y)$ being the pixel value for pixel $p(x,y)$ in exposure $i$.\n"
    "\n"
    "The flat field value is defined as the slope $c_i$ of a linear fit F to\n"
    "the data\n"
    "\n"
    "$m_i$,$v_i$. The resulting slope represents the response of an indiviual\n"
    "pixel\n"
    "\n"
    "$p(x,y)$ to illumination relative to the detector mean response.\n"
    "\n"
    "The value will thus naturally be close to 1 and a division by that value\n"
    "will\n"
    "\n"
    "correct for a pixel’s deviation from the average detector response.\n"
    "\n"
    "The fit itself is performed either using a maximum likelyhood method\n"
    "\n"
    "or a robust fitting method which minimizes the sum of the absolute value\n"
    "of\n"
    "\n"
    "the deviations rather than the sum of the squares of the deviations\n"
    "\n"
    "(see e.g. Numerical Recipes for the algorithm). The robust fitting\n"
    "method will\n"
    "\n"
    "yield better results when significant outliers (e.g. due to cosmic rays)\n"
    "can\n"
    "\n"
    "be expected, but does not allow anything but linear fits and can hence\n"
    "not be used\n"
    "\n"
    "to assess detector non-linearity.\n"
    "\n"
    "In case that a large scale flat was provided as an input, this large\n"
    "scale flat is\n"
    "\n"
    "now divided out from the detector flat,\n"
    "\n"
    "removing the large scale dependence and leaving only the pixel-to-pixel\n"
    "variations.\n"
    "\n"
    "If no large scale was provided as input, no such correction is\n"
    "performed.\n"
    "\n"
    "The flat field values are saved as an image as the main product of the\n"
    "recipe.\n"
    "\n"
    "Aditionally, the recipe may also produce as output a map of all pixels\n"
    "that\n"
    "\n"
    "are identified as non-linear. The criteria for non-linearity are set by\n"
    "the user\n"
    "\n"
    "parameters and can be either pixels that have a flat field value outside\n"
    "specified\n"
    "\n"
    "bounds and/or pixels for which the linear fit produces a reduced\n"
    "chi-squared above\n"
    "\n"
    "a given threshold value. For reliable non-linearity flagging using the\n"
    "reduced\n"
    "\n"
    "chi-squared fit many high quality input exposures are needed.\n"
    "\n"
    "In case that a non zero smoothing value was given, a large scale flat is\n"
    "also created\n"
    "\n"
    "by smoothing the flat field with either a gaussian kernel using FFT or a\n"
    "square kernel using\n"
    "\n"
    "the specific CPL filter algorithm. Unless you know what you are doing\n"
    "leave the default method\n"
    "\n"
    "here which is the FFT smoothing.\n"
    "\n"
    "Choosing and creating the correct detector flat field for IFS is an\n"
    "important but tricky step.\n"
    "\n"
    "The best type of flat to use may depend on the exact science goals and\n"
    "final procedures and advice\n"
    "\n"
    "for this step will be included in this manual at this point as soon as\n"
    "some on sky data for various\n"
    "\n"
    "type of science targets has been collected.\n"
    "\n"
    "Until that point the general advice is to:\n"
    "\n"
    "1.  calculate a preamp flat field using white lamp flat field data\n"
    "\n"
    "2.  calculate flat large scale fields for each of the coloured flat\n"
    "    field lamps using the corresponding raw frames\n"
    "\n"
    "    (but see below) and setting the lambda input parameter of this\n"
    "    recipe in each case to the lamp wavelength.\n"
    "\n"
    "    The preamp flat calculated at the previous step should be provided\n"
    "    as input.\n"
    "\n"
    "3.  calculate a large scale flat for the white lamp (setting lambda=-1),\n"
    "    again providing the preamp flat as\n"
    "\n"
    "    additional input\n"
    "\n"
    "4.  Use the preamp flat in addition to the all large scale flats when\n"
    "    using recipes that need to correct the\n"
    "\n"
    "    detector flat field (e.g. science recipes).\n"
    "\n"
    "In some cases, it may be acceptable to use the white lamp raw frames as\n"
    "input to step 2 – this is effectively assuming\n"
    "\n"
    "that there is no strong wavelength dependence of the detector flat\n"
    "field. Since taking flat field data is time consuming\n"
    "\n"
    "this is a good option for all applications where an extremely accurate\n"
    "flat field correction is not required.\n"
    "\n"
    "\n"
    "RECIPE PRODUCTS: \n"
    "-------------------- \n"
    "Tag                      Format         Description\n"
    "IFS_MASTER_DETECTOR_FLAT_FIELD     FITS[Im(4)]    The flat field. This is saved as a FITS file with 4 extensions, the flat values,\n"
    "                                        the badpixels (hotpixels and non-linear pixels), the rms error on the flat\n"
    "                                        and a weightmap. Used if the lamp in use cannot be derived.\n"
    "IFS_MASTER_DFF_LONG1               FITS[Im(4)]    Same as above, produced from all input raw frames which had LAMP1 (1.020mum) switched on\n"
    "IFS_MASTER_DFF_LONG2               FITS[Im(4)]    Same as above, produced from all input raw frames which had LAMP2 (1.230mum) switched on\n"
    "IFS_MASTER_DFF_LONG3               FITS[Im(4)]    Same as above, produced from all input raw frames which had LAMP3 (1.300mum) switched on\n"
    "IFS_MASTER_DFF_LONG4               FITS[Im(4)]    Same as above, produced from all input raw frames which had LAMP4 (1.540mum) switched on\n"
    "IFS_MASTER_DFF_LONGBB              FITS[Im(4)]    Same as above, produced from all input raw frames which had either LAMP5 or LAMP6 (broad band) switched on.\n"
    "IFS_NON_LINEAR_PIXELMAP            FITS[Im(1)]    Optional output of all the non-linear pixels determined. All pixels as determined\n"
    "                                        in this recipe using the ird.instrument_flat.badpix_*tolerance parameters.\n"
    "\n"
    "PRODUCT FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Frame          Type    Description\n"
    "\n"
;

/*-----------------------------------------------------------------------------
 Function code
 -----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/**
 * @defgroup sph_ifs_master_detector_flat Create Master Dark Recipe
 *
 * This module provides the recipe plugin and relating functions for the
 * creation of the master dark .
 *
 * @par Synopsis:
 * @code
 *   #include "sph_ifs_master_detector_flat.h"
 * @endcode
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*****************************************************************************
 *
 *
 *      First come the static and global functions necessary for the
 *      normal working of the recipe as plugin....
 *
 *
 * ***************************************************************************/
/*----------------------------------------------------------------------------*/
/**
 @brief    Build the list of available plugins, for this module.
 @param    list    the plugin list
 @return   0 if everything is ok, 1 otherwise
 @note     Only this function is exported

 Create the recipe instance and make it available to the application
 using the interface.
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe * recipe = cpl_calloc( 1, sizeof *recipe);
    cpl_plugin * plugin = &recipe->interface;

    if (cpl_plugin_init(plugin,
                        CPL_PLUGIN_API,
                        SPH_IFS_MASTER_DETECTOR_FLAT_VERSION,
                        CPL_PLUGIN_TYPE_RECIPE,
                        "sph_ifs_master_detector_flat",
                        "Creation of the master detector flat frame",
                        sph_ifs_master_detector_flat_help, "Ole Moeller-Nilsson",
                        "moeller@mpia-hd.mpg.de", cpl_get_license("SPHERE DRH","2012"),
                        sph_ifs_master_detector_flat_create_plugin,
                        sph_ifs_master_detector_flat_exec_plugin,
                        sph_ifs_master_detector_flat_destroy_plugin) )
    {
        cpl_msg_error( cpl_func, "Plugin initialization failed");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }

    if (cpl_pluginlist_append(list, plugin) )
    {
        cpl_msg_error( cpl_func, "Error adding plugin to list");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Setup the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int sph_ifs_master_detector_flat_create_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe             = NULL;

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: An error is already set: %s",
                       cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error(cpl_func, "Null plugin");
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    if ( plugin != NULL ) {
        recipe = (cpl_recipe *)plugin;
        recipe->parameters = sph_ifs_master_detector_flat_create_paramlist();
        if ( recipe->parameters == NULL ) {
            return SPH_IFS_MASTER_DETECTOR_FLAT_GENERAL;
        }
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Testing for setup of the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Wrapper to make the static plugin creation function accessible from outside
 for testing purposes.
 */
/*----------------------------------------------------------------------------*/
int sph_ifs_master_detector_flat_create_test(cpl_plugin * plugin)
{
    return (int)sph_ifs_master_detector_flat_create_plugin(plugin);
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Execute the plugin instance given by the interface
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_ifs_master_detector_flat_exec_plugin(cpl_plugin * plugin)
{

    cpl_recipe*             recipe                  = NULL;
    int                     recipe_status           = 0;
    sph_ifs_master_detector_flat*    sph_recipe      = NULL;
    cpl_errorstate          initial_errorstate      = cpl_errorstate_get();

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: A cpl error is already set: %s",
        cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
        return (int)cpl_error_get_code();
    }

    recipe = (cpl_recipe *)plugin;

    if (recipe->parameters == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL parameter list");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (recipe->frames == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL frame set");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    sph_recipe = sph_ifs_master_detector_flat_new( recipe->frames, recipe->parameters );
    if ( sph_recipe == NULL ) {
        cpl_msg_error( cpl_func, "SPHERE recipe initialisation call failed. "
                "Please check the SPHERE error messages.");
        sph_end_erex(); //close the sphere system
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    recipe_status = sph_ifs_master_detector_flat_run( sph_recipe );

    sph_ifs_master_detector_flat_delete( sph_recipe );

    if ( !cpl_errorstate_is_equal(initial_errorstate) )
    {
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);
    }
    sph_end_erex(); //close the sphere system
    return recipe_status;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Destroy what has been created by the 'create' function
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_ifs_master_detector_flat_destroy_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe;

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    recipe = (cpl_recipe *)plugin;

    cpl_parameterlist_delete(recipe->parameters);

    return 0;
}

/*****************************************************************************
 *
 *
 *      Here come the "Memnber" Functions of the sph_ifs_master_detector_flat struct
 *
 *
 * ***************************************************************************/

/*----------------------------------------------------------------------------*/
/**
 @brief    Create and return the standard parameterlist for the recipe

 @return   the standard parameterlis or NULL if an error occcured.

 This function creates the standard parameterlist for the recipe
 and returns it.

 The return value is NULL when an error occured.
 */
/*----------------------------------------------------------------------------*/
cpl_parameterlist*  sph_ifs_master_detector_flat_create_paramlist(void)
{
    cpl_parameterlist*          result              = NULL;
    cpl_parameter*              p                   = NULL;
    result = cpl_parameterlist_new();

    if (result == NULL)
    {
        cpl_msg_error( cpl_func, "Null parameterlist");
        return NULL;
    }
    else {
        /* Code to set up parameters GENERATED DO NOT EDIT */

        p = cpl_parameter_new_value("ifs.master_detector_flat.outfilename",
                                   CPL_TYPE_STRING,
                                   "The output filename for the product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"ifs.master_detector_flat",
                                   "master_detector_flat.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.save_addprod",
                                   CPL_TYPE_BOOL,
                                   "Flag signalling hwether additional products should be saved, "
                                   "in this case a large scale flat, a preamp flat, and a "
                                   "hot pixels product. "
                                   ,"ifs.master_detector_flat",
                                   0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.lss_outfilename",
                                   CPL_TYPE_STRING,
                                   "The output filename for the large scale flat product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"ifs.master_detector_flat",
                                   "large_scale_flat.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.preamp_outfilename",
                                   CPL_TYPE_STRING,
                                   "The output filename for the preamplifier flat product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"ifs.master_detector_flat",
                                   "preamp_flat.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.make_badpix",
                                   CPL_TYPE_BOOL,
                                   "Controls if a seperate static badpixel map is requested for output. "
                                   ,"ifs.master_detector_flat",
                                   0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.badpixfilename",
                                   CPL_TYPE_STRING,
                                   "Controls the filename of the badpixel map, if requested for output. Ignored if no make_badpix is FALSE.@pd "
                                   ,"ifs.master_detector_flat",
                                   "dff_badpixels.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.robust_fit",
                                   CPL_TYPE_BOOL,
                                   "Controls if fitting method is to be a robust linear fit. This will reduce the effect of cosmic "
                                   "rays and other temporary bad pixels. See e.g. Numerical Recipes for a description of the algorithm "
                                   ,"ifs.master_detector_flat",
                                   0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_enum("ifs.master_detector_flat.coll_alg",
                                   CPL_TYPE_INT,
                                   "The collapse algorithm to use. 0 = Mean, 1 = Median, 2 = Clean mean. "
                                   ,"ifs.master_detector_flat",
                                   2 ,
                                   3,0,1,2);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ifs.master_detector_flat.clean_mean.reject_high",
                                   CPL_TYPE_INT,
                                   "The clean mean reject pixels on high end. "
                                   ,"ifs.master_detector_flat",
                                   0 ,
                                   0,20);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ifs.master_detector_flat.clean_mean.reject_low",
                                   CPL_TYPE_INT,
                                   "The clean mean reject pixels on low end. "
                                   ,"ifs.master_detector_flat",
                                   0 ,
                                   0,20);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.badpix_lowtolerance",
                                   CPL_TYPE_DOUBLE,
                                   "Threshold value for linearity badpixels. "
                                   "All pixels that have a flat field (slope) below this value will be flagged as bad. "
                                   ,"ifs.master_detector_flat",
                                   0.1 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.badpix_uptolerance",
                                   CPL_TYPE_DOUBLE,
                                   "Threshold value for linearity badpixels. "
                                   "All pixels that have a flat field (slope) above this value will be flagged as bad. "
                                   ,"ifs.master_detector_flat",
                                   10.0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.badpix_chisqtolerance",
                                   CPL_TYPE_DOUBLE,
                                   "Threshold value for linearity badpixels. All pixels that have chi-squared value for "
                                   "the linear fit that is above this value will be flagged as bad "
                                   ,"ifs.master_detector_flat",
                                   50.0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.lambda",
                                   CPL_TYPE_DOUBLE,
                                   "If this is set to a value > 0, the resulting master flat will be assigned the given calibration wavelength. "
                                   "In case that there are corresponding keywords present in the input raw frames, these are ignored in this case. "
                                   ,"ifs.master_detector_flat",
                                   -1.0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ifs.master_detector_flat.smoothing_length",
                                   CPL_TYPE_DOUBLE,
                                   "The smooting length for the large scale flats. "
                                   ,"ifs.master_detector_flat",
                                   10.0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_enum("ifs.master_detector_flat.smoothing_method",
                                   CPL_TYPE_INT,
                                   "The smooting method to use: 0 is square kernel using cpl_filter, 1 gauss kernel using FFT. "
                                   ,"ifs.master_detector_flat",
                                   1 ,
                                   2,0,1);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        /* End of code to set up parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    }


    return result;


}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe frames, and assign them to the sph_ifs_master_detector_flat
           structure.
 @param    self     the sph_ifs_master_detector_flat recipe structure
 @return   error code of the check.

 This checks the frames that were given in the sph_ifs_master_detector_flat_new function
 as parameters for the correctness and completeness. If the frames needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code  sph_ifs_master_detector_flat_check_frames( sph_ifs_master_detector_flat* self )
{
    sph_error_code           rerr            = CPL_ERROR_NONE;
    int                      ii	             = 0;
    cpl_frame*               aframe          = NULL;
    cpl_propertylist*        pl              = NULL;
    int	                     haskey          = 0;
    int                      okval           = 0;
    /* Code to check frames GENERATED DO NOT EDIT */

    aframe = cpl_frameset_find( self->inframes, SPH_IFS_TAG_DFF_RAW );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_RAW );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->rawframes = sph_utils_extract_frames( self->inframes, SPH_IFS_TAG_DFF_RAW );
    if ( ! self->rawframes )
    {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_ERROR,
                         "Could not extract rawframes frames."
                         "to use them check that they have the %s tag.",
                         SPH_IFS_TAG_DFF_RAW);
        rerr = SPH_IFS_MASTER_DETECTOR_FLAT_FRAMES_MISSING;
        return rerr;
    }

    else {

         if ( cpl_frameset_get_size( self->rawframes ) < 2) {
             sph_error_raise(CPL_ERROR_ILLEGAL_INPUT,
                 __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                 "Not enough frames in frameset SPH_IFS_TAG_DFF_RAW. "
                 "Was expecting at least 2 but only got %d. ",
             cpl_frameset_get_size( self->rawframes ));
             return CPL_ERROR_ILLEGAL_INPUT;
         }
         for ( ii = 0; ii < cpl_frameset_get_size( self->rawframes ); ++ii ) {
             rerr = cpl_frame_set_group( cpl_frameset_get_frame( self->rawframes, ii ), CPL_FRAME_GROUP_RAW );
             pl = cpl_propertylist_load(cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )),0);
             if (pl == NULL ) {
                 sph_error_raise(cpl_error_get_code(),
                    __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                 "Could not read keywords from file %s. "
                 "Either the file is not readable or corrupted. ",
                  cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )));
             }
             else {
                 cpl_propertylist_delete(pl); pl=NULL;
             }
         }
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IFS_TAG_DARK_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_dark_frame = sph_utils_extract_frame( self->inframes, SPH_IFS_TAG_DARK_CALIB );
    if ( ! self->master_dark_frame )
    {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract master_dark_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IFS_TAG_DARK_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_dark_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IFS_TAG_LSS_FLAT );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_lss_flat = sph_utils_extract_frame( self->inframes, SPH_IFS_TAG_LSS_FLAT );
    if ( ! self->master_lss_flat )
    {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract master_lss_flat frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IFS_TAG_LSS_FLAT);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_lss_flat, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IFS_TAG_PREAMP_FLAT );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->preamp_flat = sph_utils_extract_frame( self->inframes, SPH_IFS_TAG_PREAMP_FLAT );
    if ( ! self->preamp_flat )
    {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract preamp_flat frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IFS_TAG_PREAMP_FLAT);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->preamp_flat, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IFS_TAG_STATIC_BADPIXELMAP_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->static_badpixel_frame = sph_utils_extract_frame( self->inframes, SPH_IFS_TAG_STATIC_BADPIXELMAP_CALIB );
    if ( ! self->static_badpixel_frame )
    {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract static_badpixel_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IFS_TAG_STATIC_BADPIXELMAP_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->static_badpixel_frame, CPL_FRAME_GROUP_CALIB );
    }

    /* End of code to check frames BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    self->current_raw_frameset = sph_utils_extract_frames_group( self->inframes, CPL_FRAME_GROUP_RAW );
    haskey = 0; /* This is just to remove compiler warnings */
    okval = 0;/* This is just to remove compiler warnings */
    ii = 0; pl = NULL; /* This is just to remove compiler warnings */

    return rerr;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe parameters, and assign them to the sph_ifs_master_detector_flat
           structure.
 @param    self     the sph_ifs_master_detector_flat recipe structure
 @return   error code of the check.

 This checks the parameters that were given in the sph_ifs_master_detector_flat_new function
 as parameters for the correctness and completeness. If the parameters needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_ifs_master_detector_flat_check_params( sph_ifs_master_detector_flat* self ) {
    sph_error_code           	   rerr               = CPL_ERROR_NONE;
    const cpl_parameter*           param              = NULL;

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
    	self->framecomb_parameterlist = NULL;
    }
    self->framecomb_parameterlist = cpl_parameterlist_new();
    /* Code to check parameters GENERATED DO NOT EDIT */

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.outfilename");
    self->outfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.outfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.save_addprod");
    self->save_addprod = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.save_addprod"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.lss_outfilename");
    self->lss_outfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.lss_outfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.preamp_outfilename");
    self->pream_outfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.preamp_outfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.make_badpix");
    self->make_badpix = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.make_badpix"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.badpixfilename");
    self->badpix_filename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.badpixfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.robust_fit");
    self->robust_fit = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.robust_fit"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.coll_alg");
    self->coll_alg = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.coll_alg"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.clean_mean.reject_high");
    self->clean_mean_reject_high = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    cpl_parameterlist_append( self->framecomb_parameterlist,
                              cpl_parameter_new_range( "clean_mean.reject_high",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       cpl_parameter_get_int( param ) ,
                                                       0,20
                                                      )
                            );

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.clean_mean.reject_low");
    self->clean_mean_reject_low = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    cpl_parameterlist_append( self->framecomb_parameterlist,
                              cpl_parameter_new_range( "clean_mean.reject_low",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       cpl_parameter_get_int( param ) ,
                                                       0,20
                                                      )
                            );

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.badpix_lowtolerance");
    self->badpix_lowtolerance = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.badpix_lowtolerance"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.badpix_uptolerance");
    self->badpix_uptolerance = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.badpix_uptolerance"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.badpix_chisqtolerance");
    self->badpix_chisqtolerance = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.badpix_chisqtolerance"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.lambda");
    self->lambda = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.lambda"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.smoothing_length");
    self->smoothl = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.smoothing_length"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ifs.master_detector_flat.smoothing_method");
    self->smooth_method = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ifs.master_detector_flat.smoothing_method"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    /* End of code to check parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/

    return rerr;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Create a new sph_ifs_master_detector_flat structure.

 @param    frameset     the input recipe frames
 @param    parlist      the input recipe parameters

 @return   pointer to newly created structure or NULL in case of error

 This creates a new sph_ifs_master_detector_flat_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_ifs_master_detector_flat* sph_ifs_master_detector_flat_new( cpl_frameset * frameset,
                                              cpl_parameterlist * parlist )
{
    sph_ifs_master_detector_flat*    result      = NULL;
    int                     rerr        = CPL_ERROR_NONE;

    sph_init_erex();
    if ( frameset == NULL || parlist == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return NULL;
    }

    result = cpl_calloc( 1, sizeof(sph_ifs_master_detector_flat) );

    if ( result == NULL ) {
        sph_error_raise( SPH_IFS_MASTER_DETECTOR_FLAT_GENERAL, __FILE__, __func__,
                         __LINE__, SPH_ERROR_ERROR,
                         "Could not allocate the structure.");
        return result;
    }

    result->inframes = frameset;
    result->inparams = parlist;

    rerr = sph_ifs_master_detector_flat_check_frames( result );
    if ( rerr == CPL_ERROR_NONE) rerr = sph_ifs_master_detector_flat_check_params( result );

    if ( rerr != CPL_ERROR_NONE ) {
    	if ( result ) {
    		sph_ifs_master_detector_flat_delete( result );
    	}
    	result = NULL;
    }

    return result;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Delete the sph_ifs_master_detector_flat structure.

 @param    self         the structure tp delete

 @return   error code of the operation

 This creates a new sph_ifs_master_detector_flat_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_ifs_master_detector_flat_delete( sph_ifs_master_detector_flat* self )
{
    sph_error_code          rerr            = CPL_ERROR_NONE;

    if ( self == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return rerr;
    }

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
	    self->framecomb_parameterlist = NULL;
    }

    /* Code to delete recipe pointers GENERATED DO NOT EDIT */

    if ( self->rawframes != NULL ) {
        cpl_frameset_delete( self->rawframes );
        self->rawframes = NULL;
    }
    if ( self->master_dark_frame != NULL ) {
        cpl_frame_delete( self->master_dark_frame );
        self->master_dark_frame = NULL;
    }
    if ( self->master_lss_flat != NULL ) {
        cpl_frame_delete( self->master_lss_flat );
        self->master_lss_flat = NULL;
    }
    if ( self->preamp_flat != NULL ) {
        cpl_frame_delete( self->preamp_flat );
        self->preamp_flat = NULL;
    }
    if ( self->static_badpixel_frame != NULL ) {
        cpl_frame_delete( self->static_badpixel_frame );
        self->static_badpixel_frame = NULL;
    }
 
    /* End of code to delete pointers BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    if ( self->current_raw_frameset ) {
    	cpl_frameset_delete(self->current_raw_frameset);
	self->current_raw_frameset = NULL;
    }
    sph_polygon_free_all();
    cpl_free(    self);
    return rerr;

}


/**@}*/
