/***********************************************************************************************/
/******** DANGER !! DANGER !! DANGER !! DO NOT EDIT FILE -- IT WAS GENERATED AUTOMATICALLY !!! */
/***********************************************************************************************/

/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <math.h>
#include "sph_common_keywords.h"
#include "sph_ird_keywords.h"
#include "sph_ird_tags.h"
#include "sph_ird_atmospheric.h"
#include "sph_master_frame.h"
#include "sph_error.h"
#include "sph_cube.h"
#include "sph_utils.h"

/*-----------------------------------------------------------------------------
  Error Codes
 -----------------------------------------------------------------------------*/

sph_error_code SPH_IRD_ATMOSPHERIC_GENERAL              = SPH_IRD_ATMOSPHERIC_ERR_START + 0;
sph_error_code SPH_IRD_ATMOSPHERIC_NO_VALID_DIT_LIST    = SPH_IRD_ATMOSPHERIC_ERR_START + 1;

sph_error_code SPH_IRD_ATMOSPHERIC_PARAMETER_MISSING    = SPH_IRD_ATMOSPHERIC_ERR_START + 3;
sph_error_code SPH_IRD_ATMOSPHERIC_FRAMES_MISSING       = SPH_IRD_ATMOSPHERIC_ERR_START + 4;

/*-----------------------------------------------------------------------------
 Private function prototypes
 -----------------------------------------------------------------------------*/

static int sph_ird_atmospheric_create_plugin ( cpl_plugin * );
static int sph_ird_atmospheric_exec_plugin ( cpl_plugin * );
static int sph_ird_atmospheric_destroy_plugin ( cpl_plugin * );

/*-----------------------------------------------------------------------------
 Static variables
 -----------------------------------------------------------------------------*/

static const char* sph_ird_atmospheric_help = 
    "RECIPE INPUT FRAMES: \n"
    "-------------------- \n"
    "Tag                           Min,Max   Description\n"
    "IRD_ATMOSPHERIC_RAW             1,500   The raw frames of a spectroscopic standard star\n"
    "IRD_MASTER_DARK                 1,  1   The master dark. Should have the same DIT and the same\n"
    "                                        read out mode as the raw frames.\n"
    "IRD_FLAT_FIELD                  1,  1   The master flat. Should have the same filter settings as\n"
    "                                        raw frames.\n"
    "IRD_WAVECALIB                   0,  1   The wavelength calibration file.\n"
    "\n"
    "RAW FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Type   Optional  Description\n"
    "\n"
    "DESCRIPTION:\n"
    "------------\n"
    "This recipe is responsible for creating sky spectra. \n"
    "Its identical to the science_spectroscopy recipe. \n"
    "RECIPE PRODUCTS: \n"
    "-------------------- \n"
    "Tag                      Format         Description\n"
    "IRD_ATMOSPHERIC                    FITS[Im(4)]    The frame containing the calculated, estimated absorption spectrum of the\n"
    "                                        atmosphere.         \n"
    "\n"
    "PRODUCT FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Frame          Type    Description\n"
    "\n"
;

/*-----------------------------------------------------------------------------
 Function code
 -----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/**
 * @defgroup sph_ird_atmospheric Create Master Dark Recipe
 *
 * This module provides the recipe plugin and relating functions for the
 * creation of the master dark .
 *
 * @par Synopsis:
 * @code
 *   #include "sph_ird_atmospheric.h"
 * @endcode
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*****************************************************************************
 *
 *
 *      First come the static and global functions necessary for the
 *      normal working of the recipe as plugin....
 *
 *
 * ***************************************************************************/
/*----------------------------------------------------------------------------*/
/**
 @brief    Build the list of available plugins, for this module.
 @param    list    the plugin list
 @return   0 if everything is ok, 1 otherwise
 @note     Only this function is exported

 Create the recipe instance and make it available to the application
 using the interface.
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe * recipe = cpl_calloc( 1, sizeof *recipe);
    cpl_plugin * plugin = &recipe->interface;

    if (cpl_plugin_init(plugin,
                        CPL_PLUGIN_API,
                        SPH_IRD_ATMOSPHERIC_VERSION,
                        CPL_PLUGIN_TYPE_RECIPE,
                        "sph_ird_atmospheric",
                        "Measure the atmospheric effect",
                        sph_ird_atmospheric_help, "Ole Moeller-Nilsson",
                        "moeller@mpia-hd.mpg.de", cpl_get_license("SPHERE DRH","2012"),
                        sph_ird_atmospheric_create_plugin,
                        sph_ird_atmospheric_exec_plugin,
                        sph_ird_atmospheric_destroy_plugin) )
    {
        cpl_msg_error( cpl_func, "Plugin initialization failed");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }

    if (cpl_pluginlist_append(list, plugin) )
    {
        cpl_msg_error( cpl_func, "Error adding plugin to list");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Setup the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int sph_ird_atmospheric_create_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe             = NULL;

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: An error is already set: %s",
                       cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error(cpl_func, "Null plugin");
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    if ( plugin != NULL ) {
        recipe = (cpl_recipe *)plugin;
        recipe->parameters = sph_ird_atmospheric_create_paramlist();
        if ( recipe->parameters == NULL ) {
            return SPH_IRD_ATMOSPHERIC_GENERAL;
        }
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Testing for setup of the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Wrapper to make the static plugin creation function accessible from outside
 for testing purposes.
 */
/*----------------------------------------------------------------------------*/
int sph_ird_atmospheric_create_test(cpl_plugin * plugin)
{
    return (int)sph_ird_atmospheric_create_plugin(plugin);
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Execute the plugin instance given by the interface
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_ird_atmospheric_exec_plugin(cpl_plugin * plugin)
{

    cpl_recipe*             recipe                  = NULL;
    int                     recipe_status           = 0;
    sph_ird_atmospheric*    sph_recipe      = NULL;
    cpl_errorstate          initial_errorstate      = cpl_errorstate_get();

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: A cpl error is already set: %s",
        cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
        return (int)cpl_error_get_code();
    }

    recipe = (cpl_recipe *)plugin;

    if (recipe->parameters == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL parameter list");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (recipe->frames == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL frame set");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    sph_recipe = sph_ird_atmospheric_new( recipe->frames, recipe->parameters );
    if ( sph_recipe == NULL ) {
        cpl_msg_error( cpl_func, "SPHERE recipe initialisation call failed. "
                "Please check the SPHERE error messages.");
        sph_end_erex(); //close the sphere system
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    recipe_status = sph_ird_atmospheric_run( sph_recipe );

    sph_ird_atmospheric_delete( sph_recipe );

    if ( !cpl_errorstate_is_equal(initial_errorstate) )
    {
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);
    }
    sph_end_erex(); //close the sphere system
    return recipe_status;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Destroy what has been created by the 'create' function
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_ird_atmospheric_destroy_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe;

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    recipe = (cpl_recipe *)plugin;

    cpl_parameterlist_delete(recipe->parameters);

    return 0;
}

/*****************************************************************************
 *
 *
 *      Here come the "Memnber" Functions of the sph_ird_atmospheric struct
 *
 *
 * ***************************************************************************/

/*----------------------------------------------------------------------------*/
/**
 @brief    Create and return the standard parameterlist for the recipe

 @return   the standard parameterlis or NULL if an error occcured.

 This function creates the standard parameterlist for the recipe
 and returns it.

 The return value is NULL when an error occured.
 */
/*----------------------------------------------------------------------------*/
cpl_parameterlist*  sph_ird_atmospheric_create_paramlist(void)
{
    cpl_parameterlist*          result              = NULL;
    cpl_parameter*              p                   = NULL;
    result = cpl_parameterlist_new();

    if (result == NULL)
    {
        cpl_msg_error( cpl_func, "Null parameterlist");
        return NULL;
    }
    else {
        /* Code to set up parameters GENERATED DO NOT EDIT */

        p = cpl_parameter_new_value("ird.atmospheric.outfilename",
                                   CPL_TYPE_STRING,
                                   "The output filename for the product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"ird.atmospheric",
                                   "ird_atmospheric.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_enum("ird.atmospheric.coll_alg",
                                   CPL_TYPE_INT,
                                   "The collapse algorithm to use. 0 = Mean, 1 = Median, 3 = Weighted mean. "
                                   ,"ird.atmospheric",
                                   0 ,
                                   3,0,1);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ird.atmospheric.airmass",
                                   CPL_TYPE_DOUBLE,
                                   "The airmass that for the raw frames. If the value is >= 1, this value "
                                   "is written to the header of the product, rather than the value taken "
                                   "from the first raw input file. "
                                   ,"ird.atmospheric",
                                   0.0 ,
                                   0.0,200.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        /* End of code to set up parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    }


    return result;


}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe frames, and assign them to the sph_ird_atmospheric
           structure.
 @param    self     the sph_ird_atmospheric recipe structure
 @return   error code of the check.

 This checks the frames that were given in the sph_ird_atmospheric_new function
 as parameters for the correctness and completeness. If the frames needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code  sph_ird_atmospheric_check_frames( sph_ird_atmospheric* self )
{
    sph_error_code           rerr            = CPL_ERROR_NONE;
    int                      ii	             = 0;
    cpl_frame*               aframe          = NULL;
    cpl_propertylist*        pl              = NULL;
    int	                     haskey          = 0;
    int                      okval           = 0;
    /* Code to check frames GENERATED DO NOT EDIT */

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_ATMOSPHERIC_RAW );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_RAW );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->rawframes = sph_utils_extract_frames( self->inframes, SPH_IRD_TAG_ATMOSPHERIC_RAW );
    if ( ! self->rawframes )
    {
        sph_error_raise( SPH_IRD_ATMOSPHERIC_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_ERROR,
                         "Could not extract rawframes frames."
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_ATMOSPHERIC_RAW);
        rerr = SPH_IRD_ATMOSPHERIC_FRAMES_MISSING;
        return rerr;
    }

    else {

         for ( ii = 0; ii < cpl_frameset_get_size( self->rawframes ); ++ii ) {
             rerr = cpl_frame_set_group( cpl_frameset_get_frame( self->rawframes, ii ), CPL_FRAME_GROUP_RAW );
             pl = cpl_propertylist_load(cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )),0);
             if (pl == NULL ) {
                 sph_error_raise(cpl_error_get_code(),
                    __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                 "Could not read keywords from file %s. "
                 "Either the file is not readable or corrupted. ",
                  cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )));
             }
             else {
                 cpl_propertylist_delete(pl); pl=NULL;
             }
         }
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_DARK_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_dark_frame = sph_utils_extract_frame( self->inframes, SPH_IRD_TAG_DARK_CALIB );
    if ( ! self->master_dark_frame )
    {
        sph_error_raise( SPH_IRD_ATMOSPHERIC_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_ERROR,
                         "Could not extract master_dark_frame frames."
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_DARK_CALIB);
        rerr = SPH_IRD_ATMOSPHERIC_FRAMES_MISSING;
        return rerr;
    }

    else {

         rerr = cpl_frame_set_group( self->master_dark_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_FLAT_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_flat_frame = sph_utils_extract_frame( self->inframes, SPH_IRD_TAG_FLAT_CALIB );
    if ( ! self->master_flat_frame )
    {
        sph_error_raise( SPH_IRD_ATMOSPHERIC_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_ERROR,
                         "Could not extract master_flat_frame frames."
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_FLAT_CALIB);
        rerr = SPH_IRD_ATMOSPHERIC_FRAMES_MISSING;
        return rerr;
    }

    else {

         rerr = cpl_frame_set_group( self->master_flat_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_WAVE_CALIB_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->wave_calib_frame = sph_utils_extract_frame( self->inframes, SPH_IRD_TAG_WAVE_CALIB_CALIB );
    if ( ! self->wave_calib_frame )
    {
        sph_error_raise( SPH_IRD_ATMOSPHERIC_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract wave_calib_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_WAVE_CALIB_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->wave_calib_frame, CPL_FRAME_GROUP_CALIB );
    }

    /* End of code to check frames BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    self->current_raw_frameset = sph_utils_extract_frames_group( self->inframes, CPL_FRAME_GROUP_RAW );
    haskey = 0; /* This is just to remove compiler warnings */
    okval = 0;/* This is just to remove compiler warnings */
    ii = 0; pl = NULL; /* This is just to remove compiler warnings */

    return rerr;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe parameters, and assign them to the sph_ird_atmospheric
           structure.
 @param    self     the sph_ird_atmospheric recipe structure
 @return   error code of the check.

 This checks the parameters that were given in the sph_ird_atmospheric_new function
 as parameters for the correctness and completeness. If the parameters needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_ird_atmospheric_check_params( sph_ird_atmospheric* self ) {
    sph_error_code           	   rerr               = CPL_ERROR_NONE;
    const cpl_parameter*           param              = NULL;

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
    	self->framecomb_parameterlist = NULL;
    }
    self->framecomb_parameterlist = cpl_parameterlist_new();
    /* Code to check parameters GENERATED DO NOT EDIT */

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.atmospheric.outfilename");
    self->outfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_ATMOSPHERIC_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.atmospheric.outfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.atmospheric.coll_alg");
    self->coll_alg = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_ATMOSPHERIC_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.atmospheric.coll_alg"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.atmospheric.airmass");
    self->airmass = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_ATMOSPHERIC_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.atmospheric.airmass"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    /* End of code to check parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/

    return rerr;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Create a new sph_ird_atmospheric structure.

 @param    frameset     the input recipe frames
 @param    parlist      the input recipe parameters

 @return   pointer to newly created structure or NULL in case of error

 This creates a new sph_ird_atmospheric_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_ird_atmospheric* sph_ird_atmospheric_new( cpl_frameset * frameset,
                                              cpl_parameterlist * parlist )
{
    sph_ird_atmospheric*    result      = NULL;
    int                     rerr        = CPL_ERROR_NONE;

    sph_init_erex();
    if ( frameset == NULL || parlist == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return NULL;
    }

    result = cpl_calloc( 1, sizeof(sph_ird_atmospheric) );

    if ( result == NULL ) {
        sph_error_raise( SPH_IRD_ATMOSPHERIC_GENERAL, __FILE__, __func__,
                         __LINE__, SPH_ERROR_ERROR,
                         "Could not allocate the structure.");
        return result;
    }

    result->inframes = frameset;
    result->inparams = parlist;

    rerr = sph_ird_atmospheric_check_frames( result );
    if ( rerr == CPL_ERROR_NONE) rerr = sph_ird_atmospheric_check_params( result );

    if ( rerr != CPL_ERROR_NONE ) {
    	if ( result ) {
    		sph_ird_atmospheric_delete( result );
    	}
    	result = NULL;
    }

    return result;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Delete the sph_ird_atmospheric structure.

 @param    self         the structure tp delete

 @return   error code of the operation

 This creates a new sph_ird_atmospheric_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_ird_atmospheric_delete( sph_ird_atmospheric* self )
{
    sph_error_code          rerr            = CPL_ERROR_NONE;

    if ( self == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return rerr;
    }

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
	    self->framecomb_parameterlist = NULL;
    }

    /* Code to delete recipe pointers GENERATED DO NOT EDIT */

    if ( self->rawframes != NULL ) {
        cpl_frameset_delete( self->rawframes );
        self->rawframes = NULL;
    }
    if ( self->master_dark_frame != NULL ) {
        cpl_frame_delete( self->master_dark_frame );
        self->master_dark_frame = NULL;
    }
    if ( self->master_flat_frame != NULL ) {
        cpl_frame_delete( self->master_flat_frame );
        self->master_flat_frame = NULL;
    }
    if ( self->wave_calib_frame != NULL ) {
        cpl_frame_delete( self->wave_calib_frame );
        self->wave_calib_frame = NULL;
    }
 
    /* End of code to delete pointers BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    if ( self->current_raw_frameset ) {
    	cpl_frameset_delete(self->current_raw_frameset);
	self->current_raw_frameset = NULL;
    }
    sph_polygon_free_all();
    cpl_free(    self);
    return rerr;

}


/**@}*/
