/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cpl.h>
#include "sph_error.h"
#include "sph_master_frame.h"
#include "sph_double_image.h"
#include "sph_quad_image.h"
#ifndef SPH_ZPL_UTILS_H_
#define SPH_ZPL_UTILS_H_

#define ZERO 1e-10
#define SMALL 1e-4
#define DELTA 1e-10
#define isZero(A) ( (A < ZERO) && (A > -ZERO) )
#define isSmall(A) ( (A < SMALL) && (A > -SMALL) )
#define isSame(A, B) ( ((A-B) < ZERO) && ((A-B) > -ZERO) )
#define isSimilar(A, B) ( ((A-B) < SMALL) && ((A-B) > -SMALL) )
#define isBetween(A, B, C) ( ((A-B) > -ZERO) && ((A-C) < ZERO) )

int sph_zpl_utils_check_format( cpl_frameset* rawframes );
int sph_zpl_utils_check_format_frame( cpl_frame* rawframe );
sph_master_frame* sph_zpl_utils_calculate_master_frame_mean_weight_from_quad_image(
        sph_quad_image* qi);

sph_double_image*
sph_zpl_utils_calculate_stokes_param_double_image_from_quad_image(
        sph_quad_image* qi);

sph_error_code
sph_zpl_utils_apply_polarization_flat(sph_double_image* doubleimage,
        sph_double_image* polarization_flat);

sph_error_code
sph_zpl_utils_apply_modem_efficiency(sph_double_image* doubleimage,
        sph_master_frame* modem);

sph_master_frame*
sph_zpl_utils_calculate_polarization_efficiency(sph_double_image* di_plus,
        sph_double_image* di_minus);

sph_master_frame*
sph_zpl_utils_calculate_polarization_offset(sph_double_image* di_plus,
        sph_double_image* di_minus);

sph_error_code
sph_zpl_utils_get_frame_center(cpl_frameset* inframes, double* cx0, double* cy0);

//cpl_frame* sph_zpl_utils_remove_planes_zpl_exposure( cpl_frame* inframe, cpl_vector* iplanes);

cpl_frameset* sph_zpl_utils_preproc(cpl_frameset* inframes);

cpl_frameset* sph_zpl_utils_preproc_imaging(cpl_frameset* inframes);

cpl_frameset* sph_zpl_utils_subtract_overscans(cpl_frameset* inframes);

cpl_propertylist* sph_zpl_utils_get_camera_header(cpl_frameset* preproc_frames,
        cpl_frameset* rawframes);

cpl_frame* sph_zpl_utils_subtract_overscans_zpl_exposure(cpl_frameset* inframes,
        const char* filename);

cpl_frame* sph_zpl_utils_subtract_overscans_zpl_exposure_imaging(
        cpl_frameset* inframes, const char* filename);

#endif /* SPH_ZPL_UTILS_H_ */
