/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <cpl.h>
#include "sph_error.h"
#ifndef SPH_FITTING_H_
#define SPH_FITTING_H_
#include <gsl_multifit_nlin.h>
#include <gsl/gsl_roots.h>


typedef struct __sph_fitting_fit_robust_params {
    int     n;
    double* xdata;
    double* ydata;
    double  median;
} _sph_fitting_fit_robust_params;

typedef struct __sph_fitting_fit_robust_ws__ {
    gsl_root_fsolver *s;
    gsl_function F;
    _sph_fitting_fit_robust_params params;
    double errest;


} _sph_fitting_fit_robust_ws_;

int sph_fitting_lm_dero( size_t n, double* r, double* y, double* sigma_y,
		             double* init, cpl_vector* fit, cpl_vector* fiterr );
sph_error_code
sph_fitting_fit_gauss2D(
        cpl_image* im,
        cpl_image* im_err,
        double* xpos, double* ypos,
        double *xwidth, double * ywidth,
        double boxfactor,
        double boxborder);

cpl_polynomial*
sph_fitting_fit_poly1d( cpl_vector* xvals,
        cpl_vector* yvals,
        cpl_vector* err,
        int mind,
        int maxdeg,
        int allowshift, double guess, double* chisq);

cpl_polynomial*
sph_fitting_fit_poly2d( cpl_vector* xvals, cpl_vector* yvals,
                           cpl_vector* zvals, double* chisq, int maxdeg1, int maxdeg2);
cpl_image*
sph_fitting_fit_robust_imlist(
        cpl_imagelist* imlist,
        cpl_vector* xvals,
        double x_lo,
        double x_hi,
        cpl_image** offset,
        cpl_image* errimage);


double
sph_fitting_fit_robust(
        cpl_vector* xvals,
        cpl_vector* yvals,
        double x_lo,
        double x_i,
        double* offset);

cpl_vector*
sph_fitting_estimate_error( cpl_polynomial* poly,
        cpl_vector* xvals,
        cpl_vector* yvals,
        cpl_vector* err,
        cpl_vector** means_o);
#endif /* SPH_FITTING_H_ */
