/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <cpl.h>
#include "sph_error.h"
#include "sph_point_pattern.h"
#include "sph_polygon.h"
#include "sph_distortion_map.h"
#include "sph_master_frame.h"
#ifndef SPH_POLYNOMIAL_DISTORTION_MODEL_H_
#define SPH_POLYNOMIAL_DISTORTION_MODEL_H_

typedef struct _sph_polynomial_distortion_model_ {
	cpl_polynomial* polyx;
	cpl_polynomial* polyy;
	double          offx;
	double          offy;
} sph_polynomial_distortion_model;

sph_error_code
sph_polynomial_distortion_model_set_coeff_x(
        sph_polynomial_distortion_model* self,
        int order_x,
        int order_y,
        double value);
sph_point_pattern*
sph_polynomial_distortion_model_apply_pp(
        sph_polynomial_distortion_model* self,
        sph_point_pattern* pp );
sph_error_code
sph_polynomial_distortion_model_set_coeff_y(
        sph_polynomial_distortion_model* self,
        int order_x,
        int order_y,
        double value);
sph_polynomial_distortion_model*
sph_polynomial_distortion_model_new( cpl_polynomial* polyx, cpl_polynomial* polyy );
sph_polynomial_distortion_model*
sph_polynomial_distortion_model_new_from_proplist( const cpl_propertylist* proplist );
sph_polynomial_distortion_model*
sph_polynomial_distortion_model_load( const char* filename, int ext );
sph_polynomial_distortion_model*
sph_polynomial_distortion_model_from_grid_image(
        sph_master_frame* mframe,
        sph_point_pattern* pp,
        int polyfit_order,
        double threshold_sigma,
        double threshold_length,
        cpl_propertylist* qclist,
        double * centx,
        double * centy,
        sph_point_pattern** obs_pp);
sph_error_code
sph_polynomial_distortion_set_offset(
        sph_polynomial_distortion_model* self,
        double offx,
        double offy);

sph_error_code sph_polynomial_distortion_model_save(
        sph_polynomial_distortion_model* self,
        const char* filename,
        int nx,
        int ny, unsigned mode,
        cpl_propertylist* pl);

cpl_polynomial*
sph_polynomial_distortion_model_get_poly_x( sph_polynomial_distortion_model* self );

cpl_polynomial*
sph_polynomial_distortion_model_get_poly_y( sph_polynomial_distortion_model* self );

cpl_propertylist*
sph_polynomial_distortion_model_get_proplist( sph_polynomial_distortion_model* self );

sph_point_pattern*
sph_polynomial_distortion_model_create_point_pattern( sph_polynomial_distortion_model* self,
															int nx, int ny, int np) ;


cpl_image*
sph_polynomial_distortion_model_create_image_x( sph_polynomial_distortion_model* self,
		int nx, int ny);

cpl_image*
sph_polynomial_distortion_model_create_image_y( sph_polynomial_distortion_model* self,
		int nx, int ny);

sph_distortion_map*
sph_polynomial_distortion_model_fit_distortion_map(
		sph_polynomial_distortion_model* self,
		sph_distortion_map* dmap, int maxdegree, double* chiX, double* chiY );
void
sph_polynomial_distortion_model_delete(sph_polynomial_distortion_model*);
sph_error_code
sph_polynomial_distortion_model_save_dfs(
        sph_polynomial_distortion_model* self,
        const char* outfilename,
        cpl_frameset* allframes,
        cpl_frame* template_frame,
        cpl_parameterlist* params,
        const char* tag,
        const char* recipe,
        const char* pipename,
        cpl_propertylist* qclist,
        int nx,
        int ny,
        double pixscale,
        int changescale);
double
sph_polynomial_distortion_model_get_dist_x( sph_polynomial_distortion_model* self,
        double xpos, double ypos);
double
sph_polynomial_distortion_model_get_dist_y( sph_polynomial_distortion_model* self,
        double xpos, double ypos);
cpl_image*
sph_polynomial_distortion_model_apply(
        sph_polynomial_distortion_model* self,
        cpl_image* image,
        cpl_kernel kerneltype,
        double radin );
sph_error_code
sph_polynomial_distortion_model_apply_poly(
        sph_polynomial_distortion_model* self,
        sph_polygon* poly );


sph_error_code
sph_polynomial_distortion_model_scale_units(
        sph_polynomial_distortion_model* self,
        double scale);

#endif /* SPH_POLYNOMIAL_DISTORTION_MODEL_H_ */
