/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifndef SPH_STREHL_H_
#define SPH_STREHL_H_

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

/*-----------------------------------------------------------------------------
                                   Define
 -----------------------------------------------------------------------------*/

#ifndef SPH_STREHL_PSCALE
#define SPH_STREHL_PSCALE "ESO INS PIXSCALE"
#endif


#ifndef SPH_STREHL_PSCALE_IRDIS
#define SPH_STREHL_PSCALE_IRDIS 0.01225
#endif

#ifndef SPH_STREHL_PSCALE_ZIMPOL
#define SPH_STREHL_PSCALE_ZIMPOL 0.0035
#endif

#ifndef SPH_STREHL_FILTER_KEY_IRD1
/* The broad band filter of IRDIS */
#define SPH_STREHL_FILTER_KEY_IRD1 "INS1 FILT"
#endif

#ifndef SPH_STREHL_FILTER_KEY_IRD2
/* The narrow band filter of IRDIS, may be open */
#define SPH_STREHL_FILTER_KEY_IRD2 "INS1 OPTI2"
#endif

#ifndef SPH_STREHL_FILTER_KEY_ZPL0
/* ZIMPOL common filter */
#define SPH_STREHL_FILTER_KEY_ZPL0 "INS3 OPTI2"
#endif

#ifndef SPH_STREHL_FILTER_KEY_ZPL1
/* ZIMPOL filter in arm 1 */
#define SPH_STREHL_FILTER_KEY_ZPL1 "INS3 OPTI5"
#endif

#ifndef SPH_STREHL_FILTER_KEY_ZPL2
/* ZIMPOL filter in arm 2 */
#define SPH_STREHL_FILTER_KEY_ZPL2 "INS3 OPTI6"
#endif

#define SPH_STREHL_FILTER_IRD1 "ESO " SPH_STREHL_FILTER_KEY_IRD1 " NAME"
#define SPH_STREHL_FILTER_IRD2 "ESO " SPH_STREHL_FILTER_KEY_IRD2 " NAME"

#define SPH_STREHL_FILTER_ZPL0 "ESO " SPH_STREHL_FILTER_KEY_ZPL0 " NAME"
#define SPH_STREHL_FILTER_ZPL1 "ESO " SPH_STREHL_FILTER_KEY_ZPL1 " NAME"
#define SPH_STREHL_FILTER_ZPL2 "ESO " SPH_STREHL_FILTER_KEY_ZPL2 " NAME"


/*-----------------------------------------------------------------------------
                                   Function declarations
 -----------------------------------------------------------------------------*/

cpl_error_code sph_strehl_fill_parlist(cpl_parameterlist *,
                                       const char        *);

cpl_error_code sph_strehl_irdis(cpl_propertylist *,
                                const cpl_image  *,
                                const cpl_image  *,
                                const cpl_frame  *,
                                const cpl_frame  *,
                                const char       *,
                                const cpl_parameterlist *);

cpl_error_code sph_strehl_zimpol(cpl_propertylist *,
                                 const cpl_image  *,
                                 const cpl_image  *,
                                 const cpl_frame  *,
                                 const cpl_frame  *,
                                 const char       *,
                                 const cpl_parameterlist *);

#endif /* SPH_STREHL_H_ */
