/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#include <cpl.h>

#include "sph_utils.h"
#include "sph_fitting.h"
#include "sph_unsharp_mask.h"

cpl_image*
sph_unsharp_mask( cpl_image* inimage, unsigned int winsize ) {
	cpl_image* temp = NULL;
	cpl_image* result = NULL;
	cpl_size xx = 0;
	cpl_size yy = 0;
	cpl_size llx = 0;
	cpl_size lly = 0;
	cpl_size urx = 0;
	cpl_size ury = 0;
	cpl_size nx = 0;
	cpl_size ny = 0;


	cpl_ensure(inimage,CPL_ERROR_NULL_INPUT,NULL);
	cpl_ensure(cpl_error_get_code() == CPL_ERROR_NONE,cpl_error_get_code(),NULL);

	nx = cpl_image_get_size_x(inimage);
	ny = cpl_image_get_size_y(inimage);
	temp = cpl_image_duplicate(inimage);
	for ( xx = 0; xx < nx ; ++xx) {
		for ( yy = 0; yy < ny; ++yy) {
			llx = xx - winsize < 0 ? 0 : xx - winsize;
			lly = yy - winsize < 0 ? 0 : yy - winsize;
			urx = xx + winsize >= nx ? nx - 1 : xx + winsize;
			ury = yy + winsize >= ny ? ny - 1 : yy + winsize;

			double median = cpl_image_get_median_window(inimage,llx + 1,lly + 1,urx + 1,ury + 1);
			if ( cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT || cpl_error_get_code() == CPL_ERROR_DATA_NOT_FOUND ) {
				cpl_error_reset();
				cpl_image_reject(temp,xx + 1, yy + 1);
			}
			else {
				cpl_image_set(temp,xx + 1, yy + 1,median);
			}
		}
	}
	result = cpl_image_subtract_create( inimage, temp );
	cpl_image_delete(temp); temp = NULL;
	return result;
}
