/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifndef SPH_TEST_ngc_simulator_H_
#define SPH_TEST_ngc_simulator_H_

#include <cpl.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

enum _sph_test_rom_ {
    SPH_TEST_NGC_ROM_SINGLE = 0, SPH_TEST_NGC_ROM_DC = 1
};

typedef enum _sph_test_rom_ sph_test_rom;

typedef enum _sph_test_ngc_ir_simulator_modes_ {
    SPH_TEST_NGC_SIMULATOR_MODES_CONSTVALUE = 0,
} sph_test_ngc_ir_simulator_modes;

typedef struct _sph_test_ngc_ir_simulator_ {
    short poisson; //flag to set if poisson randomizing
    short add_ron_noise; // flag to set if add ron
    gsl_rng* pRNG;
    int det_size_x; // entire detector size in x
    int det_size_y; // entire detector size in y
    int win_minx; // read-out window
    int win_miny; // this window is the window read-out
    int win_maxx; // by controller ( usually a 2048X1024
    int win_maxy; // region )

    int timedelay;

    double mean_bias; // mean value for bias
    double sigma_bias;
    double mean_dark;
    double sigma_dark;
    double exptime; // the current exptime setting
    sph_test_rom rom; // the readout mode;
    double ron; // the read out noise
    int ndead; // number of dead single pixels
    int nhot; // number of hot single pixels
    int ncrazy; // number of crazy single pixels
    double det_linear_factor; // the linear response factor
                              // (outsignal = factor * insignal )
    double det_qaudratic_factor; // the quadratic response factor
                                 // (outsignal = factor * insignal^2 )
    double lambda_ramp_linear; // the increase in out/in per wavelength
                               // with wavelength measured in mu
    double lambda_ramp_quadratic; // the increase in out/in per wavelength
                                  // with wavelength measured in mu
    short singlecube; // controls if frames are read into sinlge cube
    short dont_reset; // if set to yes, do not reset pixels
                      // between exposures
    double waveflat_fwhm;
    double waveflat_norm;
    double waveflat_meanl;
    double waveflat_factor;
    cpl_vector* ditherpattern_x;
    cpl_vector* ditherpattern_y;
    int current_dither_index;
    double current_dx;
    double current_dy;
    cpl_image* flatim; // the detector flat
} sph_test_ngc_ir_simulator;

sph_test_ngc_ir_simulator*
sph_test_ngc_ir_simulator_new(int nx, int ny);
cpl_frameset*
sph_test_ngc_ir_simulator_create_raw_frameset(sph_test_ngc_ir_simulator* self,
        cpl_frame* inframe, int nframes, int ndit, const char* base_filename,
        cpl_propertylist* pl, const char* tag);
//cpl_frame*
//sph_test_ngc_ir_simulator_create_master_dark( sph_test_ngc_ir_simulator* self,
//		cpl_frameset* raw_frames,
//		cpl_parameterlist* inparams,
//		const char* base_filename );
cpl_image*
sph_test_ngc_ir_simulator_create_detector_flat_image(
        sph_test_ngc_ir_simulator* self);

cpl_image*
sph_test_ngc_ir_simulator_create_dark_image(sph_test_ngc_ir_simulator* self);

void sph_test_ngc_ir_simulator_set_standard_ditherpattern(
        sph_test_ngc_ir_simulator* self, int ndithers);

cpl_image*
sph_test_ngc_ir_simulator_create_bias_image(sph_test_ngc_ir_simulator* self);
cpl_frameset*
sph_test_ngc_ir_simulator_create_raw_frameset_many(
        sph_test_ngc_ir_simulator* self, cpl_frameset* inframes, int ndit,
        const char* base_filename, cpl_propertylist* pl, const char* tag);

void sph_test_ngc_ir_simulator_set_detector_size(
        sph_test_ngc_ir_simulator* self, int nx, int ny);
void sph_test_ngc_ir_simulator_set_readout_window(
        sph_test_ngc_ir_simulator* self, int left, int bottom, int right,
        int top);
void sph_test_ngc_ir_simulator_set_bias(sph_test_ngc_ir_simulator* self,
        double mean, double sigma);
void sph_test_ngc_ir_simulator_set_dark(sph_test_ngc_ir_simulator* self,
        double mean, double sigma);
void sph_test_ngc_ir_simulator_set_exptime(sph_test_ngc_ir_simulator* self,
        double exptime);
void sph_test_ngc_ir_simulator_set_readout_mode(sph_test_ngc_ir_simulator* self,
        sph_test_rom mode);
void sph_test_ngc_ir_simulator_set_readout_noise(
        sph_test_ngc_ir_simulator* self, double ron);
void sph_test_ngc_ir_simulator_set_num_deadpixels(
        sph_test_ngc_ir_simulator* self, int n);
void sph_test_ngc_ir_simulator_set_num_hotpixels(
        sph_test_ngc_ir_simulator* self, int n);
void sph_test_ngc_ir_simulator_set_num_crazypixels(
        sph_test_ngc_ir_simulator* self, int n);
void sph_test_ngc_ir_simulator_set_flat_linear_coeff(
        sph_test_ngc_ir_simulator* self, double c);
void sph_test_ngc_ir_simulator_set_flat_quadratic_coeff(
        sph_test_ngc_ir_simulator* self, double c);
void sph_test_ngc_ir_simulator_set_dont_reset(sph_test_ngc_ir_simulator* self,
        int yesorno);
void sph_test_ngc_ir_simulator_set_save_singlecube(
        sph_test_ngc_ir_simulator* self, int yesorno);
void sph_test_ngc_ir_simulator_set_add_noise(sph_test_ngc_ir_simulator* self,
        int addnoise, int addpoisson);
void sph_test_ngc_ir_simulator_set_dff_gradient(sph_test_ngc_ir_simulator* self,
        double min, double max);
gsl_rng*
sph_test_ngc_ir_simulator_get_prng(sph_test_ngc_ir_simulator* self);
cpl_image*
sph_test_ngc_ir_simulator_create_raw_image(sph_test_ngc_ir_simulator* self,
        cpl_image* inimage, double dx, double dy);
cpl_image*
sph_test_ngc_ir_simulator_create_detector_flat_image_wdep(
        sph_test_ngc_ir_simulator* self, cpl_image* wimage);
double
sph_test_ngc_ir_simulator_get_response(sph_test_ngc_ir_simulator* self,
        double x, double y, double lambda);
void
sph_test_ngc_ir_simulator_set_flat(sph_test_ngc_ir_simulator* self,
        const cpl_image* flat);
void
sph_test_ngc_ir_simulator_unset_flat(sph_test_ngc_ir_simulator* self);
void
sph_test_ngc_ir_simulator_delete(sph_test_ngc_ir_simulator* self);
cpl_image*
sph_test_ngc_ir_simulator_create_static_badpixel_image(
        sph_test_ngc_ir_simulator* self);
cpl_frame*
sph_test_ngc_ir_simulator_raw_cube_from_frame(sph_test_ngc_ir_simulator* self,
        cpl_frame* inframe, int nplanes, const char* filename, int ff,
        cpl_propertylist* addprops, const char* tag);

#ifdef SPH_TEST_NGC_SIMULATOR_C_
static
cpl_frame*
__sph_test_ngc_ir_simulator_create_raw_cube_frame(
        const char* namein, const char* tag, int ii);
static
void
__sph_test_ngc_ir_simulator_save_im_in_fitscube(
        cpl_image* im,
        cpl_frame* frame,
        cpl_propertylist* pl,
        int pp);
static
int
__sph_test_ngc_ir_simulator_advance_dithering(
        sph_test_ngc_ir_simulator* self );
static
void
__sph_test_ngc_ir_simulator_create_raw_cube_filename(
        const char* namein, int ii, char* out);
static
void
__sph_test_ngc_ir_simulator_digest_image(
        sph_test_ngc_ir_simulator* self,
        cpl_image* animage );

static
void
__sph_test_ngc_ir_simulator_image_add_bias_and_dark( sph_test_ngc_ir_simulator* self, cpl_image* animage, double exptime );

static
void
__sph_test_ngc_ir_simulator_image_apply_flat( sph_test_ngc_ir_simulator* self, cpl_image* animage );

static
void
__sph_test_ngc_ir_simulator_image_add_noise( sph_test_ngc_ir_simulator* self, cpl_image* animage );

#endif
#endif /* SPH_TEST_ngc_simulator_H_ */
