/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <gsl_rng.h>
#include <gsl_randist.h>
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_ngc_ccd_simulator.h"
#include "sph_master_frame.h"
#include "sph_ird_tags.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_utils.h"
#include "sph_test_image_tools.h"

int cutest_init_test_ngc_simulator_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__, PACKAGE_BUGREPORT, CPL_MSG_DEBUG);

    return 0;
}

int cutest_clean_test_ngc_simulator_testsuite(void) {
    sph_error_dump(SPH_ERROR_ERROR);
    return sph_end_test();
}

/*----------------------------------------------------------------------------*/
/**
 */
/*----------------------------------------------------------------------------*/
void cutest_test_ngc_simulator_ccd_new(void) {
    sph_test_ngc_ccd_simulator* fc = NULL;

    fc = sph_test_ngc_ccd_simulator_new();
    CU_ASSERT_PTR_NOT_NULL( fc);

    sph_test_ngc_ccd_simulator_delete(fc);
    fc = NULL;

    return;
}
/*----------------------------------------------------------------------------*/
/**
 */
/*----------------------------------------------------------------------------*/
void cutest_test_ngc_simulator_ccd_create_raw_frameset(void) {
    sph_test_ngc_ccd_simulator* fc = NULL;
    cpl_frameset* frames = NULL;
    cpl_frame* inframe = NULL;
    cpl_frameset* inframeset = NULL;
    cpl_image* image = sph_test_image_tools_create_flat_image_double(256, 256,
            1.0);

    CU_ASSERT_PTR_NOT_NULL_FATAL( image);
    inframe = sph_test_frame_image(image, CPL_TYPE_DOUBLE,
            SPH_TEST_PUPILIMAGE_TAG, CPL_FRAME_GROUP_RAW);
    inframeset = cpl_frameset_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( inframe);
    CU_ASSERT_PTR_NOT_NULL_FATAL( inframeset);
    cpl_frameset_insert(inframeset, inframe);
    fc = sph_test_ngc_ccd_simulator_new();
    CU_ASSERT_PTR_NOT_NULL( fc);
    frames = sph_test_ngc_ccd_simulator_create_raw_frameset_many(fc, inframeset,
            1.0, "zpl_raw_darks", NULL, SPH_IRD_TAG_DARK_RAW);
    CU_ASSERT_PTR_NOT_NULL( frames);
    CU_ASSERT_EQUAL( cpl_frameset_get_size( frames ), 1);
    sph_test_ngc_ccd_simulator_delete(fc);
    fc = NULL;
    sph_test_cleanup_testfiles();
    cpl_frameset_delete(inframeset);
    cpl_frameset_delete(frames);
    cpl_image_delete(image);
    return;
}
/*----------------------------------------------------------------------------*/
/**
 */
/*----------------------------------------------------------------------------*/
void cutest_test_ngc_simulator_ccd_create_raw_frameset_poisson(void) {
}
/*----------------------------------------------------------------------------*/
/**
 */
/*----------------------------------------------------------------------------*/
void cutest_test_ngc_simulator_ccd_create_raw_frameset_single_cube(void) {
}

/*----------------------------------------------------------------------------*/
/**
 */
/*----------------------------------------------------------------------------*/
void cutest_test_ngc_simulator_ccd_delete(void) {
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("sph_test_famecreator_irdis",
            cutest_init_test_ngc_simulator_testsuite,
            cutest_clean_test_ngc_simulator_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    if (NULL
            == CU_add_test(pSuite, "sph_test_ngc_ccd_simulator_new",
                    cutest_test_ngc_simulator_ccd_new)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "sph_test_ngc_ccd_simulator_create_raw_frameset",
                    cutest_test_ngc_simulator_ccd_create_raw_frameset)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "sph_test_ngc_ccd_simulator_create_raw_frameset_poisson",
                    cutest_test_ngc_simulator_ccd_create_raw_frameset_poisson)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
