/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_test_ird_fixtures.h"
#include "sph_test_image_tools.h"
#include "sph_ird_tags.h"
#include "sph_ird_instrument_flat.h"
#include "sph_ird_instrument_flat.c"
#include "sph_ird_instrument_flat_run.c"
#include "sph_fits.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup A CUnit Test Suite -- representing a collection of testcases
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/
static cpl_frameset* cutest_sph_ird_instrument_flat_fixture_ideal_raws(
        int nraws, int nplanes, int nx, int ny) {
    cpl_frameset* result = NULL;
    int pp = 0;
    int ff = 0;
    cpl_image* im = NULL;
    double val = 0.0;
    cpl_frame* aframe = NULL;
    cpl_propertylist* pl = NULL;
    result = cpl_frameset_new();
    pl = cpl_propertylist_new();
    for (ff = 0; ff < nraws; ++ff) {
        val = ff * 1000.0 + 500.0;
        cpl_propertylist_update_double(pl, SPH_COMMON_KEYWORD_DIT,
                ff * 1.2 + 1.2);
        aframe = sph_filemanager_create_temp_frame("IRD_FLAT_TEST",
                SPH_IRD_TAG_FLAT_RAW, CPL_FRAME_GROUP_RAW);
        for (pp = 0; pp < nplanes; ++pp) {
            im = sph_test_image_tools_create_flat_image_double(nx, ny, val);
            CU_ASSERT_PTR_NOT_NULL_FATAL(im);
            sph_test_image_tools_apply_poisson_noise(im, NULL);
            if (pp == 0) {
                CU_ASSERT_EQUAL_FATAL(
                        sph_test_image_tools_save_image_as_new_cube(im, cpl_frame_get_filename(aframe), CPL_TYPE_DOUBLE,pl,CPL_IO_CREATE),
                        CPL_ERROR_NONE);
            } else {
                CU_ASSERT_EQUAL_FATAL(
                        sph_fits_append_image( cpl_frame_get_filename(aframe),im,0),
                        CPL_ERROR_NONE);
            }
            cpl_image_delete(im);
            im = NULL;
        }CU_ASSERT_EQUAL_FATAL(cpl_error_get_code(), CPL_ERROR_NONE);
        cpl_frameset_insert(result, aframe);
    }

    cpl_propertylist_delete(pl);
    return result;
}

static cpl_frameset* cutest_sph_ird_instrument_flat_fixture_gradient_raws(
        int nraws, int nplanes, int nx, int ny) {
    cpl_frameset* result = NULL;
    int pp = 0;
    int ff = 0;
    cpl_image* im = NULL;
    double val = 0.0;
    cpl_frame* aframe = NULL;
    cpl_propertylist* pl = NULL;
    result = cpl_frameset_new();
    pl = cpl_propertylist_new();
    for (ff = 0; ff < nraws; ++ff) {
        val = ff * 1000.0 + 500.0;
        cpl_propertylist_update_double(pl, SPH_COMMON_KEYWORD_DIT,
                ff * 1.2 + 1.2);
        aframe = sph_filemanager_create_temp_frame("IRD_FLAT_TEST",
                SPH_IRD_TAG_FLAT_RAW, CPL_FRAME_GROUP_RAW);
        for (pp = 0; pp < nplanes; ++pp) {
            im = sph_test_image_tools_create_flat_image_double(nx, ny, 0.0);
            sph_test_image_tools_add_gradient_x(im, 0.5 * val, 1.5 * val);
            CU_ASSERT_PTR_NOT_NULL_FATAL(im);
            sph_test_image_tools_apply_poisson_noise(im, NULL);
            if (pp == 0) {
                CU_ASSERT_EQUAL_FATAL(
                        sph_test_image_tools_save_image_as_new_cube(im, cpl_frame_get_filename(aframe), CPL_TYPE_DOUBLE,pl,CPL_IO_CREATE),
                        CPL_ERROR_NONE);
            } else {
                CU_ASSERT_EQUAL_FATAL(
                        sph_fits_append_image( cpl_frame_get_filename(aframe),im,0),
                        CPL_ERROR_NONE);
            }
            cpl_image_delete(im);
            im = NULL;
        }CU_ASSERT_EQUAL_FATAL(cpl_error_get_code(), CPL_ERROR_NONE);
        cpl_frameset_insert(result, aframe);
    }

    cpl_propertylist_delete(pl);
    return result;
}

static
void cutest_sph_ird_instrument_flat_fixture_const_dark(cpl_frameset* inframes,
        int nx, int ny, double val, double dit) {
    cpl_image* im = NULL;
    cpl_frame* frame = NULL;
    cpl_propertylist* pl = NULL;

    pl = cpl_propertylist_new();
    im = sph_test_image_tools_create_flat_image_double(nx, ny, val);
    CU_ASSERT_PTR_NOT_NULL_FATAL(im);
    sph_test_image_tools_apply_poisson_noise(im, NULL);
    cpl_frameset_insert(
            inframes,
            frame = sph_filemanager_create_temp_frame("test_dark.fits",
                    SPH_IRD_TAG_DARK_RAW, CPL_FRAME_GROUP_RAW));
    cpl_propertylist_update_double(pl, SPH_COMMON_KEYWORD_DIT, dit);
    SPH_RAISE_CPL;
    cpl_image_save(im, cpl_frame_get_filename(frame), CPL_BPP_32_SIGNED, pl,
            CPL_IO_CREATE);
    SPH_RAISE_CPL;CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    cpl_propertylist_delete(pl);
    cpl_image_delete(im);
    im = NULL;
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

static sph_master_frame*
__cutest_sph_ird_instrument_flat_run(cpl_frameset* inframes) {
    sph_ird_instrument_flat* insflat = NULL;
    cpl_parameterlist* params = NULL;
    sph_master_frame* flat = NULL;
    params = sph_ird_instrument_flat_create_paramlist();

    insflat = sph_ird_instrument_flat_new(inframes, params);
    CU_ASSERT_PTR_NOT_NULL_FATAL(insflat);

    CU_ASSERT_EQUAL( sph_ird_instrument_flat_run(insflat), CPL_ERROR_NONE);

    flat = sph_master_frame_load(insflat->outfilename, 0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(flat);

    sph_ird_instrument_flat_delete(insflat);
    insflat = NULL;
    cpl_parameterlist_delete(params);
    params = NULL;
    return flat;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_init_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__, PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_clean_testsuite(void) {
    sph_error_dump(SPH_ERROR_ERROR);
    sph_end_test();
    return 0;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_ird_instrument_flat_ideal_data(void) {
    cpl_frameset* inframes = NULL;
    int nx = 512;
    int ny = 256;
    int nfiles = 3;
    int nplanes = 3;
    sph_master_frame* flat = NULL;
    cpl_propertylist* pl = NULL;

    inframes = cutest_sph_ird_instrument_flat_fixture_ideal_raws(nfiles,
            nplanes, nx, ny);

    cpl_frameset_insert(inframes,
            sph_test_ird_fixtures_instrument_model_frame(nx, ny));

    cutest_sph_ird_instrument_flat_fixture_const_dark(inframes, nx, ny, 0.0,
            1.0);

    flat = __cutest_sph_ird_instrument_flat_run(inframes);
    CU_ASSERT_PTR_NOT_NULL_FATAL(flat);
    CU_ASSERT_EQUAL(sph_master_frame_get_nbads(flat), 0);
    CU_ASSERT_DOUBLE_EQUAL(sph_master_frame_get_mean(flat, NULL), 1.0, 0.00001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_max(flat->image), 1.0, 0.1);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_min(flat->image), 1.0, 0.1);
    sph_master_frame_delete(flat);
    flat = NULL;
    cpl_frameset_delete(inframes);
    inframes = NULL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_ird_instrument_flat_ideal_data_bg(void) {
    cpl_frameset* inframes = NULL;
    int nx = 512;
    int ny = 256;
    int nfiles = 3;
    int nplanes = 3;
    sph_master_frame* flat = NULL;
    cpl_propertylist* pl = NULL;

    inframes = cutest_sph_ird_instrument_flat_fixture_ideal_raws(nfiles,
            nplanes, nx, ny);

    cpl_frameset_insert(inframes,
            sph_test_ird_fixtures_instrument_model_frame(nx, ny));

    cutest_sph_ird_instrument_flat_fixture_const_dark(inframes, nx, ny, 0.0,
            1.0);
    cutest_sph_ird_instrument_flat_fixture_const_dark(inframes, nx, ny, 0.0,
            2.0);
    cutest_sph_ird_instrument_flat_fixture_const_dark(inframes, nx, ny, 0.0,
            3.7);

    flat = __cutest_sph_ird_instrument_flat_run(inframes);
    CU_ASSERT_PTR_NOT_NULL_FATAL(flat);
    CU_ASSERT_EQUAL(sph_master_frame_get_nbads(flat), 0);
    CU_ASSERT_DOUBLE_EQUAL(sph_master_frame_get_mean(flat, NULL), 1.0, 0.00001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_max(flat->image), 1.0, 0.1);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_min(flat->image), 1.0, 0.1);

    pl = cpl_propertylist_load("irdis_flat.fits", 0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pl);

    CU_ASSERT_TRUE( cpl_propertylist_has( pl, SPH_COMMON_KEYWORD_FLAT_RMS ));
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_FLAT_RMS ),
            0.016, 0.002);
    CU_ASSERT_TRUE( cpl_propertylist_has( pl, SPH_COMMON_KEYWORD_FLAT_FPN ));
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_FLAT_FPN ),
            cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_FLAT_RMS ),
            0.01);
    CU_ASSERT_TRUE( cpl_propertylist_has( pl, SPH_IRD_KEYWORD_FLAT_FPN_LEFT ));
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl, SPH_IRD_KEYWORD_FLAT_FPN_LEFT ),
            cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_FLAT_RMS ),
            0.01);
    CU_ASSERT_TRUE( cpl_propertylist_has( pl, SPH_IRD_KEYWORD_FLAT_FPN_RIGHT ));
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl, SPH_IRD_KEYWORD_FLAT_FPN_RIGHT ),
            cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_FLAT_RMS ),
            0.01);

    cpl_propertylist_delete(pl);
    pl = NULL;
    sph_master_frame_delete(flat);
    flat = NULL;
    cpl_frameset_delete(inframes);
    inframes = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_ird_instrument_flat_ideal_data_gradient(void) {
    cpl_frameset* inframes = NULL;
    int nx = 512;
    int ny = 256;
    int nfiles = 3;
    int nplanes = 3;
    sph_master_frame* flat = NULL;
    cpl_propertylist* pl = NULL;

    inframes = cutest_sph_ird_instrument_flat_fixture_gradient_raws(nfiles,
            nplanes, nx, ny);

    cpl_frameset_insert(inframes,
            sph_test_ird_fixtures_instrument_model_frame(nx, ny));

    cutest_sph_ird_instrument_flat_fixture_const_dark(inframes, nx, ny, 0.0,
            1.0);
    cutest_sph_ird_instrument_flat_fixture_const_dark(inframes, nx, ny, 0.0,
            2.0);
    cutest_sph_ird_instrument_flat_fixture_const_dark(inframes, nx, ny, 0.0,
            3.7);

    flat = __cutest_sph_ird_instrument_flat_run(inframes);
    CU_ASSERT_PTR_NOT_NULL_FATAL(flat);
    CU_ASSERT_EQUAL(sph_master_frame_get_nbads(flat), 0);
    CU_ASSERT_DOUBLE_EQUAL(sph_master_frame_get_mean(flat, NULL), 1.0, 0.00001);

    pl = cpl_propertylist_load("irdis_flat.fits", 0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pl);

    CU_ASSERT_TRUE( cpl_propertylist_has( pl, SPH_COMMON_KEYWORD_FLAT_RMS ));
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_FLAT_RMS ), 0.1,
            0.1);
    CU_ASSERT_TRUE( cpl_propertylist_has( pl, SPH_COMMON_KEYWORD_FLAT_FPN ));
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl, SPH_COMMON_KEYWORD_FLAT_FPN ),
            0.016, 0.01);
    CU_ASSERT_TRUE( cpl_propertylist_has( pl, SPH_IRD_KEYWORD_FLAT_FPN_LEFT ));
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl, SPH_IRD_KEYWORD_FLAT_FPN_LEFT ),
            0.016, 0.01);
    CU_ASSERT_TRUE( cpl_propertylist_has( pl, SPH_IRD_KEYWORD_FLAT_FPN_RIGHT ));
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl, SPH_IRD_KEYWORD_FLAT_FPN_RIGHT ),
            0.016, 0.01);
    cpl_propertylist_delete(pl);
    pl = NULL;
    sph_master_frame_delete(flat);
    flat = NULL;
    cpl_frameset_delete(inframes);
    inframes = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("Testing sph_ird_instrument_flat_run",
            cutest_init_testsuite, cutest_clean_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    if (NULL
            == CU_add_test(pSuite, "Testing simple standard case w. ideal data",
                    cutest_sph_ird_instrument_flat_ideal_data)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "Testing simple standard case w. ideal data multiple bg",
                    cutest_sph_ird_instrument_flat_ideal_data_bg)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "Testing simple standard case w. ideal data gradient",
                    cutest_sph_ird_instrument_flat_ideal_data_gradient)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
