/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "cutest_zpl_preproc.h"
#include "sph_zpl_preproc.h"
#include "sph_zpl_utils.h"
#include "sph_utils.h"
#include "sph_zpl_keywords.h"
#include "sph_zpl_tags.h"
#include "sph_common_keywords.h"
#include "sph_keyword_manager.h"
#include "sph_filemanager.h"
#include "sph_test_zpl_simulator.h"
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_image_tools.h"
#include <Basic.h>
#include <CUnit.h>

/*----------------------------------------------------------------------------*/
/**
 * @defgroup Unit test of the polarimetric pre-processing
 * 			 recipe and associated functions.
 *
 */
/*----------------------------------------------------------------------------*/

static int cutest_init_preproc_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     *--------------------------------------------------------------------*/
    sph_init_debug_verbose();
    //sph_init_test();
    return 0;
}

static int cutest_clean_preproc_testsuite(void) {
    sph_error_dump(SPH_ERROR_ERROR);
    cpl_end();
    //sph_end_test();
    return 0;
}

void cutest_zpl_preproc_run(void) {
    sph_zpl_preproc* zpl_preproc = NULL;
    cpl_parameterlist* inparams = NULL;
    cpl_frameset* inframes = NULL;
    cpl_frame* inframe = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;

    inframe = cpl_frame_new();
    if (inframe) {
        cpl_frame_set_tag(inframe, SPH_ZPL_TAG_PREPROC_RAW);
        cpl_frame_set_group(inframe, CPL_FRAME_GROUP_RAW);
        cpl_frame_set_filename(
                inframe,
                "/home/pavlov/prj/sphere/data/data-15.04.2011/110328/fastpol/DRHfpflatH000.fits");
        //cpl_frame_set_filename( inframe, "../../../../pySphere/example/ZIMPOL/test_double_ext.fits" );

        //cpl_frame_set_filename( frame, "ZIMPOL/data/test_double_ext.fits" );
        //cpl_frame_set_filename( frame, "pol250_10sec_nshut.fits" );
    }

    inframes = cpl_frameset_new();
    rerr = cpl_frameset_insert(inframes, inframe);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    inparams = sph_zpl_preproc_create_paramlist();
    zpl_preproc = sph_zpl_preproc_new(inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL( zpl_preproc);

    rerr = sph_zpl_preproc_run(zpl_preproc);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    rerr = sph_zpl_preproc_delete(zpl_preproc);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of  recipe polarimetric pre-processing
 * 		   recipe and associated functions.
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;

    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("sph_zpl_preproc_run", cutest_init_preproc_testsuite,
            cutest_clean_preproc_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */

    /*
     if ( NULL == CU_add_test(pSuite, "zpl_preproc_run",
     cutest_zpl_preproc_run ) )
     {
     CU_cleanup_registry();
     return CU_get_error();
     }
     */
    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
