/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "cutest_zpl_science_p1.h"
#include "sph_zpl_science_p1.h"
#include "sph_zpl_utils.h"
#include "sph_utils.h"
#include "sph_zpl_keywords.h"
#include "sph_zpl_tags.h"
#include "sph_common_keywords.h"
#include "sph_keyword_manager.h"
#include "sph_filemanager.h"
#include "sph_test_zpl_simulator.h"
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_image_tools.h"
#include "sph_fctable.h"
#include <Basic.h>
#include <CUnit.h>

/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/
static
void cutest_zpl_science_create_head_file_hwpz_plus_0(const char* infile) {
    FILE* fp;
    char* outfilename = NULL;

    outfilename = sph_filemanager_filename_new_ext(infile, "head");

    fp = fopen(outfilename, "w");
    if (fp != NULL) {
        fprintf(fp, "ESO DRS ZPL HWPZ SETTING = 0.0\n");
        fclose(fp);
    }
    cpl_free(outfilename);
    outfilename = NULL;

}

static
void cutest_zpl_science_create_head_file_hwpz_minus_45(const char* infile) {
    FILE* fp;
    char* outfilename = NULL;

    outfilename = sph_filemanager_filename_new_ext(infile, "head");
    fp = fopen(outfilename, "w");
    if (fp != NULL) {
        fprintf(fp, "ESO DRS ZPL HWPZ SETTING = 45.0\n");
        fclose(fp);
    }
    cpl_free(outfilename);
    outfilename = NULL;

}

static sph_keyword_manager* cutest_zpl_science_create_cfg_file(void) {
    FILE* fp;
    sph_keyword_manager* km = NULL;

    fp = fopen("sphere.cfg", "w");
    fprintf(
            fp,
            "SPH.KEYWORDS.ESO_DRS_ZPL_HWPZ_SETTING.NAME = \"ESO DRS ZPL HWPZ SETTING\"\n");
    fprintf(fp, "SPH.KEYWORDS.ESO_DRS_ZPL_HWPZ_SETTING.TYPE = \"DOUBLE\"\n");
    fprintf(fp, "SPH.KEYWORDS.ESO_DRS_ZPL_HWPZ_SETTING.VALUE = \"0.0\"\n");
    fclose(fp);

    km = sph_keyword_manager_new();
    sph_keyword_manager_read_config("sphere.cfg");

    return km;
}

static int cutest_init_science_p1_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     *--------------------------------------------------------------------*/
    sph_init_test(__FILE__, PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
    return CPL_ERROR_NONE;
}


static int cutest_clean_science_p1_testsuite(void) {
    sph_error_dump(SPH_ERROR_ERROR);
    return sph_end_test();
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_science_p1 with preproc frames but without keywords
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_science_p1_run_1(void) {
    sph_zpl_science_p1* zpl_science_p1 = NULL;
    cpl_parameterlist* inparams = NULL;
    cpl_frameset* inframes = NULL;
    cpl_frame* zexpcube = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    int nx = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int ny = 100; // -//-

    cpl_propertylist* pl = NULL;
    sph_fctable* fctable = NULL;
    cpl_frame* fcframe = NULL;

    cpl_frame* frame_fc = NULL;
    char* fname = NULL;
    char* fname2 = NULL;

    sph_zpl_exposure* zplexp_plane = NULL;

    int nframes = 5;
    //double dx[] = {1.0, 1.0, 1.0, 1.0, 1.0 };
    //double dy[] = {1.0, -1.0, 2.0, -2.0, 3.0 };
    double dx[] = { 0.0, 0.0, 0.0, 0.0, 0.0 };
    double dy[] = { 0.0, 0.0, 0.0, 0.0, 0.0 };
    double angle[] = { 0, 15, 30, 45, 60 };
    double mean[] = { 0.0, 0.0, 0.0, 0.0, 0.0 };

    inparams = sph_zpl_science_p1_create_paramlist();
    inframes = cpl_frameset_new();

    //simulate fctable
    fcframe = sph_test_simulator_create_fctable(nx, ny, dx, dy, angle, nframes);
    //rerr = cpl_frameset_insert( inframes, fcframe );

    //simulate cube of rotated pre-processed (ZPL EXP) science frames
    sph_test_simulator_create_zplpreproc_science_cube(&zexpcube, fcframe, nx,
            ny);
    cpl_frame_set_tag(zexpcube, SPH_ZPL_TAG_SCIENCE_P1_PREPROC_CAM1);
    cpl_frame_set_group(zexpcube, CPL_FRAME_GROUP_RAW);

    double fullmean = 0.0;
    for (int ii = 0; ii < nframes; ii++) {
        zplexp_plane = sph_zpl_exposure_load(zexpcube, ii);
        mean[ii] = cpl_image_get_mean(zplexp_plane->image_zero_odd)
                + cpl_image_get_mean(zplexp_plane->image_zero_even)
                + cpl_image_get_mean(zplexp_plane->image_pi_odd)
                + cpl_image_get_mean(zplexp_plane->image_pi_even);
        //printf("mean (%d) = %f", ii, mean[ii]);
        fullmean = fullmean + mean[ii];
        sph_zpl_exposure_delete(zplexp_plane);
    }

    fullmean = fullmean / nframes;
    //printf("fullmean %f = ", fullmean);

    rerr = cpl_frameset_insert(inframes, zexpcube);

    //load fctable
    fctable = sph_fctable_load_ascii(cpl_frame_get_filename(fcframe));
    cpl_frame_delete(fcframe); fcframe = NULL;
    fname2 = sph_filemanager_filename_new_ext(cpl_frame_get_filename(zexpcube),
            "txt");
    fname = sph_filemanager_new_filename_from_base(fname2, "fctable");
    sph_fctable_save_ascii(fctable, fname);

    frame_fc = cpl_frame_new();
    cpl_frame_set_filename(frame_fc, fname);
    cpl_frame_set_tag(frame_fc, SPH_ZPL_TAG_STAR_CENTER_CALIB);
    cpl_frame_set_group(frame_fc, CPL_FRAME_GROUP_CALIB);

    //sph_fctable_save_ascii(fctable, cpl_frame_get_filename(frame));
    cpl_free(fname2);
    fname2 = NULL;
    cpl_free(fname);
    fname = NULL;
    sph_fctable_delete(fctable);
    fctable = NULL;


    rerr = cpl_frameset_insert(inframes, frame_fc);

    zpl_science_p1 = sph_zpl_science_p1_new(inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_science_p1);

    rerr = sph_zpl_science_p1_run(zpl_science_p1);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    pl = cpl_propertylist_load(
            zpl_science_p1->science_p1_plus_q_outfilename_cam1, 0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME),
            fullmean, 1e-3);
    //CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-10);
    cpl_propertylist_delete(pl);
    pl = NULL;

    //destroy filemanager and keywordmanager
    sph_filemanager_delete(0);
    sph_keyword_manager_delete();

    rerr = sph_zpl_science_p1_delete(zpl_science_p1);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);

    //destroy inframes and inparams of the test
    cpl_frameset_delete(inframes);
    cpl_parameterlist_delete(inparams);


    return;
}

#if 0
/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_science_p1 with preproc frames but without keywords
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_science_p1_run_1_1(void) {
    sph_zpl_science_p1* zpl_science_p1 = NULL;
    cpl_parameterlist* inparams = NULL;
    cpl_frameset* inframes = NULL;
    cpl_frame* zexpcube = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    int nx = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int ny = 100;// -//-

    cpl_propertylist* pl = NULL;
    sph_fctable* fctable = NULL;
    cpl_frame* fcframe = NULL;

    cpl_frame* frame_fc = NULL;
    char* fname = NULL;
    char* fname2 = NULL;

    sph_zpl_exposure* zplexp_plane = NULL;
    cpl_table* fctab = NULL;
    cpl_propertylist* pl_fctab = NULL;

    int nframes = 5;
    //double dx[] = {1.0, 1.0, 1.0, 1.0, 1.0 };
    //double dy[] = {1.0, -1.0, 2.0, -2.0, 3.0 };
    double dx[] = {0.0, 0.0, 0.0, 0.0, 0.0};
    double dy[] = {0.0, 0.0, 0.0, 0.0, 0.0};
    double angle[] = {0, 15, 30, 45, 60};
    double mean[] = {0.0, 0.0, 0.0, 0.0, 0.0};

    inparams = sph_zpl_science_p1_create_paramlist();
    inframes = cpl_frameset_new();

    //simulate fctable
    fcframe = sph_test_simulator_create_fctable(nx, ny, dx, dy, angle, nframes);
    //rerr = cpl_frameset_insert( inframes, fcframe );

    //simulate cube of rotated pre-processed (ZPL EXP) science frames
    sph_test_simulator_create_zplpreproc_science_cube(&zexpcube, fcframe, nx, ny );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_SCIENCE_P1_PREPROC_CAM1 );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );

    double fullmean = 0.0;
    for ( int ii = 0; ii < nframes; ii++) {
        zplexp_plane = sph_zpl_exposure_load( zexpcube , ii);
        mean[ii] = cpl_image_get_mean( zplexp_plane->image_zero_odd )+
        cpl_image_get_mean( zplexp_plane->image_zero_even )+
        cpl_image_get_mean( zplexp_plane->image_pi_odd) +
        cpl_image_get_mean( zplexp_plane->image_pi_even );
        //printf("mean (%d) = %f", ii, mean[ii]);
        fullmean = fullmean + mean[ii];
        sph_zpl_exposure_delete( zplexp_plane );
    }

    fullmean = fullmean/nframes;
    //printf("fullmean %f = ", fullmean);

    rerr = cpl_frameset_insert( inframes, zexpcube );

    //load fctable
    fctable = sph_fctable_load_ascii( cpl_frame_get_filename( fcframe ));
    fname2 = sph_filemanager_filename_new_ext( cpl_frame_get_filename(zexpcube), "txt");
//    fname2 = sph_filemanager_filename_new_ext( cpl_frame_get_filename(zexpcube), "fits");
    fname = sph_filemanager_new_filename_from_base( fname2, "fctable");
//    sph_fctable_save_ascii(fctable, fname);
    fctab = sph_fctable_get_as_cpl_table(fctable);
    pl_fctab = cpl_propertylist_new();
    cpl_propertylist_append_string( pl_fctab, SPH_COMMON_KEYWORD_PRO_FIRST_RAW_FRAME, cpl_frame_get_filename(zexpcube) );
    cpl_table_save( fctab, pl_fctab, pl_fctab, fname, CPL_IO_CREATE);

    frame_fc = cpl_frame_new();
    cpl_frame_set_filename( frame_fc, fname );
    cpl_frame_set_tag(frame_fc,SPH_ZPL_TAG_STAR_CENTER_CALIB);
    cpl_frame_set_group(frame_fc,CPL_FRAME_GROUP_CALIB);

    //sph_fctable_save_ascii(fctable, cpl_frame_get_filename(frame));
    cpl_free(fname2); fname2 = NULL;
    cpl_free(fname); fname = NULL;
    sph_fctable_delete(fctable); fctable = NULL;
    cpl_frame_delete( fcframe );

    rerr = cpl_frameset_insert( inframes, frame_fc );

    zpl_science_p1 = sph_zpl_science_p1_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_science_p1);

    rerr = sph_zpl_science_p1_run( zpl_science_p1 );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );

    pl = cpl_propertylist_load( zpl_science_p1->science_p1_outfilename_q_cam1, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), fullmean, 1e-3);
    //CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_science_p1_delete( zpl_science_p1 );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_science_p1 with preproc frames but with .head files hwpz keywords
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_science_p1_run_2(void) {
    sph_zpl_science_p1* zpl_science_p1 = NULL;
    cpl_parameterlist* inparams = NULL;
    cpl_frameset* inframes = NULL;
    cpl_frame* zexpcube = NULL;
    cpl_frame* zexpcube1 = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    int nx = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int ny = 100;// -//-

    cpl_propertylist* pl = NULL;
    sph_fctable* fctable = NULL;
    cpl_frame* fcframe = NULL;

    cpl_frame* frame_fc = NULL;
    char* fname = NULL;
    char* fname2 = NULL;

    sph_zpl_exposure* zplexp_plane = NULL;
    sph_keyword_manager* keyman = NULL;

    int nframes = 5;
    //double dx[] = {1.0, 1.0, 1.0, 1.0, 1.0 };
    //double dy[] = {1.0, -1.0, 2.0, -2.0, 3.0 };
    double dx[] = {0.0, 0.0, 0.0, 0.0, 0.0};
    double dy[] = {0.0, 0.0, 0.0, 0.0, 0.0};
    double angle[] = {0, 15, 30, 45, 60};
    double mean[] = {0.0, 0.0, 0.0, 0.0, 0.0};

    //create sphere.cfg
    keyman = cutest_zpl_science_create_cfg_file();
    CU_ASSERT_PTR_NOT_NULL_FATAL( keyman );

    inparams = sph_zpl_science_p1_create_paramlist();
    inframes = cpl_frameset_new();

    //FIRST CUBE
    //simulate fctable
    fcframe = sph_test_simulator_create_fctable(nx, ny, dx, dy, angle, nframes);
    //rerr = cpl_frameset_insert( inframes, fcframe );

    //simulate cube of rotated pre-processed (ZPL EXP) science frames
    sph_test_simulator_create_zplpreproc_science_cube(&zexpcube, fcframe, nx, ny );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_SCIENCE_P1_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );

    double fullmean = 0.0;
    for ( int ii = 0; ii < nframes; ii++) {
        zplexp_plane = sph_zpl_exposure_load( zexpcube , ii);
        mean[ii] = cpl_image_get_mean( zplexp_plane->image_zero_odd )+
        cpl_image_get_mean( zplexp_plane->image_zero_even )+
        cpl_image_get_mean( zplexp_plane->image_pi_odd) +
        cpl_image_get_mean( zplexp_plane->image_pi_even );
        //printf("mean (%d) = %f", ii, mean[ii]);
        fullmean = fullmean + mean[ii];
        sph_zpl_exposure_delete( zplexp_plane );
    }

    //create .head file for first zplexp cube
    cutest_zpl_science_create_head_file_hwpz_plus_0( cpl_frame_get_filename(zexpcube) );

    rerr = cpl_frameset_insert( inframes, zexpcube );

    //load fctable
    fctable = sph_fctable_load_ascii( cpl_frame_get_filename( fcframe ));
    fname2 = sph_filemanager_filename_new_ext( cpl_frame_get_filename(zexpcube), "txt");
    fname = sph_filemanager_new_filename_from_base( fname2, "fctable");
    sph_fctable_save_ascii(fctable, fname);

    frame_fc = cpl_frame_new();
    cpl_frame_set_filename( frame_fc, fname );
    cpl_frame_set_tag(frame_fc,SPH_ZPL_TAG_STAR_CENTER_CALIB);
    cpl_frame_set_group(frame_fc,CPL_FRAME_GROUP_CALIB);

    //sph_fctable_save_ascii(fctable, cpl_frame_get_filename(frame));
    cpl_free(fname2); fname2 = NULL;
    cpl_free(fname); fname = NULL;
    sph_fctable_delete(fctable); fctable = NULL;
    cpl_frame_delete( fcframe );

    rerr = cpl_frameset_insert( inframes, frame_fc );

    //SECOND CUBE
    angle[0] = 5.0; angle[1] = 25.0; angle[2] = 40.0; angle[3] = 60.0; angle[4] = 75.0;
    //simulate fctable
    fcframe = sph_test_simulator_create_fctable(nx, ny, dx, dy, angle, nframes);

    //simulate cube of rotated pre-processed (ZPL EXP) science frames
    sph_test_simulator_create_zplpreproc_science_cube(&zexpcube1, fcframe, nx, ny );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_SCIENCE_P1_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );

    //double fullmean = 0.0;
    for ( int ii = 0; ii < nframes; ii++) {
        zplexp_plane = sph_zpl_exposure_load( zexpcube1 , ii);
        mean[ii] = cpl_image_get_mean( zplexp_plane->image_zero_odd )+
        cpl_image_get_mean( zplexp_plane->image_zero_even )+
        cpl_image_get_mean( zplexp_plane->image_pi_odd) +
        cpl_image_get_mean( zplexp_plane->image_pi_even );
        //printf("mean (%d) = %f", ii, mean[ii]);
        fullmean = fullmean + mean[ii];
        sph_zpl_exposure_delete( zplexp_plane );
    }

    fullmean = fullmean/(2*nframes);

    //create .head file for second zplexp cube1
    cutest_zpl_science_create_head_file_hwpz_minus_45(cpl_frame_get_filename(zexpcube1));

    rerr = cpl_frameset_insert( inframes, zexpcube1 );

    //load fctable
    fctable = sph_fctable_load_ascii( cpl_frame_get_filename( fcframe ));
    fname2 = sph_filemanager_filename_new_ext( cpl_frame_get_filename(zexpcube1), "txt");
    fname = sph_filemanager_new_filename_from_base( fname2, "fctable");
    sph_fctable_save_ascii(fctable, fname);

    frame_fc = cpl_frame_new();
    cpl_frame_set_filename( frame_fc, fname );
    cpl_frame_set_tag(frame_fc,SPH_ZPL_TAG_STAR_CENTER_CALIB);
    cpl_frame_set_group(frame_fc,CPL_FRAME_GROUP_CALIB);

    //sph_fctable_save_ascii(fctable, cpl_frame_get_filename(frame));
    cpl_free(fname2); fname2 = NULL;
    cpl_free(fname); fname = NULL;
    sph_fctable_delete(fctable); fctable = NULL;
    cpl_frame_delete( fcframe );

    rerr = cpl_frameset_insert( inframes, frame_fc );

    zpl_science_p1 = sph_zpl_science_p1_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_science_p1);

    rerr = sph_zpl_science_p1_run( zpl_science_p1 );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );

    pl = cpl_propertylist_load( zpl_science_p1->science_p1_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), fullmean, 1e-3);
    //CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    rerr = sph_zpl_science_p1_delete( zpl_science_p1 );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    sph_keyword_manager_delete();

    return;
}

#endif

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_science_p1 with preproc frames but without hwpz keywords
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_science_p1_run_3(void) {
    sph_zpl_science_p1* zpl_science_p1 = NULL;
    cpl_parameterlist* inparams = NULL;
    cpl_frameset* inframes = NULL;
    cpl_frame* zexpcube = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    int nx = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int ny = 100; // -//-

    cpl_propertylist* pl = NULL;
    sph_fctable* fctable = NULL;
    cpl_frame* fcframe = NULL;

    cpl_frame* frame_fc = NULL;
    char* fname = NULL;
    char* fname2 = NULL;

    sph_zpl_exposure* zplexp_plane = NULL;

    double ovsc[8] = { 15.0, 1.0, 25.0, 2.0, 35.0, 3.0, 45.0, 4.0 };

    int nframes = 5;
    //double dx[] = {1.0, 1.0, 1.0, 1.0, 1.0 };
    //double dy[] = {1.0, -1.0, 2.0, -2.0, 3.0 };
    double dx[] = { 0.0, 0.0, 0.0, 0.0, 0.0 };
    double dy[] = { 0.0, 0.0, 0.0, 0.0, 0.0 };
    double angle[] = { 0, 15, 30, 45, 60 };
    double mean[] = { 0.0, 0.0, 0.0, 0.0, 0.0 };

    inparams = sph_zpl_science_p1_create_paramlist();
    inframes = cpl_frameset_new();

    //simulate fctable
    fcframe = sph_test_simulator_create_fctable(nx, ny, dx, dy, angle, nframes);
    //rerr = cpl_frameset_insert( inframes, fcframe );

    //simulate cube of rotated pre-processed (ZPL EXP) science frames
    sph_test_simulator_create_zplpreproc_science_cube_overscan(&zexpcube,
            fcframe, nx, ny, ovsc);
    cpl_frame_set_tag(zexpcube, SPH_ZPL_TAG_SCIENCE_P1_PREPROC_CAM2);
    cpl_frame_set_group(zexpcube, CPL_FRAME_GROUP_RAW);

    double fullmean = 0.0;
    for (int ii = 0; ii < nframes; ii++) {
        zplexp_plane = sph_zpl_exposure_load(zexpcube, ii);
        mean[ii] = cpl_image_get_mean(zplexp_plane->image_zero_odd)
                + cpl_image_get_mean(zplexp_plane->image_zero_even)
                + cpl_image_get_mean(zplexp_plane->image_pi_odd)
                + cpl_image_get_mean(zplexp_plane->image_pi_even)
                - (ovsc[0] + ovsc[2] + ovsc[4] + ovsc[6]);
        //printf("mean (%d) = %f", ii, mean[ii]);
        fullmean = fullmean + mean[ii];
        sph_zpl_exposure_delete(zplexp_plane);
    }

    fullmean = fullmean / nframes;
    //printf("fullmean %f = ", fullmean);

    rerr = cpl_frameset_insert(inframes, zexpcube);

    //load fctable
    fctable = sph_fctable_load_ascii(cpl_frame_get_filename(fcframe));
    cpl_frame_delete(fcframe);

    fname2 = sph_filemanager_filename_new_ext(cpl_frame_get_filename(zexpcube),
            "txt");
    fname = sph_filemanager_new_filename_from_base(fname2, "fctable");
    sph_fctable_save_ascii(fctable, fname);

    frame_fc = cpl_frame_new();
    cpl_frame_set_filename(frame_fc, fname);
    cpl_frame_set_tag(frame_fc, SPH_ZPL_TAG_STAR_CENTER_CALIB);
    cpl_frame_set_group(frame_fc, CPL_FRAME_GROUP_CALIB);

    //sph_fctable_save_ascii(fctable, cpl_frame_get_filename(frame));
    cpl_free(fname2);
    fname2 = NULL;
    cpl_free(fname);
    fname = NULL;
    sph_fctable_delete(fctable);
    fctable = NULL;

    rerr = cpl_frameset_insert(inframes, frame_fc);
    //destroy filemanager and keywordmanager

    zpl_science_p1 = sph_zpl_science_p1_new(inframes, inparams);
    //CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_science_p1);

    rerr = sph_zpl_science_p1_run(zpl_science_p1);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    pl = cpl_propertylist_load(
            zpl_science_p1->science_p1_plus_q_outfilename_cam2, 0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME),
            fullmean, 1e-3);
    //CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-10);
    cpl_propertylist_delete(pl);
    pl = NULL;

    sph_filemanager_delete(0);
    sph_keyword_manager_delete();

    rerr = sph_zpl_science_p1_delete(zpl_science_p1);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);

    //destroy inframes and inparams of the test
    cpl_frameset_delete(inframes);
    cpl_parameterlist_delete(inparams);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_zpl_science_p1 recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;

    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("science_p1_test", cutest_init_science_p1_testsuite,
            cutest_clean_science_p1_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */

/*
    if (NULL
            == CU_add_test(
                    pSuite,
                    "sph_science_p1_run_1: 2 cube-rawframes/preproc without keywords +Q/-Q and +U/-U",
                    cutest_zpl_science_p1_run_1)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
*/

    /*
     if ( NULL == CU_add_test(pSuite, "sph_science_p1_run_1_1: 2 cube-rawframes/preproc without keywords +Q/-Q and +U/-U",
     cutest_zpl_science_p1_run_1_1 ) )
     {
     CU_cleanup_registry();
     return CU_get_error();
     }

     */
    /*
     if ( NULL == CU_add_test(pSuite, "sph_science_p1_run_2: 2 cube-rawframes/preproc with "
     ".head files containing keywords for +Q[hwp2=0.0] and -Q[hwp2=45.0] ",
     cutest_zpl_science_p1_run_2 ) )
     {
     CU_cleanup_registry();
     return CU_get_error();
     }
     */


/*
    if (NULL
            == CU_add_test(
                    pSuite,
                    "sph_science_p1_run_3: 1 cube-rawframes/preproc with overscan tables",
                    cutest_zpl_science_p1_run_3)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
*/




    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
