/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_dataset_peak.h"
#include "sph_dataset.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and 
 *                                  associated functions.               
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/

sph_dataset*
cutest_util_dataset_new_load(void) {
    sph_dataset* dat = NULL;
    cpl_vector* lams = NULL;
    cpl_vector* vals = NULL;
    int ii = 0;

    dat = sph_dataset_new(100, 1.0, 100.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( dat);

    lams = dat->xvalues;
    vals = dat->yvalues;
    CU_ASSERT_PTR_NOT_NULL_FATAL( lams);
    CU_ASSERT_PTR_NOT_NULL_FATAL( vals);

    for (ii = 0; ii < cpl_vector_get_size(lams); ++ii) {
        cpl_vector_set(vals, ii, (double) ii);
    }

    CU_ASSERT_EQUAL( cpl_vector_get( lams, 0 ), 1.0);
    CU_ASSERT_EQUAL( cpl_vector_get( lams, 99 ), 100.0);
    return dat;
}
sph_dataset*
cutest_util_dataset_new_gauss(void) {
    sph_dataset* dat = NULL;
    cpl_vector* lams = NULL;
    cpl_vector* vals = NULL;
    double val = 0.0;
    int ii = 0;

    dat = sph_dataset_new(100, 1.0, 100.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( dat);

    lams = dat->xvalues;
    vals = dat->yvalues;
    CU_ASSERT_PTR_NOT_NULL_FATAL( lams);
    CU_ASSERT_PTR_NOT_NULL_FATAL( vals);

    for (ii = 0; ii < cpl_vector_get_size(lams); ++ii) {
        val = cpl_vector_get(lams, ii) - 50.0;
        val = exp(-val * val / 25.0);
        cpl_vector_set(vals, ii, val);
    }

    return dat;
}
sph_dataset*
cutest_util_dataset_new_double_gauss(void) {
    sph_dataset* dat = NULL;
    cpl_vector* lams = NULL;
    cpl_vector* vals = NULL;
    double val = 0.0;
    double valB = 0.0;
    int ii = 0;

    dat = sph_dataset_new(100, 1.0, 100.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( dat);

    lams = dat->xvalues;
    vals = dat->yvalues;
    CU_ASSERT_PTR_NOT_NULL_FATAL( lams);
    CU_ASSERT_PTR_NOT_NULL_FATAL( vals);

    for (ii = 0; ii < cpl_vector_get_size(lams); ++ii) {
        val = cpl_vector_get(lams, ii) - 25.0;
        val = 5.0 * exp(-val * val / 400.0);
        valB = cpl_vector_get(lams, ii) - 75.0;
        valB = exp(-valB * valB / 400.0);
        val = val + valB;
        cpl_vector_set(vals, ii, val);
    }

    return dat;
}

int cutest_init_dataset_peak_testsuite(void) {
    sph_init_test(__FILE__, "moeller@mpia.de", CPL_MSG_DEBUG);
    return 0;
}

int cutest_clean_dataset_peak_testsuite(void) {
    sph_end_test();
    return 0;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_dataset_peak_new function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_dataset_peak_new_root(void) {
    sph_dataset_peak* peak = NULL;
    sph_dataset* dat = NULL;
    int rerr = CPL_ERROR_NONE;

    /* Setup and run ...*/
    dat = cutest_util_dataset_new_gauss();
    CU_ASSERT_PTR_NOT_NULL_FATAL( dat);

    peak = sph_dataset_peak_new_root(dat);

    CU_ASSERT_PTR_NOT_NULL_FATAL( peak);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    /*Verify */
    CU_ASSERT_PTR_NOT_NULL_FATAL( dat);
    sph_dataset_peak_delete(peak);
    peak = NULL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_dataset_peak_new function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_dataset_peak_new_sub(void) {
    sph_dataset_peak* root = NULL;
    sph_dataset_peak* peak = NULL;
    sph_dataset* dat = NULL;
    int rerr = CPL_ERROR_NONE;

    /* Setup and run ...*/
    dat = cutest_util_dataset_new_gauss();
    CU_ASSERT_PTR_NOT_NULL_FATAL( dat);

    root = sph_dataset_peak_new_root(dat);
    CU_ASSERT_PTR_NOT_NULL_FATAL( root);

    peak = sph_dataset_peak_new_sub(root, 1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( peak);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    /*Verify */
    CU_ASSERT_PTR_NOT_NULL_FATAL( dat);
    sph_dataset_peak_delete(peak);
    peak = NULL;
    sph_dataset_peak_delete(root);
    root = NULL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_dataset_peak_new function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_dataset_peak_find_subpeaks(void) {
    sph_dataset_peak* root = NULL;
    sph_dataset* dat = NULL;
    int rerr = CPL_ERROR_NONE;

    /* Setup and run ...*/
    dat = cutest_util_dataset_new_double_gauss();
    CU_ASSERT_PTR_NOT_NULL_FATAL( dat);

    root = sph_dataset_peak_new_root(dat);
    CU_ASSERT_PTR_NOT_NULL_FATAL( root);

    root->frac_threshold = 0.0;
    root->frac_acc = 0.01;

    rerr = sph_dataset_peak_find_subpeaks(root);

    /*Verify */
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_PTR_NOT_NULL_FATAL( root->firstchild);
    CU_ASSERT_PTR_NOT_NULL_FATAL( root->firstchild->next);
    CU_ASSERT_PTR_NULL( root->firstchild->next->next);
    CU_ASSERT_DOUBLE_EQUAL( root->firstchild->maxpos, 25.0, 0.05);
    CU_ASSERT_DOUBLE_EQUAL( root->firstchild->next->maxpos, 75.0, 1.15);
    if (CU_get_number_of_failures() > 0) {
        printf("First peak: %lf, Second peak: %lf\n", root->firstchild->maxpos,
                root->firstchild->next->maxpos);
    }
    sph_dataset_peak_delete(root);
    root = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_dataset_peak_delete function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_dataset_peak_delete(void) {
    sph_dataset* dat = NULL;
    sph_dataset_peak* peak = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;

    dat = cutest_util_dataset_new_gauss();
    CU_ASSERT_PTR_NOT_NULL_FATAL( dat);
    peak = sph_dataset_peak_new_root(dat);
    CU_ASSERT_PTR_NOT_NULL_FATAL( peak);

    rerr = sph_dataset_peak_delete(peak);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("dataset_peak_test",
            cutest_init_dataset_peak_testsuite,
            cutest_clean_dataset_peak_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    if (NULL
            == CU_add_test(pSuite, "sph_dataset_peak_new_root",
                    cutest_dataset_peak_new_root)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_dataset_peak_new_sub",
                    cutest_dataset_peak_new_sub)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_dataset_find_subpeaks",
                    cutest_dataset_peak_find_subpeaks)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_dataset_peak_delete",
                    cutest_dataset_peak_delete)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
