/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>
#include <float.h>

#include "sph_ifs_lenslet_model.h"
#include "sph_dataset.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"
#include "sph_image_grid.h"
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "sph_utils.h"
#include "cutest_sph_ifs_lenslet_model.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup cutest_sph_ifs_lenslet_model    unit tests for sph_ifs_lenslet_model
 * and associated functions.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("sph_ifs_lenslet_model_test",
            cutest_init_ifs_lenslet_model_testsuite,
            cutest_clean_ifs_lenslet_model_testsuite);

    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_new",
                    cutest_ifs_lenslet_model_new)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_create_pdt_images",
                    cutest_ifs_lenslet_model_create_pdt_images)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "sph_ifs_lenslet_model_new_from_propertylist",
                    cutest_ifs_lenslet_model_new_from_propertylist)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "sph_ifs_lenslet_model_new_from_propertylist_distmodel",
                    cutest_ifs_lenslet_model_new_from_propertylist_distmodel)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_save",
                    cutest_ifs_lenslet_model_save)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_load",
                    cutest_ifs_lenslet_model_load)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_get_lambda_vector",
                    cutest_ifs_lenslet_model_get_lambda_vector)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_get_as_propertylist",
                    cutest_ifs_lenslet_model_get_as_propertylist)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_get_coords",
                    cutest_ifs_lenslet_model_get_coords)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_get_index",
                    cutest_ifs_lenslet_model_get_index)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_delete",
                    cutest_ifs_lenslet_model_delete)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_get_centre",
                    cutest_ifs_lenslet_model_get_centre)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_ifs_lenslet_model_get_poly",
                    cutest_ifs_lenslet_model_get_poly)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "sph_ifs_lenslet_model_set_lenslet_distortion",
                    cutest_ifs_lenslet_model_set_lenslet_distortion)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "sph_ifs_lenslet_model_set_get_hex_w_lenslet_distortion",
                    cutest_ifs_lenslet_model_set_get_hex_w_lenslet_distortion)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_new function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_new(void) {
    sph_ifs_lenslet_model* model = NULL;
    int rerr = CPL_ERROR_NONE;

    /* Setup and run ...*/
    cpl_error_reset();

    model = sph_ifs_lenslet_model_new();

    CU_ASSERT_PTR_NOT_NULL_FATAL( model);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    /*Verify */
    CU_ASSERT_PTR_NOT_NULL_FATAL( model);
    CU_ASSERT_DOUBLE_EQUAL( model->detsize_microns,
            SPH_IFS_LENSLET_MODEL_DET_SIZE_MICRONS, 0.0);
    CU_ASSERT_EQUAL( model->detsize_pixels, SPH_IFS_LENSLET_MODEL_DET_SIZE_PIX);
    CU_ASSERT_DOUBLE_EQUAL( model->lensize_microns,
            SPH_IFS_LENSLET_MODEL_LENSLET_SIZE_MICRONS, 0.0);
    CU_ASSERT_EQUAL( model->lenslets_per_side,
            SPH_IFS_LENSLET_MODEL_LENSLETS_SIDE);
    CU_ASSERT_DOUBLE_EQUAL( model->pixsize_microns,
            SPH_IFS_LENSLET_MODEL_PIX_SIZE_MICRONS, 0.0);
    CU_ASSERT_DOUBLE_EQUAL( model->speclength_microns,
            SPH_IFS_LENSLET_MODEL_SPEC_LENGTH_MICRONS, 0.0);
    CU_ASSERT_DOUBLE_EQUAL( model->specwidth_microns,
            SPH_IFS_LENSLET_MODEL_SPEC_WIDTH_MICRONS, 0.0);
    CU_ASSERT_DOUBLE_EQUAL(
            model->speclength_pixels,
            SPH_IFS_LENSLET_MODEL_SPEC_LENGTH_MICRONS / SPH_IFS_LENSLET_MODEL_PIX_SIZE_MICRONS,
            0.000001);
    CU_ASSERT_DOUBLE_EQUAL(
            model->specwidth_pixels,
            SPH_IFS_LENSLET_MODEL_SPEC_WIDTH_MICRONS / SPH_IFS_LENSLET_MODEL_PIX_SIZE_MICRONS,
            0.00001);
    sph_ifs_lenslet_model_delete(model);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_save function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_save(void) {
    sph_ifs_lenslet_model* model = NULL;
    int rerr = CPL_ERROR_NONE;

    /* Setup and run ...*/
    cpl_error_reset();

    model = sph_ifs_lenslet_model_new();

    CU_ASSERT_PTR_NOT_NULL_FATAL( model);

    rerr = sph_ifs_lenslet_model_save(model, "lenslet_model_test.ini");

    /*Verify */
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    sph_ifs_lenslet_model_delete(model);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_load function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_load(void) {
    sph_ifs_lenslet_model* model = NULL;

    /* Setup and run ...*/
    cpl_error_reset();

    model = sph_ifs_lenslet_model_load("lenslet_model_test.ini");

#ifdef SPH_NO_ASCII
    CU_ASSERT_PTR_NULL_FATAL( model );
    cpl_error_reset();
    return;
#endif
    CU_ASSERT_PTR_NOT_NULL_FATAL( model);
    /*Verify */
    CU_ASSERT_DOUBLE_EQUAL( model->detsize_microns,
            SPH_IFS_LENSLET_MODEL_DET_SIZE_MICRONS, 0.0);
    CU_ASSERT_EQUAL( model->detsize_pixels, SPH_IFS_LENSLET_MODEL_DET_SIZE_PIX);
    CU_ASSERT_DOUBLE_EQUAL( model->lensize_microns,
            SPH_IFS_LENSLET_MODEL_LENSLET_SIZE_MICRONS, 0.0);
    CU_ASSERT_EQUAL( model->lenslets_per_side,
            SPH_IFS_LENSLET_MODEL_LENSLETS_SIDE);
    CU_ASSERT_DOUBLE_EQUAL( model->pixsize_microns,
            SPH_IFS_LENSLET_MODEL_PIX_SIZE_MICRONS, 0.0);
    CU_ASSERT_DOUBLE_EQUAL( model->speclength_microns,
            SPH_IFS_LENSLET_MODEL_SPEC_LENGTH_MICRONS, 0.0);
    CU_ASSERT_DOUBLE_EQUAL( model->specwidth_microns,
            SPH_IFS_LENSLET_MODEL_SPEC_WIDTH_MICRONS, 0.0);
    CU_ASSERT_DOUBLE_EQUAL(
            model->speclength_pixels,
            SPH_IFS_LENSLET_MODEL_SPEC_LENGTH_MICRONS / SPH_IFS_LENSLET_MODEL_PIX_SIZE_MICRONS,
            0.000001);
    CU_ASSERT_DOUBLE_EQUAL(
            model->specwidth_pixels,
            SPH_IFS_LENSLET_MODEL_SPEC_WIDTH_MICRONS / SPH_IFS_LENSLET_MODEL_PIX_SIZE_MICRONS,
            0.000001);
    SPH_RAISE_CPL;
    sph_ifs_lenslet_model_delete(model);
    SPH_RAISE_CPL;CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    SPH_RAISE_CPL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_new_from_propertylist function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_get_centre(void) {
    sph_ifs_lenslet_model* model = NULL;
    double x = 0;
    double y = 0;

    model = sph_ifs_lenslet_model_new();
    CU_ASSERT_EQUAL( sph_ifs_lenslet_model_get_centre(model,0,0,&x,&y),
            CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL(x, 0.0, 0.000000001);
    CU_ASSERT_DOUBLE_EQUAL(y, 0.0, 0.000000001);
    CU_ASSERT_EQUAL( sph_ifs_lenslet_model_get_centre(model,1,1,&x,&y),
            CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL(x, CPL_MATH_SQRT3 * model->lensize_microns * 1.5,
            0.000000001);
    CU_ASSERT_DOUBLE_EQUAL(y, 1.5 * model->lensize_microns, 0.000000001);
    sph_ifs_lenslet_model_delete(model);
    model = NULL;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_new_from_propertylist function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_new_from_propertylist(void) {
    sph_ifs_lenslet_model* model = NULL;
    cpl_propertylist* pl = NULL;

    /* Setup and run ...*/
    cpl_error_reset();
    pl = cpl_propertylist_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( pl);
    cpl_propertylist_append_int(pl, SPH_IFS_LENSLET_MODEL_DET_SIZE_NAME, 2048);
    cpl_propertylist_append_int(pl, SPH_IFS_LENSLET_MODEL_LENSLET_SIDE_NAME,
            145);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_LENSLET_SIZE_NAME,
            10.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_PIX_SIZE_NAME,
            1.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_SPEC_LENGTH_NAME,
            100.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_SPEC_WIDTH_NAME,
            4.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_ROTANGLE_NAME,
            1.333);

    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    model = sph_ifs_lenslet_model_new_from_propertylist(pl);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);

    CU_ASSERT_PTR_NOT_NULL_FATAL( model);
    /*Verify */
    CU_ASSERT_DOUBLE_EQUAL( model->detsize_microns, 2048.0, DBL_EPSILON);
    CU_ASSERT_EQUAL( model->detsize_pixels, 2048);
    CU_ASSERT_DOUBLE_EQUAL( model->lensize_microns, 10.0, DBL_EPSILON);
    CU_ASSERT_EQUAL( model->lenslets_per_side, 145);
    CU_ASSERT_DOUBLE_EQUAL( model->pixsize_microns, 1.0, DBL_EPSILON);
    CU_ASSERT_DOUBLE_EQUAL( model->speclength_microns, 100.0, DBL_EPSILON);
    CU_ASSERT_DOUBLE_EQUAL( model->specwidth_microns, 4.0, DBL_EPSILON);
    CU_ASSERT_DOUBLE_EQUAL( model->rotangle, 1.333, DBL_EPSILON);
    cpl_propertylist_delete(pl);
    sph_ifs_lenslet_model_delete(model);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_new_from_propertylist function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_get_lambda_vector(void) {
    sph_ifs_lenslet_model* model = NULL;
    cpl_vector* lvect = NULL;
    double dl = 0.0;
    model = sph_ifs_lenslet_model_new();

    lvect = sph_ifs_lenslet_model_get_lambda_vector(model);
    dl = model->dispersion;
    CU_ASSERT_PTR_NOT_NULL_FATAL(lvect);
    CU_ASSERT_EQUAL( cpl_vector_get_size(lvect), (int)model->speclength_pixels);
    CU_ASSERT_DOUBLE_EQUAL(cpl_vector_get(lvect,0), model->minlambda, 0.001);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_vector_get(lvect, cpl_vector_get_size(lvect) - 1 ),
            model->maxlambda, 0.001);

    //cpl_vector_delete(lvect);
    sph_ifs_lenslet_model_delete(model);
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_new_from_propertylist function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_new_from_propertylist_distmodel(void) {
    sph_ifs_lenslet_model* model = NULL;
    cpl_propertylist* pl = NULL;
    sph_polynomial_distortion_model* dist = NULL;
    cpl_propertylist* pl2 = NULL;
    /* Setup and run ...*/
    cpl_error_reset();
    pl = cpl_propertylist_new();
    dist = sph_polynomial_distortion_model_new(NULL, NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pl);
    cpl_propertylist_append_int(pl, SPH_IFS_LENSLET_MODEL_DET_SIZE_NAME, 2048);
    cpl_propertylist_append_int(pl, SPH_IFS_LENSLET_MODEL_LENSLET_SIDE_NAME,
            145);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_LENSLET_SIZE_NAME,
            10.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_PIX_SIZE_NAME,
            1.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_SPEC_LENGTH_NAME,
            100.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_SPEC_WIDTH_NAME,
            4.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_ROTANGLE_NAME,
            1.333);
    cpl_propertylist_append(pl, pl2 =
            sph_polynomial_distortion_model_get_proplist(dist));
    cpl_propertylist_delete(pl2);
    pl2 = NULL;
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    model = sph_ifs_lenslet_model_new_from_propertylist(pl);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);

    CU_ASSERT_PTR_NOT_NULL_FATAL( model);
    /*Verify */
    CU_ASSERT_DOUBLE_EQUAL( model->detsize_microns, 2048.0, DBL_EPSILON);
    CU_ASSERT_EQUAL( model->detsize_pixels, 2048);
    CU_ASSERT_DOUBLE_EQUAL( model->lensize_microns, 10.0, DBL_EPSILON);
    CU_ASSERT_EQUAL( model->lenslets_per_side, 145);
    CU_ASSERT_DOUBLE_EQUAL( model->pixsize_microns, 1.0, DBL_EPSILON);
    CU_ASSERT_DOUBLE_EQUAL( model->speclength_microns, 100.0, DBL_EPSILON);
    CU_ASSERT_DOUBLE_EQUAL( model->specwidth_microns, 4.0, DBL_EPSILON);
    CU_ASSERT_DOUBLE_EQUAL( model->rotangle, 1.333, DBL_EPSILON);
    CU_ASSERT_PTR_NOT_NULL_FATAL( model->detector_distortion);
    cpl_propertylist_delete(pl);
    pl = NULL;
    sph_ifs_lenslet_model_delete(model);
    model = NULL;
    sph_polynomial_distortion_model_delete(dist);
    dist = NULL;
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_get_as_propertylist function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_get_as_propertylist(void) {
    sph_ifs_lenslet_model* model = NULL;
    cpl_propertylist* pl = NULL;
    cpl_propertylist* pl2 = NULL;

    /* Setup and run ...*/
    cpl_error_reset();
    pl = cpl_propertylist_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( pl);
    cpl_propertylist_append_int(pl, SPH_IFS_LENSLET_MODEL_DET_SIZE_NAME, 2048);
    cpl_propertylist_append_int(pl, SPH_IFS_LENSLET_MODEL_LENSLET_SIDE_NAME,
            145);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_LENSLET_SIZE_NAME,
            10.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_PIX_SIZE_NAME,
            1.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_SPEC_LENGTH_NAME,
            100.0);
    cpl_propertylist_append_double(pl, SPH_IFS_LENSLET_MODEL_SPEC_WIDTH_NAME,
            4.0);

    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    model = sph_ifs_lenslet_model_new_from_propertylist(pl);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);

    CU_ASSERT_PTR_NOT_NULL_FATAL( model);
    pl2 = sph_ifs_lenslet_model_get_as_propertylist(model);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pl2);

    /*Verify */
    CU_ASSERT_DOUBLE_EQUAL( model->detsize_microns, 2048, 0.0);
    CU_ASSERT_EQUAL( model->detsize_pixels, 2048);
    CU_ASSERT_DOUBLE_EQUAL( model->lensize_microns, 10.0, 0.0);
    CU_ASSERT_EQUAL( model->lenslets_per_side, 145);
    CU_ASSERT_DOUBLE_EQUAL( model->pixsize_microns, 1.0, 0.0);
    CU_ASSERT_DOUBLE_EQUAL( model->speclength_microns, 100.0, 0.0);
    CU_ASSERT_DOUBLE_EQUAL( model->specwidth_microns, 4.0, 0.0);

    CU_ASSERT_TRUE(
            cpl_propertylist_has(pl2, SPH_IFS_LENSLET_MODEL_DET_SIZE_NAME));
    CU_ASSERT_TRUE(
            cpl_propertylist_has(pl2, SPH_IFS_LENSLET_MODEL_LENSLET_SIDE_NAME));
    CU_ASSERT_TRUE(
            cpl_propertylist_has(pl2, SPH_IFS_LENSLET_MODEL_LENSLET_SIZE_NAME));
    CU_ASSERT_TRUE(
            cpl_propertylist_has(pl2, SPH_IFS_LENSLET_MODEL_PIX_SIZE_NAME));
    CU_ASSERT_TRUE(
            cpl_propertylist_has(pl2, SPH_IFS_LENSLET_MODEL_SPEC_LENGTH_NAME));
    CU_ASSERT_TRUE(
            cpl_propertylist_has(pl2, SPH_IFS_LENSLET_MODEL_SPEC_WIDTH_NAME));
    CU_ASSERT_EQUAL(
            cpl_propertylist_get_int( pl2, SPH_IFS_LENSLET_MODEL_DET_SIZE_NAME),
            2048);
    CU_ASSERT_EQUAL(
            cpl_propertylist_get_int( pl2, SPH_IFS_LENSLET_MODEL_LENSLET_SIDE_NAME),
            145);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl2, SPH_IFS_LENSLET_MODEL_LENSLET_SIZE_NAME),
            10.0, 0.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl2, SPH_IFS_LENSLET_MODEL_PIX_SIZE_NAME),
            1.0, 0.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl2, SPH_IFS_LENSLET_MODEL_SPEC_LENGTH_NAME),
            100, 0.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( pl2, SPH_IFS_LENSLET_MODEL_SPEC_WIDTH_NAME),
            4.0, 0.0);

    cpl_propertylist_delete(pl);
    cpl_propertylist_delete(pl2);
    sph_ifs_lenslet_model_delete(model);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ldt_predict_spec_loc
 function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_predict_spec_loc_pp(void) {
    sph_ifs_lenslet_model* model = NULL;
    int rerr = CPL_ERROR_NONE;
    cpl_image* timage = NULL;
    cpl_image* diffim = NULL;
    sph_point_pattern* pp0 = NULL;
    sph_point_pattern* pp = NULL;
    cpl_image* timage_shifted = NULL;
    int xx = 0;
    int yy = 0;
    int bpix = 0;
    double offx = 1.0;
    double offy = 2.0;

    cpl_error_reset();

    model = sph_ifs_lenslet_model_new();
    model->lenslets_per_side = 7;
    model->detsize_pixels = 128;

    CU_ASSERT_PTR_NOT_NULL_FATAL(model);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    pp0 = sph_ifs_lenslet_model_predict_spec_loc_pp(model, 0.0, 0.0);
    SPH_RAISE_CPL_RESET;CU_ASSERT_PTR_NOT_NULL_FATAL(pp0);
    timage = sph_point_pattern_create_image(pp0, 128, 128, 2.0);
    SPH_RAISE_CPL_RESET;CU_ASSERT_PTR_NOT_NULL_FATAL( timage);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    SPH_RAISE_CPL_RESET;
    pp = sph_ifs_lenslet_model_predict_spec_loc_pp(model, offx, offy);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pp);
    timage_shifted = sph_point_pattern_create_image(pp, 128, 128, 2.0);
    SPH_RAISE_CPL_RESET;CU_ASSERT_PTR_NOT_NULL_FATAL( timage_shifted);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    diffim = cpl_image_new(model->detsize_pixels, model->detsize_pixels,
            CPL_TYPE_DOUBLE);
    SPH_RAISE_CPL_RESET;CU_ASSERT_PTR_NOT_NULL_FATAL(diffim);
    cpl_image_save(timage,
            "sph_lenslet_model_predict_spec_loc_pp_shift_residual.fits",
            CPL_TYPE_DOUBLE, NULL, CPL_IO_CREATE);
    cpl_image_save(timage_shifted,
            "sph_lenslet_model_predict_spec_loc_pp_shift_residual.fits",
            CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND);

    for (xx = 0; xx < model->detsize_pixels; ++xx) {
        for (yy = 0; yy < model->detsize_pixels; ++yy) {
            if (xx + offx >= 0 && yy + offy >= 0
                    && yy + offy < model->detsize_pixels
                    && xx + offx < model->detsize_pixels) {
                cpl_image_set(
                        diffim,
                        xx + 1,
                        yy + 1,
                        cpl_image_get(timage, xx + 1, yy + 1, &bpix)
                                - cpl_image_get(timage_shifted, xx + 1 + offx,
                                        yy + 1 + offy, &bpix));
            }
        }
    }CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_absflux_window(diffim,20,20,108,108),
            0.0, 0.00001);

    cpl_image_save(diffim,
            "sph_lenslet_model_predict_spec_loc_pp_shift_residual.fits",
            CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND);

    sph_point_pattern_delete(pp);
    pp = NULL;
    sph_point_pattern_delete(pp0);
    pp0 = NULL;
    cpl_image_delete(timage);
    timage = NULL;
    cpl_image_delete(timage_shifted);
    timage_shifted = NULL;
    cpl_image_delete(diffim);
    diffim = NULL;
    sph_ifs_lenslet_model_delete(model);
    model = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ldt_predict_spec_loc
 function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_predict_spec_loc_pp_model(void) {
    sph_ifs_lenslet_model* model = NULL;
    int rerr = CPL_ERROR_NONE;
    cpl_image* timage = NULL;
    cpl_image* diffim = NULL;
    sph_point_pattern* pp0 = NULL;
    sph_point_pattern* pp = NULL;
    cpl_image* timage_shifted = NULL;
    int xx = 0;
    int yy = 0;
    int bpix = 0;
    double offx = 1.0;
    double offy = 2.0;

    cpl_error_reset();

    model = sph_ifs_lenslet_model_new();
    model->lenslets_per_side = 7;
    model->detsize_pixels = 128;

    CU_ASSERT_PTR_NOT_NULL_FATAL(model);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    pp0 = sph_ifs_lenslet_model_predict_spec_loc_pp(model, 0.0, 0.0);
    SPH_RAISE_CPL_RESET;CU_ASSERT_PTR_NOT_NULL_FATAL(pp0);
    timage = sph_point_pattern_create_image(pp0, 128, 128, 2.0);
    SPH_RAISE_CPL_RESET;CU_ASSERT_PTR_NOT_NULL_FATAL( timage);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    SPH_RAISE_CPL_RESET;
    model->zero_offsetx = model->zero_offsetx + offx;
    model->zero_offsety = model->zero_offsety + offy;
    pp = sph_ifs_lenslet_model_predict_spec_loc_pp(model, 0.0, 0.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pp);
    timage_shifted = sph_point_pattern_create_image(pp, 128, 128, 2.0);
    SPH_RAISE_CPL_RESET;CU_ASSERT_PTR_NOT_NULL_FATAL( timage_shifted);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    diffim = cpl_image_new(model->detsize_pixels, model->detsize_pixels,
            CPL_TYPE_DOUBLE);
    SPH_RAISE_CPL_RESET;CU_ASSERT_PTR_NOT_NULL_FATAL(diffim);
    cpl_image_save(timage, "sph_lenslet_model_predict_spec_loc_pp_mod.fits",
            CPL_TYPE_DOUBLE, NULL, CPL_IO_CREATE);
    cpl_image_save(timage_shifted,
            "sph_lenslet_model_predict_spec_loc_pp_mod.fits", CPL_TYPE_DOUBLE,
            NULL, CPL_IO_EXTEND);

    for (xx = 0; xx < model->detsize_pixels; ++xx) {
        for (yy = 0; yy < model->detsize_pixels; ++yy) {
            if (xx + offx >= 0 && yy + offy >= 0
                    && yy + offy < model->detsize_pixels
                    && xx + offx < model->detsize_pixels) {
                cpl_image_set(
                        diffim,
                        xx + 1,
                        yy + 1,
                        cpl_image_get(timage, xx + 1, yy + 1, &bpix)
                                - cpl_image_get(timage_shifted, xx + 1 + offx,
                                        yy + 1 + offy, &bpix));
            }
        }
    }CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_absflux_window(diffim,20,20,108,108),
            0.0, 0.00001);

    cpl_image_save(diffim, "sph_lenslet_model_predict_spec_loc_pp_mod.fits",
            CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND);

    sph_point_pattern_delete(pp);
    pp = NULL;
    sph_point_pattern_delete(pp0);
    pp0 = NULL;
    cpl_image_delete(timage);
    timage = NULL;
    cpl_image_delete(timage_shifted);
    timage_shifted = NULL;
    cpl_image_delete(diffim);
    diffim = NULL;
    sph_ifs_lenslet_model_delete(model);
    model = NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ldt_get_poly function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_get_poly(void) {
    sph_ifs_lenslet_model* model = NULL;
    int rerr = CPL_ERROR_NONE;
    cpl_image* timage = NULL;
    sph_image_grid* tgrid = NULL;
    int uu = 0;
    int vv = 0;
    sph_polygon* pol = NULL;
    /* Setup and run ...*/
    int nrings = 0;

    cpl_error_reset();
    model = sph_ifs_lenslet_model_new();

    CU_ASSERT_PTR_NOT_NULL_FATAL(model);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    timage = cpl_image_new(256, 256, CPL_TYPE_DOUBLE);
    tgrid = sph_image_grid_new_from_image(timage, -2500.5, 2500.5, -2500.5,
            2500.5);
    sph_image_grid_rotate_and_shift(tgrid, 0.0, 0.0, 20.0, 0.0, 0.0);
    nrings = model->lenslets_per_side;
    cpl_image_delete(timage);
    timage = NULL;
    CU_ASSERT_EQUAL_FATAL( nrings, SPH_IFS_LENSLET_MODEL_LENSLETS_SIDE);
    for (vv = -nrings; vv <= nrings; ++vv) {
        if (vv >= 0) {
            for (uu = -nrings; uu <= nrings - vv; ++uu) {
                pol = sph_ifs_lenslet_model_get_poly(model, uu, vv);
                CU_ASSERT_PTR_NOT_NULL_FATAL( pol);
                sph_image_grid_map_polygon(tgrid, pol, 4.0);
                sph_polygon_delete(pol);
                pol = NULL;
            }
        } else {
            for (uu = 0 - (nrings + vv); uu <= nrings; ++uu) {
                pol = sph_ifs_lenslet_model_get_poly(model, uu, vv);
                CU_ASSERT_PTR_NOT_NULL_FATAL( pol);
                sph_image_grid_map_polygon(tgrid, pol, 4.0);
                sph_polygon_delete(pol);
                pol = NULL;
            }
        }
    }

    /*Verify */
    timage = sph_image_grid_extract_image(tgrid);
    rerr += cpl_image_save(timage, "lensletimage.fits", CPL_TYPE_DOUBLE, NULL,
            CPL_IO_DEFAULT);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    sph_image_grid_delete(tgrid);
    tgrid = NULL;
    cpl_image_delete(timage);
    timage = NULL;
    sph_ifs_lenslet_model_delete(model);
    model = NULL;
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_get_coords function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_get_coords(void) {
    sph_ifs_lenslet_model* model = NULL;
    int rerr = CPL_ERROR_NONE;
    int u = -100000;
    int v = -100000;
    double dx = 0.0;
    double dy = 0.0;
    int ii = 0;
    sph_polygon* poly = NULL;
    double x = 0.0;
    double y = 0.0;
    unsigned long ulSeed = 1;
    gsl_rng* pRNG = NULL;
    int inside = 0;
    sph_image_grid* markgrid = NULL;
    cpl_image* markimage = NULL;

    /* Setup and run ...*/
    cpl_error_reset();
    pRNG = gsl_rng_alloc(gsl_rng_taus);

    gsl_rng_set(pRNG, ulSeed);

    model = sph_ifs_lenslet_model_new();

    CU_ASSERT_PTR_NOT_NULL_FATAL( model);

    markimage = cpl_image_new(548, 548, CPL_TYPE_DOUBLE);

    markgrid = sph_image_grid_new_from_image(markimage,
            -model->lensize_microns * model->lenslets_per_side,
            model->lensize_microns * model->lenslets_per_side,
            -model->lensize_microns * model->lenslets_per_side,
            model->lensize_microns * model->lenslets_per_side);
    dx = model->lensize_microns * CPL_MATH_SQRT3;
    dy = model->lensize_microns * 3.0 / 2.0;

    rerr = sph_ifs_lenslet_model_get_coords(model, 0.0, 0.0, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, 0);
    CU_ASSERT_EQUAL( v, 0);

    rerr = sph_ifs_lenslet_model_get_coords(model, dx, 0.0, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, 1);
    CU_ASSERT_EQUAL( v, 0);

    rerr = sph_ifs_lenslet_model_get_coords(model, dx / 2.0, dy, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, 0);
    CU_ASSERT_EQUAL( v, 1);

    rerr = sph_ifs_lenslet_model_get_coords(model, dx - 0.0001, dy, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, 0);
    CU_ASSERT_EQUAL( v, 1);

    rerr = sph_ifs_lenslet_model_get_coords(model, dx + 0.00001, dy, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, 1);
    CU_ASSERT_EQUAL( v, 1);
    rerr = sph_ifs_lenslet_model_get_coords(model, dx, dy * 2.0, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, 0);
    CU_ASSERT_EQUAL( v, 2);
    rerr = sph_ifs_lenslet_model_get_coords(model, dx * 1.1, dy * 2.0, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, 0);
    CU_ASSERT_EQUAL( v, 2);
    rerr = sph_ifs_lenslet_model_get_coords(model, dx * 1.3, dy * 2.0, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, 0);
    CU_ASSERT_EQUAL( v, 2);
    rerr = sph_ifs_lenslet_model_get_coords(model,
            dx * model->lenslets_per_side, 0.0, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, model->lenslets_per_side);
    CU_ASSERT_EQUAL( v, 0);
    rerr = sph_ifs_lenslet_model_get_coords(model, 0.00001,
            dy * model->lenslets_per_side, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, -72);
    CU_ASSERT_EQUAL( v, model->lenslets_per_side);
    rerr = sph_ifs_lenslet_model_get_coords(model, 0.0,
            dy * (model->lenslets_per_side + 1), &u, &v);
    CU_ASSERT_EQUAL( rerr, SPH_IFS_LENSLET_MODEL_OUTSIDE_ARRAY);
    rerr = sph_ifs_lenslet_model_get_coords(model, -52.6 * dx, 87.5005 * dy, &u,
            &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, -96);
    CU_ASSERT_EQUAL( v, 87);
    poly = sph_ifs_lenslet_model_get_poly(model, u, v);
    CU_ASSERT_PTR_NOT_NULL_FATAL( poly);
    inside = sph_polygon_test_inside(poly, -52.6 * dx, 87.5005 * dy);
    CU_ASSERT_TRUE( inside);
    rerr = sph_ifs_lenslet_model_get_coords(model, -556.78, -1201.48, &u, &v);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_EQUAL( u, 1);
    CU_ASSERT_EQUAL( v, -5);
    sph_polygon_delete(poly);
    poly = NULL;
    poly = sph_ifs_lenslet_model_get_poly(model, u, v);
    CU_ASSERT_PTR_NOT_NULL_FATAL( poly);
    inside = sph_polygon_test_inside(poly, -556.78, -1201.48);
    CU_ASSERT_TRUE( inside);
    sph_polygon_delete(poly);
    poly = NULL;
    for (ii = 0; ii < 100000; ++ii) {
        x = gsl_ran_flat(pRNG,
                -model->lensize_microns * (model->lenslets_per_side + 0.5),
                model->lensize_microns * (model->lenslets_per_side + 0.5));
        y = gsl_ran_flat(pRNG,
                -model->lensize_microns * (model->lenslets_per_side + 0.5),
                model->lensize_microns * (model->lenslets_per_side + 0.5));

        rerr = sph_ifs_lenslet_model_get_coords(model, x, y, &u, &v);
        CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
        poly = sph_ifs_lenslet_model_get_poly(model, u, v);
        CU_ASSERT_PTR_NOT_NULL_FATAL( poly);
        inside = sph_polygon_test_inside(poly, x, y);
        CU_ASSERT_TRUE( inside);

        if (!inside) {
            sph_image_grid_mark_point(markgrid, x, y, 1.0);
        }

        sph_polygon_delete(poly);
        poly = NULL;
    }
    cpl_image_delete(markimage);
    markimage = NULL;
    markimage = sph_image_grid_extract_image(markgrid);
    cpl_image_save(markimage, "markimage.fits", CPL_TYPE_DOUBLE, NULL,
            CPL_IO_DEFAULT);
    sph_ifs_lenslet_model_delete(model);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    cpl_image_delete(markimage);
    markimage = NULL;
    sph_image_grid_delete(markgrid);
    markgrid = NULL;
    gsl_rng_free(pRNG);
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ldt_predict_spec_loc
 function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_create_pdt_images(void) {
    sph_ifs_lenslet_model* model = NULL;
    int rerr = CPL_ERROR_NONE;
    cpl_image* image_wav = NULL;
    cpl_image* image_id = NULL;
    cpl_image* specidim = NULL;
    cpl_image* image_ill = NULL;
    cpl_image* image_dw = NULL;
    cpl_mask* mask = NULL;
    cpl_size nregs = 0;

    cpl_error_reset();

    model = sph_ifs_lenslet_model_new();
    model->lenslets_per_side = 64;

    CU_ASSERT_PTR_NOT_NULL_FATAL(model);

    SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL, "Started PDT image creations...");
    rerr = sph_ifs_lenslet_model_create_pdt_images(model, &image_wav, &image_id,
            &image_ill, &image_dw, 0.0, 0.0);
    SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL, "Done.");

    cpl_image_save(image_id, "idimage.fits", CPL_TYPE_UNSPECIFIED, NULL,
            CPL_IO_CREATE);
    CU_ASSERT_EQUAL( cpl_image_get_max( image_id),
            sph_ifs_lenslet_model_get_nlens(model));
    cpl_image_save(image_ill, "illimage.fits", CPL_TYPE_UNSPECIFIED, NULL,
            CPL_IO_CREATE);
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_max( image_ill), 1.0, 0.0000001);
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_min( image_ill), 0.0, 0.0000001);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_PTR_NOT_NULL_FATAL(image_wav);
    CU_ASSERT_PTR_NOT_NULL_FATAL(image_id);
    CU_ASSERT_PTR_NOT_NULL_FATAL(image_dw);
    CU_ASSERT_PTR_NOT_NULL_FATAL(image_ill);
    cpl_image_delete(specidim);
    cpl_image_delete(image_wav);
    cpl_image_delete(image_dw);
    cpl_image_delete(image_ill);
    cpl_image_delete(image_id);
    sph_ifs_lenslet_model_delete(model);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_get_index function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_get_index(void) {
    sph_ifs_lenslet_model* model = NULL;
    int index = 0;

    /* Setup and run ...*/
    cpl_error_reset();

    model = sph_ifs_lenslet_model_new();

    CU_ASSERT_PTR_NOT_NULL_FATAL( model);
    index = sph_ifs_lenslet_model_get_index(model, 0, -145);
    CU_ASSERT_EQUAL( index, 1);
    index = sph_ifs_lenslet_model_get_index(model, 0, 145);
    CU_ASSERT_EQUAL( index, sph_ifs_lenslet_model_get_nlens( model ));

    sph_ifs_lenslet_model_delete(model);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_delete function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_delete(void) {
    sph_ifs_lenslet_model* model = NULL;
    int rerr = CPL_ERROR_NONE;

    /* Setup and run ...*/
    cpl_error_reset();

    model = sph_ifs_lenslet_model_new();

    CU_ASSERT_PTR_NOT_NULL_FATAL( model);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    /*Verify */
    CU_ASSERT_PTR_NOT_NULL_FATAL( model);
    sph_ifs_lenslet_model_delete(model);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_delete function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_set_lenslet_distortion(void) {
    sph_ifs_lenslet_model* model = NULL;
    int rerr = CPL_ERROR_NONE;
    sph_polynomial_distortion_model* dist = NULL;
    /* Setup and run ...*/
    cpl_error_reset();

    model = sph_ifs_lenslet_model_new();

    CU_ASSERT_PTR_NOT_NULL_FATAL( model);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    dist = sph_polynomial_distortion_model_new(NULL, NULL);
    sph_ifs_lenslet_model_set_lenslet_distortion(model, dist);

    /*Verify */
    CU_ASSERT_PTR_NOT_NULL_FATAL( model->lenslet_distortion);

    sph_ifs_lenslet_model_delete(model);
    //sph_polynomial_distortion_model_delete(dist);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_ifs_lenslet_model_delete function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_ifs_lenslet_model_set_get_hex_w_lenslet_distortion(void) {
    sph_ifs_lenslet_model* model_distorted = NULL;
    sph_ifs_lenslet_model* model_undistorted = NULL;
    int rerr = CPL_ERROR_NONE;
    sph_polynomial_distortion_model* dist = NULL;
    cpl_size pows[2];
    sph_polygon* hex = NULL;
    int u = 0;
    int v = 0;
    double x1 = 0.0;
    double x2 = 0.0;
    double y1 = 0.0;
    double y2 = 0.0;
    double dx = 0.0;
    double dy = 0.0;
    double cx0 = 0.0;
    double cy0 = 0.0;
    double cx1 = 0.01;
    double cy1 = 0.01;
    /* Setup and run ...*/
    cpl_error_reset();

    model_distorted = sph_ifs_lenslet_model_new();
    model_undistorted = sph_ifs_lenslet_model_new();

    CU_ASSERT_PTR_NOT_NULL_FATAL( model_distorted);
    CU_ASSERT_PTR_NOT_NULL_FATAL( model_undistorted);
    dist = sph_polynomial_distortion_model_new(NULL, NULL);
    pows[0] = 1;
    pows[1] = 0;
    cpl_polynomial_set_coeff(dist->polyx, pows, cx1);
    pows[0] = 0;
    pows[1] = 1;
    cpl_polynomial_set_coeff(dist->polyy, pows, cy1);
    pows[0] = 0;
    pows[1] = 0;
    cpl_polynomial_set_coeff(dist->polyx, pows, cx0);
    cpl_polynomial_set_coeff(dist->polyy, pows, cy0);
    sph_ifs_lenslet_model_set_lenslet_distortion(model_distorted, dist);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    u = 10;
    v = 10;
    hex = sph_ifs_lenslet_model_get_poly(model_distorted, u, v);
    x1 = sph_polygon_get_midx(hex);
    y1 = sph_polygon_get_midy(hex);
    sph_ifs_lenslet_model_get_centre(model_distorted, u, v, &x2, &y2);
    /*Verify */
    CU_ASSERT_DOUBLE_EQUAL(x1, x2, 0.0001);
    CU_ASSERT_DOUBLE_EQUAL(y1, y2, 0.0001);
    sph_ifs_lenslet_model_get_centre(model_undistorted, u, v, &x1, &y1);
    dx = x1 * cx1 + cx0;
    dy = y1 * cy1 + cy0;
    CU_ASSERT_DOUBLE_EQUAL(x1-x2, -dx, 0.0001);
    CU_ASSERT_DOUBLE_EQUAL(y1-y2, -dy, 0.0001);
    sph_polygon_delete(hex);
    hex = NULL;

    sph_ifs_lenslet_model_delete(model_distorted);
    sph_ifs_lenslet_model_delete(model_undistorted);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

static
int cutest_init_ifs_lenslet_model_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__, PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
    return 0;
}

static
int cutest_clean_ifs_lenslet_model_testsuite(void) {
    sph_error_dump(SPH_ERROR_ERROR);
    return sph_end_test();
}

/**@}*/
