/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_keyword_reader.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"
#include <float.h>
#include "sph_utils.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and 
 *                                  associated functions.               
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/

sph_keyword_reader*
cutest_util_keyword_reader_new_load(void) {
    sph_keyword_reader* reader = NULL;
    FILE* filehandle = NULL;

    filehandle = fopen("keylist_test.txt", "w");
    fprintf(filehandle, "#Table of keywords\n");
    fprintf(filehandle, "#TAG        KEYWORD            TYPE        VALUE\n");
    fprintf(
            filehandle,
            "TEST_TAG_A        STRING_KEYWORD        STRING        \"A VALUE\"\n");
    fprintf(filehandle,
            "TEST_TAG_B        \"INT KEYWORD\"        INT            234\n");
    fprintf(filehandle,
            "TEST_TAG_C        DOUBLE_KEYWORD        DOUBLE        112.991\n");
    fprintf(filehandle,
            "TEST_TAG_D        \"ESO DPR TYPE\"    STRING      DARK\n");
    fprintf(filehandle,
            "TEST_TAG_E        \"ESO NREADS\"      INT         12\n");
    fprintf(filehandle,
            "TEST_TAG_F        \"ESO DET DIT\"         DOUBLE      1.0\n");

    fclose(filehandle);

    reader = sph_keyword_reader_new_load("keylist_test.txt");
    return reader;
}

int cutest_init_keyword_reader_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__, PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
    return 0;
}

int cutest_clean_keyword_reader_testsuite(void) {
    sph_error_dump(SPH_ERROR_ERROR);
    return sph_end_test();
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_keyword_reader_new function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_keyword_reader_new(void) {
    sph_keyword_reader* reader = NULL;

    reader = sph_keyword_reader_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( reader);
    sph_keyword_reader_delete(reader);
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_keyword_reader_new function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_keyword_reader_new_load(void) {
    sph_keyword_reader* reader = NULL;

    reader = cutest_util_keyword_reader_new_load();

    CU_ASSERT_PTR_NOT_NULL_FATAL( reader);
    CU_ASSERT_EQUAL( cpl_table_get_nrow(reader->table ), 6);
    sph_keyword_reader_delete(reader);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_keyword_reader_create function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_keyword_reader_create(void) {
    CU_FAIL( "Not yet implemented!");
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_keyword_reader_get_value_string function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_keyword_reader_set_proplist_from_file(void) {
    cpl_frameset* frames = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    sph_keyword_reader* reader = NULL;

    reader = cutest_util_keyword_reader_new_load();
    CU_ASSERT_PTR_NOT_NULL_FATAL(reader);

    frames = cpl_frameset_new();

    rerr = sph_test_create_ird_raw_dark_frameset(frames, 1);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    rerr = sph_keyword_reader_set_proplist_from_file(reader, "temp_0.fits", 0);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    sph_keyword_reader_delete(reader);
    cpl_frameset_delete(frames);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_keyword_reader_get_value_string function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_keyword_reader_get_value_string(void) {
    cpl_frameset* frames = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    sph_keyword_reader* reader = NULL;
    char* result = NULL;
    int cmp = 0;

    reader = cutest_util_keyword_reader_new_load();
    CU_ASSERT_PTR_NOT_NULL_FATAL(reader);

    frames = cpl_frameset_new();

    rerr = sph_test_create_ird_raw_dark_frameset(frames, 1);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    rerr = sph_keyword_reader_set_proplist_from_file(reader, "temp_0.fits", 0);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    //result = sph_keyword_reader_get_value_string( reader, "TEST_TAG_A" );
    //CU_ASSERT_PTR_NOT_NULL_FATAL( result );
    //cmp = strncmp( result, "A VALUE", 6);
    //CU_ASSERT_EQUAL( cmp, 0 );
    //cpl_free( result );

    result = sph_keyword_reader_get_value_string(reader, "TEST_TAG_D");
    CU_ASSERT_PTR_NOT_NULL_FATAL( result);
    cmp = strncmp(result, "DARK", 4);
    CU_ASSERT_EQUAL( cmp, 0);
    cpl_free(result);

    sph_keyword_reader_delete(reader);
    cpl_frameset_delete(frames);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_keyword_reader_get_value_int function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_keyword_reader_get_value_int(void) {
    cpl_frameset* frames = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    sph_keyword_reader* reader = NULL;
    int result = 0;

    reader = cutest_util_keyword_reader_new_load();
    CU_ASSERT_PTR_NOT_NULL_FATAL(reader);

    frames = cpl_frameset_new();

    rerr = sph_test_create_ird_raw_dark_frameset(frames, 1);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    rerr = sph_keyword_reader_set_proplist_from_file(reader, "temp_0.fits", 0);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    result = sph_keyword_reader_get_value_int(reader, "TEST_TAG_E");
    CU_ASSERT_EQUAL( result, 1);

    cpl_frameset_delete(frames);
    sph_keyword_reader_delete(reader);

    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_keyword_reader_get_value_double function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_keyword_reader_get_value_double(void) {
    cpl_frameset* frames = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    sph_keyword_reader* reader = NULL;
    double result = 0.0;

    reader = cutest_util_keyword_reader_new_load();
    CU_ASSERT_PTR_NOT_NULL_FATAL(reader);

    frames = cpl_frameset_new();

    rerr = sph_test_create_ird_raw_dark_frameset(frames, 1);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    rerr = sph_keyword_reader_set_proplist_from_file(reader, "temp_0.fits", 0);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);

    result = sph_keyword_reader_get_value_double(reader,
            SPH_COMMON_KEYWORD_EXPTIME);
    CU_ASSERT_DOUBLE_EQUAL(result, 1.3, DBL_EPSILON);

    cpl_frameset_delete(frames);
    sph_keyword_reader_delete(reader);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_keyword_reader_delete function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_keyword_reader_delete(void) {
    sph_keyword_reader* reader = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;

    reader = sph_keyword_reader_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( reader);

    rerr = sph_keyword_reader_delete(reader);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("keyword_reader_test",
            cutest_init_keyword_reader_testsuite,
            cutest_clean_keyword_reader_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    if (NULL
            == CU_add_test(pSuite, "sph_keyword_reader_new",
                    cutest_keyword_reader_new)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    if (NULL
            == CU_add_test(pSuite, "sph_keyword_reader_new_load",
                    cutest_keyword_reader_new_load)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_keyword_reader_set_proplist_from_file",
                    cutest_keyword_reader_set_proplist_from_file)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_keyword_reader_get_values_int",
                    cutest_keyword_reader_get_value_int)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_keyword_reader_get_value_double",
                    cutest_keyword_reader_get_value_double)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    if (NULL
            == CU_add_test(pSuite, "sph_keyword_reader_get_value_string",
                    cutest_keyword_reader_get_value_string)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    if (NULL
            == CU_add_test(pSuite, "sph_keyword_reader_delete",
                    cutest_keyword_reader_delete)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
