/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_ldt.h"
#include "sph_dataset.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"
#include "sph_utils.h"
#include "sph_test_image_tools.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/
static sph_ldt* __ldt = NULL;
/**@{*/

static sph_ldt*
create_ldt_big(void) {
    sph_ifs_lenslet_model* model = NULL;
    cpl_image* inimage = NULL;
    sph_pixel_description_table* pdt = NULL;
    sph_ldt* ldt = NULL;
    int nx = 2048;
    int ny = 2048;
    int ll = 0;
    sph_lenslet_descriptor* ldescr = NULL;
    cpl_vector* vec = NULL;

    model = sph_ifs_lenslet_model_new();

    inimage = sph_test_image_tools_create_flat_image_double(nx, ny, 0.0);
    pdt = sph_pixel_description_table_new_from_model(model, 0.0, 0.0);
    ldt = sph_ldt_new_from_pdt_image(pdt, model, inimage);
    sph_pixel_description_table_delete(pdt);
    cpl_image_delete(inimage);
    vec = cpl_vector_duplicate(ldt->reflambdas);
    for (ll = 0; ll < ldt->nlens; ++ll) {
        ldescr = ldt->arr[ll];
        if (ldescr) {
            cpl_vector_fill(vec, 2.0);
            sph_lenslet_descriptor_fill_values(ldescr, vec);
            cpl_vector_fill(vec, 0.0);
            sph_lenslet_descriptor_fill_bpix(ldescr, vec);
            cpl_vector_fill(vec, 3.0);
            sph_lenslet_descriptor_fill_rms(ldescr, vec);
            cpl_vector_fill(vec, 4.0);
            sph_lenslet_descriptor_fill_ncomb(ldescr, vec);
        }
    }
    cpl_vector_delete(vec);
    vec = NULL;

    return ldt;
}

static sph_ldt*
create_ldt_standard(void) {
    sph_ifs_lenslet_model* model = NULL;
    cpl_image* inimage = NULL;
    sph_pixel_description_table* pdt = NULL;
    sph_ldt* ldt = NULL;
    int nx = 256;
    int ny = 256;
    int ll = 0;
    sph_lenslet_descriptor* ldescr = NULL;
    cpl_vector* vec = NULL;

    model = sph_test_create_small_lenslet_model();

    inimage = sph_test_image_tools_create_flat_image_double(nx, ny, 0.0);
    pdt = sph_pixel_description_table_new_from_model(model, 0.0, 0.0);
    ldt = sph_ldt_new_from_pdt_image(pdt, model, inimage);
    sph_pixel_description_table_delete(pdt);
    cpl_image_delete(inimage);
    vec = cpl_vector_duplicate(ldt->reflambdas);
    for (ll = 0; ll < ldt->nlens; ++ll) {
        ldescr = ldt->arr[ll];
        if (ldescr) {
            cpl_vector_fill(vec, 2.0);
            sph_lenslet_descriptor_fill_values(ldescr, vec);
            cpl_vector_fill(vec, 0.0);
            sph_lenslet_descriptor_fill_bpix(ldescr, vec);
            cpl_vector_fill(vec, 3.0);
            sph_lenslet_descriptor_fill_rms(ldescr, vec);
            cpl_vector_fill(vec, 4.0);
            sph_lenslet_descriptor_fill_ncomb(ldescr, vec);
        }
    }
    cpl_vector_delete(vec);
    vec = NULL;

    return ldt;
}

static
int cutest_init_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__, PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
    __ldt = create_ldt_standard();
    if (!__ldt)
        return 1;
    if (cpl_error_get_code())
        return 1;
    return 0;
}

static
int cutest_clean_testsuite(void) {
    if (__ldt)
        sph_ldt_delete(__ldt);
    __ldt = NULL;
    return sph_end_test();

    return 0;
}

static
void cutest_standard_happy_case(void) {
    sph_error_code rerr = CPL_ERROR_NONE;
    sph_master_frame* mplane = NULL;

    CU_ASSERT_PTR_NOT_NULL_FATAL(__ldt);
    rerr = sph_ldt_save_cube(__ldt, "acube.fits", NULL, NULL, NULL, NULL, NULL,
            NULL, NULL);
    CU_ASSERT_EQUAL(rerr, CPL_ERROR_NONE);
    mplane = sph_master_frame_load("acube.fits", 1);
    CU_ASSERT_PTR_NOT_NULL_FATAL(mplane);
    //CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_flux(mplane->image),
    //        __ldt->nlens * 1.0, 0.001);
    //printf("Flux: %f nlens: %d",cpl_image_get_flux(mplane->image),__ldt->nlens );
    sph_master_frame_delete(mplane);
}
#ifdef SPHERE_TEST_FULL
static
void cutest_big_happy_case(void) {
    sph_error_code rerr = CPL_ERROR_NONE;
    sph_ldt* ldt = create_ldt_big();
    sph_master_frame* mplane = NULL;
    sph_master_frame* mframe = NULL;

    CU_ASSERT_PTR_NOT_NULL_FATAL(ldt);
    rerr = sph_ldt_save_cube(ldt,"bigcube.fits");
    CU_ASSERT_EQUAL(rerr, CPL_ERROR_NONE);

    mplane = sph_master_frame_load("bigcube.fits",1);
    CU_ASSERT_PTR_NOT_NULL_FATAL(mplane);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_flux(mplane->image),
            ldt->nlens * 2.0, 0.1);
    mframe = sph_ldt_create_mframe(ldt,
            cpl_image_get_size_x(mplane->image),
            cpl_image_get_size_y(mplane->image),1,0.0,NULL);
    CU_ASSERT_PTR_NOT_NULL_FATAL(mframe);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_flux(mframe->image),
            ldt->nlens * 2.0, 0.001);

    sph_master_frame_subtract_master_frame(mplane,mframe);
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_flux(mplane->image),
            0.0, 0.1);
    sph_master_frame_save(mplane,"diffmframe.fits",NULL);

    sph_master_frame_delete(mplane);
    sph_master_frame_delete(mframe);
    sph_ldt_delete(ldt);
}
#endif
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("ldt_save_cube", cutest_init_testsuite,
            cutest_clean_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    if (NULL
            == CU_add_test(pSuite, "standard happy case",
                    cutest_standard_happy_case)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    /* add the tests to the suite */
#ifdef SPHERE_TEST_FULL
    if ( NULL == CU_add_test(pSuite, "big happy case",
                    cutest_big_happy_case ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
    }
#endif
    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
