/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>
#include <float.h>

#include "sph_point.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"
#include "sph_utils.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and 
 *                                  associated functions.               
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/
int cutest_init_point_testsuite(void) {
    return sph_init_test(__FILE__, PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
}

int cutest_clean_point_testsuite(void) {
    sph_error_dump(SPH_ERROR_ERROR);
    return sph_end_test();
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_point_new function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_point_new(void) {
    sph_point* p = NULL;

    p = sph_point_new(-121233.1212, -1234245.4578);

    CU_ASSERT_PTR_NOT_NULL_FATAL( p);

    CU_ASSERT_DOUBLE_EQUAL( p->x, -121233.1212, 121233 * DBL_EPSILON);
    CU_ASSERT_DOUBLE_EQUAL( p->y, -1234245.4578, 1234245 * DBL_EPSILON);

    sph_point_delete(p);

    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_point_new function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_point_ccw_test(void) {
    sph_point* p1 = NULL;
    sph_point* p2 = NULL;
    sph_point* p3 = NULL;
    sph_point* p4 = NULL;
    sph_point* p5 = NULL;
    sph_point* p6 = NULL;
    int result = 0;

    p1 = sph_point_new(0.0, 0.0);
    p2 = sph_point_new(1.0, 0.0);
    p3 = sph_point_new(0.0, 1.0);
    p4 = sph_point_new(0.0, 0.0);
    p5 = sph_point_new(-1.0, 0.0);
    p6 = sph_point_new(0.0, 2.0);

    CU_ASSERT_PTR_NOT_NULL_FATAL( p1);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p2);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p3);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p4);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p5);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p6);

    result = sph_point_ccw_test(*p1, *p2, *p3);
    CU_ASSERT_EQUAL( result, 1);
    result = sph_point_ccw_test(*p1, *p3, *p2);
    CU_ASSERT_EQUAL( result, 0);
    result = sph_point_ccw_test(*p1, *p3, *p6);
    CU_ASSERT_EQUAL( result, 0);
    result = sph_point_ccw_test(*p1, *p4, *p1);
    CU_ASSERT_EQUAL( result, 0);
    result = sph_point_ccw_test(*p1, *p5, *p3);
    CU_ASSERT_EQUAL( result, 0);

    sph_point_delete(p1);
    sph_point_delete(p2);
    sph_point_delete(p3);
    sph_point_delete(p4);
    sph_point_delete(p5);
    sph_point_delete(p6);

    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_point_rotate_around function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_point_rotate_around(void) {
    sph_point* p1 = NULL;
    sph_point* p2 = NULL;
    sph_point* p3 = NULL;
    sph_point* p4 = NULL;
    sph_point* p5 = NULL;
    sph_point* p6 = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    int ii = 40;

    p1 = sph_point_new(0.0, 0.0);
    p2 = sph_point_new(1.0, 0.0);
    p3 = sph_point_new(0.0, 1.0);
    p4 = sph_point_new(0.0, 0.0);
    p5 = sph_point_new(-1.0, 0.0);
    p6 = sph_point_new(0.0, 2.0);

    CU_ASSERT_PTR_NOT_NULL_FATAL( p1);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p2);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p3);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p4);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p5);
    CU_ASSERT_PTR_NOT_NULL_FATAL( p6);

    rerr = sph_point_rotate_around(p1, 0.0, 0.0, 0.12, -0.145, 0.0, 0.0);
    CU_ASSERT_EQUAL_FATAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL( p1->x, 0.0, 1.0e-15);
    CU_ASSERT_DOUBLE_EQUAL( p1->y, 0.0, 1.0e-15);
    rerr = sph_point_rotate_around(p5, -1.0, 0.0, 0.12, -0.145, 0.0, 0.0);
    CU_ASSERT_EQUAL_FATAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL( p5->x, -1.0, 1.0e-15);
    CU_ASSERT_DOUBLE_EQUAL( p5->y, 0.0, 1.0e-15);
    rerr = sph_point_rotate_around(p1, 0.0, 0.0, 0.12, -0.145, 12.0, -4.0);
    CU_ASSERT_EQUAL_FATAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL( p1->x, 12.0, 1.0e-15);
    CU_ASSERT_DOUBLE_EQUAL( p1->y, -4.0, 1.0e-15);
    rerr = sph_point_rotate_around(p3, 0.0, 0.0, cos(SPH_PI / 2.0),
            sin(SPH_PI / 2.0), 1.0, 0.0);
    CU_ASSERT_EQUAL_FATAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL( p3->x, 0.0, 1.0e-15);
    CU_ASSERT_DOUBLE_EQUAL( p3->y, 0.0, 1.0e-15);
    for (ii = 0; ii < 40; ++ii) {
        rerr |= sph_point_rotate_around(p6, 0.0, 0.0, cos(SPH_PI / 2.0),
                sin(SPH_PI / 2.0), 1.0, 0.0);
    }CU_ASSERT_EQUAL_FATAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL( p6->x, 0.0, 1.0e-14);
    CU_ASSERT_DOUBLE_EQUAL( p6->y, 2.0, 1.0e-14);

    sph_point_delete(p1);
    sph_point_delete(p2);
    sph_point_delete(p3);
    sph_point_delete(p4);
    sph_point_delete(p5);
    sph_point_delete(p6);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_point_delete function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_point_delete(void) {
    sph_point* p = NULL;
    p = sph_point_new(-121233.1212, -1234245.4578);

    CU_ASSERT_PTR_NOT_NULL_FATAL( p);

    sph_point_delete(p);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(void) {

    int result = 0;
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("point_test", cutest_init_point_testsuite,
            cutest_clean_point_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    if (NULL == CU_add_test(pSuite, "sph_point_new", cutest_point_new)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_point_rotate_around",
                    cutest_point_rotate_around)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_point_ccw_test", cutest_point_ccw_test)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL == CU_add_test(pSuite, "sph_point_delete", cutest_point_delete)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
