/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_smart_imagelist.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>
#include <gsl/gsl_rng.h>
#include "sph_cube.h"
#include "sph_utils.h"
#include "sph_test_image_tools.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup cutest_sph_smart_imagelist Unit test for the sph_smart_imagelist
 *                  module            
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/

int cutest_init_sph_smart_imagelist_testsuite(void) {
    sph_init_test(__FILE__, "moeller@mpia.de", CPL_MSG_DEBUG);
    return 0;
}

int cutest_clean_sph_smart_imagelist_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Close and dump errors.
     * -------------------------------------------------------------------*/
    sph_end_test();
    return 0;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_smart_imagelist_new function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_smart_imagelist_new(void) {
    CU_FAIL( "Not yet implemented!");
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_smart_imagelist_create function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_smart_imagelist_create(void) {
    CU_FAIL( "Not yet implemented!");
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_smart_imagelist_create_cube function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_smart_imagelist_create_cube(void) {
    sph_master_frame* mframe = NULL;
    sph_cube* cube = NULL;
    int nframes = 2;
    int ii = 0;
    sph_smart_imagelist* slist = NULL;

    cube = sph_cube_new("testcube.fits");
    CU_ASSERT_PTR_NOT_NULL_FATAL( cube);
    slist = sph_smart_imagelist_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( slist);
    slist->maxmem = 50000000; /* Set max to 50MB */
    for (ii = 0; ii < nframes; ++ii) {
        mframe = sph_master_frame_new_empty();
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe);
        mframe->image = cpl_image_new(2048, 2048, CPL_TYPE_DOUBLE);
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->image);
        mframe->badpixelmap = cpl_image_new(2048, 2048, CPL_TYPE_INT);
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->badpixelmap);
        mframe->rmsmap = cpl_image_new(2048, 2048, CPL_TYPE_DOUBLE);
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->rmsmap);
        mframe->ncombmap = cpl_image_new(2048, 2048, CPL_TYPE_DOUBLE);
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->ncombmap);
        cpl_image_fill_gaussian(mframe->image, 1024, 1024, 2.0 * ii, 100.0 * ii,
                100.0 * ii);
        sph_cube_append_master(cube, mframe, ii);
        sph_master_frame_delete(mframe);
    }
    sph_cube_finalise_file(cube->filename);
    slist = sph_smart_imagelist_create_cube(slist, cube, SPH_CUBE_IMAGE_EXT);
    CU_ASSERT_PTR_NOT_NULL_FATAL( slist);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    sph_smart_imagelist_delete(slist);
    sph_cube_delete(cube);
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_smart_imagelist_get_set function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_smart_imagelist_get_set(void) {
    CU_FAIL( "Not yet implemented!");
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_smart_imagelist_get_set_cube function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_smart_imagelist_get_set_cube(void) {
    sph_master_frame* mframe = NULL;
    sph_cube* cube = NULL;
    sph_cube* cubeout = NULL;
    int nframes = 2;
    int ii = 0;
    cpl_imagelist* imlist = NULL;
    sph_smart_imagelist* slist = NULL;
    int cc = 0;
    double zzval = 0;

    cube = sph_cube_new("testcube.fits");
    CU_ASSERT_PTR_NOT_NULL_FATAL( cube);
    slist = sph_smart_imagelist_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( slist);
    slist->maxmem = 50000000; /* Set max to 50MB */

    for (ii = 0; ii < nframes; ++ii) {
        mframe = sph_master_frame_new_empty();
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe);
        mframe->image = cpl_image_new(2048, 2048, CPL_TYPE_DOUBLE);
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->image);
        mframe->badpixelmap = cpl_image_new(2048, 2048, CPL_TYPE_INT);
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->badpixelmap);
        mframe->rmsmap = cpl_image_new(2048, 2048, CPL_TYPE_DOUBLE);
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->rmsmap);
        mframe->ncombmap = cpl_image_new(2048, 2048, CPL_TYPE_DOUBLE);
        CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->ncombmap);
        cpl_image_fill_gaussian(mframe->image, 1024, 1024, 2.0 * ii, 100.0 * ii,
                100.0 * ii);
        sph_cube_append_master(cube, mframe, ii);
        sph_master_frame_delete(mframe);
    }
    sph_cube_finalise_file(cube->filename);

    slist = sph_smart_imagelist_create_cube(slist, cube, SPH_CUBE_IMAGE_EXT);
    CU_ASSERT_PTR_NOT_NULL_FATAL( slist);
    imlist = sph_smart_imagelist_get_set_cube(slist, 0);
    while (imlist) {
        cubeout = sph_cube_new("cubout.fits");
        CU_ASSERT_PTR_NOT_NULL_FATAL( cubeout);
        for (ii = 0; ii < cpl_imagelist_get_size(imlist); ++ii) {
            zzval = ii * 1.0;
            mframe = sph_master_frame_new_empty();
            CU_ASSERT_PTR_NOT_NULL_FATAL( mframe);
            mframe->image = cpl_image_duplicate(
                    cpl_imagelist_get_const(imlist, ii));
            CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->image);
            mframe->badpixelmap = cpl_image_new(
                    cpl_image_get_size_x(mframe->image),
                    cpl_image_get_size_y(mframe->image), CPL_TYPE_INT);
            CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->badpixelmap);
            mframe->rmsmap = cpl_image_new(cpl_image_get_size_x(mframe->image),
                    cpl_image_get_size_y(mframe->image), CPL_TYPE_DOUBLE);
            CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->rmsmap);
            mframe->ncombmap = cpl_image_new(
                    cpl_image_get_size_x(mframe->image),
                    cpl_image_get_size_y(mframe->image), CPL_TYPE_DOUBLE);
            CU_ASSERT_PTR_NOT_NULL_FATAL( mframe->ncombmap);
            sph_cube_append_master(cubeout, mframe, ii);
            sph_master_frame_delete(mframe);

            cpl_imagelist_delete(imlist);
            imlist = NULL;
            cc++;
            if (cc < slist->nregions) {
                imlist = sph_smart_imagelist_get_set_cube(slist, cc);
            } else {
                imlist = NULL;
            }
        }
        sph_cube_finalise_file(cubeout->filename);
        sph_cube_delete(cubeout);
    }
    if (!imlist) {
        cpl_error_reset();
    }CU_ASSERT_TRUE( cc > 1);
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    sph_smart_imagelist_delete(slist);
    sph_cube_delete(cube);
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_smart_imagelist_reassemble_master_frame function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_smart_imagelist_get_raw_median_properties(void) {
    cpl_frameset* frameset = NULL;
    cpl_frame* aframe = NULL;
    sph_smart_imagelist* slist = NULL;
    cpl_image* im = NULL;
    int ii = 0;
    gsl_rng* pRNG = NULL;
    cpl_propertylist* plist = NULL;

    frameset = cpl_frameset_new();

    pRNG = gsl_rng_alloc(gsl_rng_taus);

    for (ii = 0; ii < 11; ++ii) {
        im = sph_test_image_tools_create_flat_image_double(256, 256,
                ii * 1000.0 + 1000.0);
        CU_ASSERT_PTR_NOT_NULL_FATAL(im);
        sph_test_image_tools_apply_poisson_noise(im, pRNG);
        aframe = sph_filemanager_create_temp_frame("test_smart_imlist",
                "TESTDATA", CPL_FRAME_GROUP_RAW);
        cpl_image_save(im, cpl_frame_get_filename(aframe), CPL_TYPE_FLOAT, NULL,
                CPL_IO_CREATE);
        cpl_frameset_insert(frameset, aframe);
        cpl_image_delete(im);
        im = NULL;
    }

    slist = sph_smart_imagelist_create(frameset, 0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(slist);
    plist = cpl_propertylist_new();
    sph_smart_imagelist_get_raw_median_properties(slist, plist);
    CU_ASSERT( cpl_propertylist_get_size(plist) == 5);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_AVG),
            6000.0, 100.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_MED),
            6000.0, 10.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_MIN),
            1000.0, 10.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_MAX),
            11000.0, 10.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_RANGE),
            3316.0, 10.0);

    im = sph_test_image_tools_create_flat_image_double(256, 256, 111.0);
    aframe = sph_filemanager_create_temp_frame("test_smart_imlist_result",
            "TESTDATA", CPL_FRAME_GROUP_RAW);
    cpl_image_save(im, cpl_frame_get_filename(aframe), CPL_TYPE_FLOAT, plist,
            CPL_IO_CREATE);
    sph_smart_imagelist_delete(slist);
    cpl_frame_delete(aframe);
    cpl_frameset_delete(frameset);
    cpl_image_delete(im);
    im = NULL;
    cpl_propertylist_delete(plist);
    gsl_rng_free(pRNG);
    //sph_filemanager_clean();
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_smart_imagelist_reassemble_master_frame function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_smart_imagelist_get_raw_median_properties_cube(void) {
    cpl_imagelist* imlist = NULL;
    cpl_frameset* frameset = NULL;
    cpl_frame* aframe = NULL;
    sph_smart_imagelist* slist = NULL;
    cpl_image* im = NULL;
    int ii = 0;
    gsl_rng* pRNG = NULL;
    cpl_propertylist* plist = NULL;

    frameset = cpl_frameset_new();
    imlist = cpl_imagelist_new();

    pRNG = gsl_rng_alloc(gsl_rng_taus);

    for (ii = 0; ii < 11; ++ii) {
        im = sph_test_image_tools_create_flat_image_double(256, 256,
                ii * 1000.0 + 1000.0);
        CU_ASSERT_PTR_NOT_NULL_FATAL(im);
        sph_test_image_tools_apply_poisson_noise(im, pRNG);
        cpl_imagelist_set(imlist, im, ii);
    }

    aframe = sph_filemanager_create_temp_frame("test_smart_imlist_cube",
            "TESTDATA", CPL_FRAME_GROUP_RAW);
    cpl_imagelist_save(imlist, cpl_frame_get_filename(aframe), CPL_TYPE_FLOAT,
            NULL, CPL_IO_CREATE);
    cpl_frameset_insert(frameset, aframe);
    slist = sph_smart_imagelist_create(frameset, 0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(slist);

    plist = cpl_propertylist_new();
    sph_smart_imagelist_get_raw_median_properties(slist, plist);
    CU_ASSERT( cpl_propertylist_get_size(plist) == 5);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_AVG),
            6000.0, 100.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_MED),
            6000.0, 10.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_MIN),
            1000.0, 10.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_MAX),
            11000.0, 10.0);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double(plist,SPH_COMMON_KEYWORD_RAW_MEDIANS_RANGE),
            3316.0, 10.0);

    cpl_imagelist_save(imlist, cpl_frame_get_filename(aframe), CPL_TYPE_FLOAT,
            plist, CPL_IO_CREATE);
    sph_smart_imagelist_delete(slist);
    cpl_imagelist_delete(imlist);
    cpl_frameset_delete(frameset);
    cpl_propertylist_delete(plist);
    gsl_rng_free(pRNG);
    //sph_filemanager_clean();
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_smart_imagelist_reassemble_master_frame function.
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_smart_imagelist_reassemble_master_frame(void) {
    CU_FAIL( "Not yet implemented!");
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_smart_imagelist_delete function. 
 */
/*----------------------------------------------------------------------------*/
void cutest_sph_smart_imagelist_delete(void) {
    CU_FAIL( "Not yet implemented!");
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief   Main to run the tests.
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("cutest_sph_smart_imagelist",
            cutest_init_sph_smart_imagelist_testsuite,
            cutest_clean_sph_smart_imagelist_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    if (NULL
            == CU_add_test(pSuite, "sph_smart_imagelist_create_cube",
                    cutest_sph_smart_imagelist_create_cube)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "sph_smart_imagelist_get_raw_median_properties",
                    cutest_sph_smart_imagelist_get_raw_median_properties)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite,
                    "sph_smart_imagelist_get_raw_median_properties_cube",
                    cutest_sph_smart_imagelist_get_raw_median_properties_cube)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_smart_imagelist_get_set_cube",
                    cutest_sph_smart_imagelist_get_set_cube)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
