/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_utils.h"
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_zpl_simulator.h"
#include "sph_test_image_tools.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_test_image_tools.h"
#include "sph_time.h"
static
void cutest_sph_time_test_it(void);

static
void cutest_sph_time_test_get_times(void);

static
int cutest_sph_time_clean_testsuite(void);

static
int cutest_sph_time_init_testsuite(void);

/*----------------------------------------------------------------------------*/
/**
 * @defgroup A CUnit Test Suite -- representing a collection of testcases
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(void) {
    int result = 0;
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("A cutest_sph_time testsuite",
            cutest_sph_time_init_testsuite, cutest_sph_time_clean_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */
    if (NULL
            == CU_add_test(pSuite, "Testing function", cutest_sph_time_test_it)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    if (NULL
            == CU_add_test(pSuite, "Testing function: sph_time_test_get_times",
                    cutest_sph_time_test_get_times)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_time_test_it(void) {
    return;
}

static
void cutest_sph_time_test_get_times(void) {
    double mjd_start = 0.0;
    double mjd_stop = 0.0;
    double mjd_frame_ut_et = 0.0;
    const char* filename = "sph_time_get_times.fits";
    double mjd_obs_et = 56799.00106531; // 2014-05-22T00:01:32.0429
    const char* frame_ut_string_et = "2014-05-22T00:04:57.0520";

    sph_time_mjd_from_string(&mjd_frame_ut_et, frame_ut_string_et);

    cpl_frame* inframe = NULL;

    inframe = sph_test_zpl_simulator_create_double_image_frame(filename, 10, 10,
            5.0, 6.0);
    sph_time_get_times(inframe, &mjd_start, &mjd_stop);

    CU_ASSERT_DOUBLE_EQUAL( mjd_start, mjd_obs_et, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL( mjd_stop, mjd_frame_ut_et, 1e-10);
    //printf("start: %f\n", mjd_start);
    //printf("stop: %f\n", mjd_stop);
    //printf("stop_et: %f\n", mjd_obs_et);

    cpl_frame_delete(inframe);

    return;
}

/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_sph_time_init_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__, PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_sph_time_clean_testsuite(void) {
    sph_error_dump(SPH_ERROR_ERROR);
    return sph_end_test();
}
/**@}*/
