/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_utils.h"
#include "sph_master_frame.h"
#include "sph_double_image.h"
#include "sph_common_keywords.h"
#include "sph_keyword_manager.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>

/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/
int cutest_init_utils_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__, PACKAGE_BUGREPORT, CPL_MSG_DEBUG);
    unlink("sphere.cfg");
    return 0;
}

int cutest_clean_utils_testsuite(void) {
    sph_error_dump(SPH_ERROR_ERROR);
    return sph_end_test();
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_utils_create_master_frameset function.
 */
/*----------------------------------------------------------------------------*/
void cutest_utils_create_master_frameset(void) {
    cpl_frameset* inframes = NULL;
    cpl_frameset* newframes = NULL;
    sph_error_code rerr = CPL_ERROR_NONE;
    FILE* fp = NULL;

    fp = fopen("sph_utils_create_master_frameset_web.txt", "w");
    fprintf(fp, "<table><tr><td>Test passed</td></tr></table>\n");
    /* Setup... */
    inframes = cpl_frameset_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( inframes);
    rerr = sph_test_create_ifs_raw_dark_frameset(inframes, 2);
    CU_ASSERT_EQUAL_FATAL( rerr, CPL_ERROR_NONE);

    /* Run ... */
    newframes = sph_utils_create_master_frameset(inframes);

    /* Verfiy */

    CU_ASSERT_PTR_NOT_NULL( newframes);

    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    fclose(fp);
    cpl_frameset_delete(inframes);
    cpl_frameset_delete(newframes);

    return;
}

/*
 **/
/*----------------------------------------------------------------------------*/
void cutest_utils_attach_plist(void) {
    cpl_propertylist* plist = NULL;
    cpl_propertylist* compplist = NULL;
    sph_master_frame* mframe = NULL;
    cpl_frame* aframe = NULL;

    plist = cpl_propertylist_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL(plist);
    mframe = sph_master_frame_new(100, 100);
    CU_ASSERT_PTR_NOT_NULL_FATAL(mframe);
    aframe = cpl_frame_new();
    cpl_frame_set_tag(aframe, "TESTFRAME");
    cpl_frame_set_filename(aframe, "test_attach_plist.fits");

    sph_master_frame_save(mframe, cpl_frame_get_filename(aframe), plist);

    compplist = sph_keyword_manager_load_properties(
            cpl_frame_get_filename(aframe), 0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( compplist);

    CU_ASSERT_EQUAL( cpl_propertylist_has( compplist, "ESO TESTIT" ), 0);

    cpl_propertylist_append_double(plist, "ESO TESTIT", 10.0);
    sph_utils_attach_plist(plist, aframe, 0);
    cpl_propertylist_delete(compplist);
    compplist = NULL;
    compplist = sph_keyword_manager_load_properties(
            cpl_frame_get_filename(aframe), 0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( compplist);
    CU_ASSERT_EQUAL( cpl_propertylist_has( compplist, "ESO TESTIT" ), 1);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_propertylist_get_double( compplist, "ESO TESTIT" ), 10.0, 0.01);
    cpl_propertylist_delete(compplist);
    compplist = NULL;
    sph_master_frame_delete(mframe);
    cpl_propertylist_delete(plist);
    plist = NULL;
    cpl_frame_delete(aframe);
}

/*----------------------------------------------------------------------------*/
/**
 * @brief Unit test for the sph_utils_cpl_image_fill_double
 */
/*----------------------------------------------------------------------------*/
void cutest_utils_cpl_image_fill_double(void) {
    sph_error_code rerr = CPL_ERROR_NONE;
    cpl_image* image = NULL;
    double dvalue = 10.0;

    image = cpl_image_new(10, 10, CPL_TYPE_DOUBLE);
    rerr = sph_utils_cpl_image_fill_double(image, dvalue);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_mean( image ), 10.0, 1e-20);
    //cpl_image_save( image, "test_fill_double.fits.tmp", CPL_TYPE_FLOAT, NULL, CPL_IO_DEFAULT );
    cpl_image_delete(image);
}

/*----------------------------------------------------------------------------*/
/**
 * @brief Unit test for the sph_utils_cpl_image_fill_double
 */
/*----------------------------------------------------------------------------*/
void cutest_utils_update_key(void) {
    int i = 10;
    int j = 5;
    cpl_propertylist* plist = NULL;

    plist = cpl_propertylist_new();
    CU_ASSERT_PTR_NOT_NULL(plist);
    CU_ASSERT_EQUAL(
            sph_utils_update_key(plist, "MYKEYINTi",CPL_TYPE_INT,5,-1,222),
            CPL_ERROR_NONE);
    CU_ASSERT_EQUAL(
            sph_utils_update_key( plist,"MYKEYDBLi_j",CPL_TYPE_DOUBLE,5,2,2.22),
            CPL_ERROR_NONE);
    cpl_propertylist_save(plist, "myfile.fits", CPL_IO_CREATE);
    cpl_propertylist_delete(plist);
}

/*----------------------------------------------------------------------------*/
/**
 * @brief Unit test for the sph_utils_cpl_image_fill_double
 */
/*----------------------------------------------------------------------------*/
void cutest_utils_update_key_vector(void) {
    int i = 10;
    int j = 5;
    cpl_propertylist* plist = NULL;
    cpl_vector* vec = NULL;

    plist = cpl_propertylist_new();
    vec = cpl_vector_new(10);
    cpl_vector_fill(vec, 1.0);
    CU_ASSERT_PTR_NOT_NULL(plist);
    CU_ASSERT_PTR_NOT_NULL(vec);
    CU_ASSERT_EQUAL( sph_utils_update_key_vector(plist, "MYKEYINTi",vec),
            CPL_ERROR_NONE);
    cpl_propertylist_save(plist, "myfile.fits", CPL_IO_CREATE);
    cpl_propertylist_delete(plist);
    cpl_vector_delete(vec);
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(void) {

    int result = 0;
    CU_pSuite pSuite = NULL;
    unlink("sphere.cfg ");

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
        return CU_get_error();

    /* add a suite to the registry */
    pSuite = CU_add_suite("sph_utils_test", cutest_init_utils_testsuite,
            cutest_clean_utils_testsuite);
    if (NULL == pSuite) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* add the tests to the suite */

    if (NULL
            == CU_add_test(pSuite, "cutest_utils_cpl_image_fill_double",
                    cutest_utils_cpl_image_fill_double)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    if (NULL
            == CU_add_test(pSuite, "sph_utils_create_master_frameset",
                    cutest_utils_create_master_frameset)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_utils_attach_plist",
                    cutest_utils_attach_plist)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_utils_update_key",
                    cutest_utils_update_key)) {
        CU_cleanup_registry();
        return CU_get_error();
    }
    if (NULL
            == CU_add_test(pSuite, "sph_utils_update_key_vector",
                    cutest_utils_update_key_vector)) {
        CU_cleanup_registry();
        return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode(CU_BRM_VERBOSE);
    CU_basic_run_tests();
    sph_test_create_results_file("results.txt");
    result = sph_test_check_result();
    CU_cleanup_registry();
    unlink("sphere.cfg");
    return result;
}

/**@}*/
