##### http://autoconf-archive.cryp.to/ax_boost.html
#
# SYNOPSIS
#
#   AX_BOOST([MINIMUM-VERSION], [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
#
# DESCRIPTION
#
#   Test for the Boost C++ libraries of a particular version (or newer)
#
#   If no path to the installed boost library is given the macro
#   searchs under /usr, /usr/local, and /opt, and evaluates the
#   $BOOST_ROOT environment variable. Further documentation is
#   available at <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_CPPFLAGS) / AC_SUBST(BOOST_LDFLAGS)
#     AC_SUBST(BOOST_FILESYSTEM_LIB)
#     AC_SUBST(BOOST_PROGRAM_OPTIONS_LIB)
#     AC_SUBST(BOOST_THREAD_LIB)
#     AC_SUBST(BOOST_IOSTREAMS_LIB)
#     AC_SUBST(BOOST_SERIALIZATION_LIB)
#     AC_SUBST(BOOST_WSERIALIZATION_LIB)
#     AC_SUBST(BOOST_SIGNALS_LIB)
#     AC_SUBST(BOOST_DATE_TIME_LIB)
#     AC_SUBST(BOOST_REGEX_LIB)
#     AC_SUBST(BOOST_UNIT_TEST_FRAMEWORK_LIB)
#
#   And sets:
#
#     HAVE_BOOST
#     HAVE_BOOST_FILESYSTEM
#     HAVE_BOOST_PROGRAM_OPTIONS
#     HAVE_BOOST_THREAD
#     HAVE_BOOST_IOSTREAMS
#     HAVE_BOOST_SERIALIZATION
#     HAVE_BOOST_SIGNALS
#     HAVE_BOOST_DATE_TIME
#     HAVE_BOOST_REGEX
#     HAVE_BOOST_UNIT_TEST_FRAMEWORK
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST],
[
    AC_ARG_WITH([boost],
                AS_HELP_STRING([--with-boost=DIR],
                [use boost (default is NO) specify the root directory for boost library (optional)]),
                [
                if test "$withval" = "no"; then
                  want_boost="no"
                elif test "$withval" = "yes"; then
                    want_boost="yes"
                    ac_boost_path=""
                else
                 want_boost="yes"
                  ac_boost_path="$withval"
              fi
               ],
                [want_boost="yes"])

    AC_CANONICAL_BUILD
   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      boost_lib_version_req=ifelse([$1], ,1.20.0,$1)
      boost_lib_version_req_shorten=`expr $boost_lib_version_req : '\([[0-9]]*\.[[0-9]]*\)'`
      boost_lib_version_req_major=`expr $boost_lib_version_req : '\([[0-9]]*\)'`
      boost_lib_version_req_minor=`expr $boost_lib_version_req : '[[0-9]]*\.\([[0-9]]*\)'`
      boost_lib_version_req_sub_minor=`expr $boost_lib_version_req : '[[0-9]]*\.[[0-9]]*\.\([[0-9]]*\)'`
      if test "x$boost_lib_version_req_sub_minor" = "x" ; then
         boost_lib_version_req_sub_minor="0"
       fi
      WANT_BOOST_VERSION=`expr $boost_lib_version_req_major \* 100000 \+  $boost_lib_version_req_minor \* 100 \+ $boost_lib_version_req_sub_minor`
      AC_MSG_CHECKING(for boostlib >= $boost_lib_version_req)
      succeeded=no

      dnl first we check the system location for boost libraries
      dnl this location ist chosen if boost libraries are installed with the --layout=system option
      dnl or if you install boost with RPM
      if test "$ac_boost_path" != ""; then
         BOOST_LDFLAGS="-L$ac_boost_path/lib"
         BOOST_CPPFLAGS="-I$ac_boost_path/include"
      else
         for ac_boost_path_tmp in $INSROOT /usr /usr/local /opt /opt/local; do
            if test -d "$ac_boost_path_tmp/include/boost" && test -r "$ac_boost_path_tmp/include/boost"; then
               BOOST_LDFLAGS="-L$ac_boost_path_tmp/lib"
               BOOST_CPPFLAGS="-I$ac_boost_path_tmp/include"
               break;
            fi
         done
      fi
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

   AC_LANG_PUSH(C++)
        AC_COMPILE_IFELSE([AC_LANG_PROGRAM([[
@%:@include <boost/version.hpp>
]],
       [[
#if BOOST_VERSION >= $WANT_BOOST_VERSION
// Everything is okay
#else
#  error Boost version is too old
#endif

      ]])],
       [
         AC_MSG_RESULT(yes)
       succeeded=yes
       found_system=yes
         ifelse([$2], , :, [$2])
       ],
       [
       ])
       AC_LANG_POP([C++])
      dnl if we found no boost with system layout we search for boost libraries
      dnl built and installed without the --layout=system option or for a staged(not installed) version
      if test "x$succeeded" != "xyes"; then
         _version=0
         if test "$ac_boost_path" != ""; then
                      BOOST_LDFLAGS="-L$ac_boost_path/lib"
            if test -d "$ac_boost_path" && test -r "$ac_boost_path"; then
               for i in `ls -d $ac_boost_path/include/boost-* 2>/dev/null`; do
                  _version_tmp=`echo $i | sed "s#$ac_boost_path##" | sed 's/\/include\/boost-//' | sed 's/_/./'`
                  V_CHECK=`expr $_version_tmp \> $_version`
                  if test "$V_CHECK" = "1" ; then
                     _version=$_version_tmp
                  fi
                  VERSION_UNDERSCORE=`echo $_version | sed 's/\./_/'`
                  BOOST_CPPFLAGS="-I$ac_boost_path/include/boost-$VERSION_UNDERSCORE"
               done
            fi
         else
            for ac_boost_path in $INSROOT /usr /usr/local /opt /opt/local ; do
               if test -d "$ac_boost_path" && test -r "$ac_boost_path"; then
                  for i in `ls -d $ac_boost_path/include/boost-* 2>/dev/null`; do
                     _version_tmp=`echo $i | sed "s#$ac_boost_path##" | sed 's/\/include\/boost-//' | sed 's/_/./'`
                     V_CHECK=`expr $_version_tmp \> $_version`
                     if test "$V_CHECK" = "1" ; then
                        _version=$_version_tmp
                        best_path=$ac_boost_path
                     fi
                  done
               fi
            done

            VERSION_UNDERSCORE=`echo $_version | sed 's/\./_/'`
            BOOST_CPPFLAGS="-I$best_path/include/boost-$VERSION_UNDERSCORE"
            BOOST_LDFLAGS="-L$best_path/lib"

             if test "x$BOOST_ROOT" != "x"; then
                    if test -d "$BOOST_ROOT" && test -r "$BOOST_ROOT" && test -d "$BOOST_ROOT/stage/lib" && test -r "$BOOST_ROOT/stage/lib"; then
                  version_dir=`expr //$BOOST_ROOT : '.*/\(.*\)'`
                  stage_version=`echo $version_dir | sed 's/boost_//' | sed 's/_/./g'`
                  stage_version_shorten=`expr $stage_version : '\([[0-9]]*\.[[0-9]]*\)'`
                  V_CHECK=`expr $stage_version_shorten \>\= $_version`
                  if test "$V_CHECK" = "1" ; then
                     AC_MSG_NOTICE(We will use a staged boost library from $BOOST_ROOT)
                     BOOST_CPPFLAGS="-I$BOOST_ROOT"
                     BOOST_LDFLAGS="-L$BOOST_ROOT/stage/lib"
                  fi
               fi
             fi
         fi

         CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
         export CPPFLAGS
         LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
         export LDFLAGS

            AC_LANG_PUSH(C++)
            AC_COMPILE_IFELSE([AC_LANG_PROGRAM([[
@%:@include <boost/version.hpp>
]],
       [[
#if BOOST_VERSION >= $WANT_BOOST_VERSION
// Everything is okay
#else
#  error Boost version is too old
#endif

      ]])],
       [
         AC_MSG_RESULT(yes ($_version))
       succeeded=yes
         ifelse([$2], , :, [$2])
       ],
       [
         AC_MSG_RESULT(no ($_version))
         ifelse([$3], , :, [$3])
       ])
       AC_LANG_POP([C++])
      fi

      if test "$succeeded" != "yes" ; then
         if test "$_version" = "0" ; then
            AC_MSG_ERROR([[We could not detect the boost libraries (version $boost_lib_version_req_shorten or higher). If you have a staged boost library (still not installed) please specify \$BOOST_ROOT in your environment and do not give a PATH to --with-boost option.  If you are sure you have boost installed, then check your version number looking in <boost/version.hpp>. See http://randspringer.de/boost for more documentation.]])
         else
            AC_MSG_ERROR('Your boost libraries seems to old (version $_version).  We need at least $boost_lib_version_shorten')
         fi
      else
         AC_SUBST(BOOST_CPPFLAGS)
         AC_SUBST(BOOST_LDFLAGS)
         AC_DEFINE(HAVE_BOOST,,[define if the Boost library is available])

         AC_CACHE_CHECK([whether the Boost::Filesystem library is available],
                     ax_cv_boost_filesystem,
                  [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/filesystem/path.hpp>]],
                                   [[using namespace boost::filesystem;
                                   path my_path( "foo/bar/data.txt" );
                                   return 0;]]),
                               ax_cv_boost_filesystem=yes, ax_cv_boost_filesystem=no)
                                   AC_LANG_POP([C++])
         ])
         if test "$ax_cv_boost_filesystem" = "yes"; then
            AC_DEFINE(HAVE_BOOST_FILESYSTEM,,[define if the Boost::FILESYSTEM library is available])
            BN=boost_filesystem
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s $BN-$_version \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s  \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main,
                                 [BOOST_FILESYSTEM_LIB="-l$ax_lib" AC_SUBST(BOOST_FILESYSTEM_LIB) link_filesystem="yes" break],
                                 [link_filesystem="no"])
              done
            if test "x$link_filesystem" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi
         fi

         AC_CACHE_CHECK([whether the Boost::Program_Options library is available],
                     ax_cv_boost_program_options,
                     [AC_LANG_PUSH([C++])
                        AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/program_options.hpp>]],
                                   [[boost::program_options::options_description generic("Generic options");
                                   return 0;]]),
                           ax_cv_boost_program_options=yes, ax_cv_boost_program_options=no)
                           AC_LANG_POP([C++])
         ])
         if test "$ax_cv_boost_program_options" = yes; then
            AC_DEFINE(HAVE_BOOST_PROGRAM_OPTIONS,,[define if the Boost::PROGRAM_OPTIONS library is available])
            BN=boost_program_options
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main,
                                 [BOOST_PROGRAM_OPTIONS_LIB="-l$ax_lib" AC_SUBST(BOOST_PROGRAM_OPTIONS_LIB) link_program_options="yes" break],
                                 [link_program_options="no"])
              done
            if test "x$link_program_options="no"" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi
         fi

         AC_CACHE_CHECK(whether the Boost::Thread library is available,
                     ax_cv_boost_thread,
                  [AC_LANG_PUSH([C++])
          CXXFLAGS_SAVE=$CXXFLAGS

          if test "x$build_os" = "xsolaris" ; then
             CXXFLAGS="-pthreads $CXXFLAGS"
          elif test "x$build_os" = "xming32" ; then
             CXXFLAGS="-mthreads $CXXFLAGS"
          elif test "x$build_os" = "xlinux-gnu" ; then
             CXXFLAGS="-pthread $CXXFLAGS"
          fi
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/thread/thread.hpp>]],
                                   [[boost::thread_group thrds;
                                   return 0;]]),
                   ax_cv_boost_thread=yes, ax_cv_boost_thread=no)
          CXXFLAGS=$CXXFLAGS_SAVE
             AC_LANG_POP([C++])
         ])

         if test "x$ax_cv_boost_thread" = "xyes"; then
               if test "x$build_os" = "xsolaris" ; then
               BOOST_CPPFLAGS="-pthreads $BOOST_CPPFLAGS"
            elif test "x$build_os" = "xming32" ; then
               BOOST_CPPFLAGS="-mthreads $BOOST_CPPFLAGS"
            elif test "x$build_os" = "xlinx-gnu" ; then
               BOOST_CPPFLAGS="-pthread $BOOST_CPPFLAGS"
            fi

            AC_SUBST(BOOST_CPPFLAGS)
            AC_DEFINE(HAVE_BOOST_THREAD,,[define if the Boost::THREAD library is available])
            BN=boost_thread
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main, [BOOST_THREAD_LIB="-l$ax_lib" AC_SUBST(BOOST_THREAD_LIB) link_thread="yes" break],
                                 [link_thread="no"])
              done
            if test "x$link_thread" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi
         fi

         AC_CACHE_CHECK(whether the Boost::IOStreams library is available,
                     ax_cv_boost_iostreams,
                  [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/iostreams/filtering_stream.hpp>
                                     @%:@include <boost/range/iterator_range.hpp>
                                    ]],
                                   [[std::string  input = "Hello World!";
                            namespace io = boost::iostreams;
                            io::filtering_istream  in(boost::make_iterator_range(input));
                            return 0;
                                   ]]),
                   ax_cv_boost_iostreams=yes, ax_cv_boost_iostreams=no)
          AC_LANG_POP([C++])
         ])
         if test "x$ax_cv_boost_iostreams" = "xyes"; then
            AC_DEFINE(HAVE_BOOST_IOSTREAMS,,[define if the Boost::IOStreams library is available])
            BN=boost_iostreams
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main, [BOOST_IOSTREAMS_LIB="-l$ax_lib" AC_SUBST(BOOST_IOSTREAMS_LIB) link_thread="yes" break],
                                 [link_thread="no"])
              done
            if test "x$link_thread" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi
         fi

         AC_CACHE_CHECK(whether the Boost::Serialization library is available,
                     ax_cv_boost_serialization,
                  [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <fstream>
                                     @%:@include <boost/archive/text_oarchive.hpp>
                                                 @%:@include <boost/archive/text_iarchive.hpp>
                                    ]],
                                   [[std::ofstream ofs("filename");
                           boost::archive::text_oarchive oa(ofs);
                            return 0;
                                   ]]),
                   ax_cv_boost_serialization=yes, ax_cv_boost_serialization=no)
          AC_LANG_POP([C++])
         ])
         if test "x$ax_cv_boost_serialization" = "xyes"; then
            AC_DEFINE(HAVE_BOOST_SERIALIZATION,,[define if the Boost::Serialization library is available])
            BN=boost_serialization
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main,
                                 [BOOST_SERIALIZATION_LIB="-l$ax_lib" AC_SUBST(BOOST_SERIALIZATION_LIB) link_serialization="yes" break],
                                 [link_serialization="no"])
              done
            if test "x$link_serialization" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi

            BN=boost_wserialization
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main,
                                 [BOOST_WSERIALIZATION_LIB="-l$ax_lib" AC_SUBST(BOOST_WSERIALIZATION_LIB) link_wserialization="yes" break],
                                 [link_wserialization="no"])
              done
            if test "x$link_wserialization" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi
         fi

         AC_CACHE_CHECK(whether the Boost::Signals library is available,
                     ax_cv_boost_signals,
                  [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/signal.hpp>
                                    ]],
                                   [[boost::signal<void ()> sig;
                                     return 0;
                                   ]]),
                   ax_cv_boost_signals=yes, ax_cv_boost_signals=no)
          AC_LANG_POP([C++])
         ])
         if test "x$ax_cv_boost_signals" = "xyes"; then
            AC_DEFINE(HAVE_BOOST_SIGNALS,,[define if the Boost::Signals library is available])
            BN=boost_signals
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main, [BOOST_SIGNALS_LIB="-l$ax_lib" AC_SUBST(BOOST_SIGNALS_LIB) link_signals="yes" break],
                                 [link_signals="no"])
              done
            if test "x$link_signals" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi
         fi

         AC_CACHE_CHECK(whether the Boost::Date_Time library is available,
                     ax_cv_boost_date_time,
                  [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/date_time/gregorian/gregorian_types.hpp>
                                    ]],
                                   [[using namespace boost::gregorian; date d(2002,Jan,10);
                                     return 0;
                                   ]]),
                   ax_cv_boost_date_time=yes, ax_cv_boost_date_time=no)
          AC_LANG_POP([C++])
         ])
         if test "x$ax_cv_boost_date_time" = "xyes"; then
            AC_DEFINE(HAVE_BOOST_DATE_TIME,,[define if the Boost::Date_Time library is available])
            BN=boost_date_time
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main, [BOOST_DATE_TIME_LIB="-l$ax_lib" AC_SUBST(BOOST_DATE_TIME_LIB) link_thread="yes" break],
                                 [link_thread="no"])
              done
            if test "x$link_thread"="no" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi
         fi

         AC_CACHE_CHECK(whether the Boost::Regex library is available,
                     ax_cv_boost_regex,
                  [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/regex.hpp>
                                    ]],
                                   [[boost::regex r(); return 0;]]),
                   ax_cv_boost_regex=yes, ax_cv_boost_regex=no)
          AC_LANG_POP([C++])
         ])
         if test "x$ax_cv_boost_regex" = "xyes"; then
            AC_DEFINE(HAVE_BOOST_REGEX,,[define if the Boost::Regex library is available])
            BN=boost_regex
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main, [BOOST_REGEX_LIB="-l$ax_lib" AC_SUBST(BOOST_REGEX_LIB) link_regex="yes" break],
                                 [link_regex="no"])
              done
            if test "x$link_regex" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi
                                
         fi

         AC_CACHE_CHECK(whether the Boost::UnitTestFramework library is available,
                     ax_cv_boost_unit_test_framework,
                  [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/test/unit_test.hpp>]],
                                    [[using boost::unit_test::test_suite;
                                test_suite* test= BOOST_TEST_SUITE( "Unit test example 1" ); return 0;]]),
                   ax_cv_boost_unit_test_framework=yes, ax_cv_boost_unit_test_framework=no)
          AC_LANG_POP([C++])
         ])
         if test "x$ax_cv_boost_unit_test_framework" = "xyes"; then
          AC_DEFINE(HAVE_BOOST_UNIT_TEST_FRAMEWORK,,[define if the Boost::Unit_test_framework library is available])
         BN=boost_unit_test_framework
          saved_ldflags="${LDFLAGS}"
         for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                          lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                          $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                LDFLAGS="${LDFLAGS} -l$ax_lib"
             AC_CACHE_CHECK(the name of the Boost::UnitTestFramework library,
                           ax_cv_boost_unit_test_framework_link,
                  [AC_LANG_PUSH([C++])
                   AC_LINK_IFELSE([AC_LANG_PROGRAM([[@%:@include <boost/test/unit_test.hpp>
                                                     using boost::unit_test::test_suite;
                                                     test_suite* init_unit_test_suite( int argc, char * argv[] ) {
                                                     test_suite* test= BOOST_TEST_SUITE( "Unit test example 1" );
                                                     return test;
                                                     }
                                                   ]],
                                 [[ return 0;]])],
                                 link_unit_test_framework="yes",link_unit_test_framework="no")
          AC_LANG_POP([C++])
               ])
                LDFLAGS="${saved_ldflags}"
             if test "x$link_unit_test_framework" = "xyes"; then
                    BOOST_UNIT_TEST_FRAMEWORK_LIB="-l$ax_lib"
                    AC_SUBST(BOOST_UNIT_TEST_FRAMEWORK_LIB)
               break
            fi
              done
             if test "x$link_unit_test_framework" = "xno"; then
               AC_MSG_NOTICE(Could not link against $ax_lib !)
            fi
         fi
      fi
        CPPFLAGS="$CPPFLAGS_SAVED"
        LDFLAGS="$LDFLAGS_SAVED"
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_base.html
#
# SYNOPSIS
#
#   AX_BOOST_BASE([MINIMUM-VERSION])
#
# DESCRIPTION
#
#   Test for the Boost C++ libraries of a particular version (or newer)
#
#   If no path to the installed boost library is given the macro
#   searchs under /usr, /usr/local, and /opt, and evaluates the
#   $BOOST_ROOT environment variable. Further documentation is
#   available at <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_CPPFLAGS) / AC_SUBST(BOOST_LDFLAGS)
#
#   And sets:
#
#     HAVE_BOOST
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_BASE],
[
AC_ARG_WITH([boost],
   AS_HELP_STRING([--with-boost@<:@=DIR@:>@], [use boost (default is No) - it is possible to specify the root directory for boost (optional)]),
   [
    if test "$withval" = "no"; then
      want_boost="no"
    elif test "$withval" = "yes"; then
        want_boost="yes"
        ac_boost_path=""
    else
       want_boost="yes"
        ac_boost_path="$withval"
   fi
    ],
    [want_boost="yes"])

if test "x$want_boost" = "xyes"; then
   boost_lib_version_req=ifelse([$1], ,1.20.0,$1)
   boost_lib_version_req_shorten=`expr $boost_lib_version_req : '\([[0-9]]*\.[[0-9]]*\)'`
   boost_lib_version_req_major=`expr $boost_lib_version_req : '\([[0-9]]*\)'`
   boost_lib_version_req_minor=`expr $boost_lib_version_req : '[[0-9]]*\.\([[0-9]]*\)'`
   boost_lib_version_req_sub_minor=`expr $boost_lib_version_req : '[[0-9]]*\.[[0-9]]*\.\([[0-9]]*\)'`
   if test "x$boost_lib_version_req_sub_minor" = "x" ; then
      boost_lib_version_req_sub_minor="0"
       fi
   WANT_BOOST_VERSION=`expr $boost_lib_version_req_major \* 100000 \+  $boost_lib_version_req_minor \* 100 \+ $boost_lib_version_req_sub_minor`
   AC_MSG_CHECKING(for boostlib >= $boost_lib_version_req)
   succeeded=no

   dnl first we check the system location for boost libraries
   dnl this location ist chosen if boost libraries are installed with the --layout=system option
   dnl or if you install boost with RPM
   if test "$ac_boost_path" != ""; then
      BOOST_LDFLAGS="-L$ac_boost_path/lib"
      BOOST_CPPFLAGS="-I$ac_boost_path/include"
   else
      for ac_boost_path_tmp in $INSROOT /usr /usr/local /opt /opt/local ; do
         if test -d "$ac_boost_path_tmp/include/boost" && test -r "$ac_boost_path_tmp/include/boost"; then
            BOOST_LDFLAGS="-L$ac_boost_path_tmp/lib"
            BOOST_CPPFLAGS="-I$ac_boost_path_tmp/include"
            break;
         fi
      done
   fi
   CPPFLAGS_SAVED="$CPPFLAGS"
   CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
   export CPPFLAGS
   LDFLAGS_SAVED="$LDFLAGS"
   LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
   export LDFLAGS

   AC_LANG_PUSH(C++)
        AC_COMPILE_IFELSE([AC_LANG_PROGRAM([[
   @%:@include <boost/version.hpp>
   ]], [[
   #if BOOST_VERSION >= $WANT_BOOST_VERSION
   // Everything is okay
   #else
   #  error Boost version is too old
   #endif
   ]])],[
        AC_MSG_RESULT(yes)
   succeeded=yes
   found_system=yes
          ],[
          ])
   AC_LANG_POP([C++])



   dnl if we found no boost with system layout we search for boost libraries
   dnl built and installed without the --layout=system option or for a staged(not installed) version
   if test "x$succeeded" != "xyes"; then
      _version=0
      if test "$ac_boost_path" != ""; then
                     BOOST_LDFLAGS="-L$ac_boost_path/lib"
         if test -d "$ac_boost_path" && test -r "$ac_boost_path"; then
            for i in `ls -d $ac_boost_path/include/boost-* 2>/dev/null`; do
               _version_tmp=`echo $i | sed "s#$ac_boost_path##" | sed 's/\/include\/boost-//' | sed 's/_/./'`
               V_CHECK=`expr $_version_tmp \> $_version`
               if test "$V_CHECK" = "1" ; then
                  _version=$_version_tmp
               fi
               VERSION_UNDERSCORE=`echo $_version | sed 's/\./_/'`
               BOOST_CPPFLAGS="-I$ac_boost_path/include/boost-$VERSION_UNDERSCORE"
            done
         fi
      else
         for ac_boost_path in $INSROOT /usr /usr/local /opt ; do
            if test -d "$ac_boost_path" && test -r "$ac_boost_path"; then
               for i in `ls -d $ac_boost_path/include/boost-* 2>/dev/null`; do
                  _version_tmp=`echo $i | sed "s#$ac_boost_path##" | sed 's/\/include\/boost-//' | sed 's/_/./'`
                  V_CHECK=`expr $_version_tmp \> $_version`
                  if test "$V_CHECK" = "1" ; then
                     _version=$_version_tmp
                                 best_path=$ac_boost_path
                  fi
               done
            fi
         done

         VERSION_UNDERSCORE=`echo $_version | sed 's/\./_/'`
         BOOST_CPPFLAGS="-I$best_path/include/boost-$VERSION_UNDERSCORE"
         BOOST_LDFLAGS="-L$best_path/lib"

             if test "x$BOOST_ROOT" != "x"; then
            if test -d "$BOOST_ROOT" && test -r "$BOOST_ROOT" && test -d "$BOOST_ROOT/stage/lib" && test -r "$BOOST_ROOT/stage/lib"; then
               version_dir=`expr //$BOOST_ROOT : '.*/\(.*\)'`
               stage_version=`echo $version_dir | sed 's/boost_//' | sed 's/_/./g'`
                    stage_version_shorten=`expr $stage_version : '\([[0-9]]*\.[[0-9]]*\)'`
               V_CHECK=`expr $stage_version_shorten \>\= $_version`
                    if test "$V_CHECK" = "1" ; then
                  AC_MSG_NOTICE(We will use a staged boost library from $BOOST_ROOT)
                  BOOST_CPPFLAGS="-I$BOOST_ROOT"
                  BOOST_LDFLAGS="-L$BOOST_ROOT/stage/lib"
               fi
            fi
             fi
      fi

      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS

      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

      AC_LANG_PUSH(C++)
           AC_COMPILE_IFELSE([AC_LANG_PROGRAM([[
      @%:@include <boost/version.hpp>
      ]], [[
      #if BOOST_VERSION >= $WANT_BOOST_VERSION
      // Everything is okay
      #else
      #  error Boost version is too old
      #endif
      ]])],[
           AC_MSG_RESULT(yes)
      succeeded=yes
      found_system=yes
             ],[
             ])
      AC_LANG_POP([C++])
   fi

   if test "$succeeded" != "yes" ; then
      if test "$_version" = "0" ; then
         AC_MSG_ERROR([[We could not detect the boost libraries (version $boost_lib_version_req_shorten or higher). If you have a staged boost library (still not installed) please specify \$BOOST_ROOT in your environment and do not give a PATH to --with-boost option.  If you are sure you have boost installed, then check your version number looking in <boost/version.hpp>. See http://randspringer.de/boost for more documentation.]])
      else
         AC_MSG_NOTICE([Your boost libraries seems to old (version $_version).])
      fi
   else
      AC_SUBST(BOOST_CPPFLAGS)
      AC_SUBST(BOOST_LDFLAGS)
      AC_DEFINE(HAVE_BOOST,,[define if the Boost library is available])
   fi

        CPPFLAGS="$CPPFLAGS_SAVED"
          LDFLAGS="$LDFLAGS_SAVED"
fi

])
##### http://autoconf-archive.cryp.to/ax_boost_date_time.html
#
# SYNOPSIS
#
#   AX_BOOST_DATE_TIME
#
# DESCRIPTION
#
#   Test for Date_Time library from the Boost C++ libraries. The macro
#   requires a preceding call to AX_BOOST_BASE. Further documentation
#   is available at <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_DATE_TIME_LIB)
#
#   And sets:
#
#     HAVE_BOOST_DATE_TIME
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#   Copyright (c) 2006 Michael Tindal <mtindal@paradoxpoint.com>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_DATE_TIME],
[
   AC_ARG_WITH([boost-date-time],
   AS_HELP_STRING([--with-boost-date-time@<:@=special-lib@:>@],
                   [use the Date_Time library from boost - it is possible to specify a certain library for the linker
                        e.g. --with-boost-date-time=boost_date_time-gcc-mt-d-1_33_1 ]),
        [
        if test "$withval" = "no"; then
         want_boost="no"
        elif test "$withval" = "yes"; then
            want_boost="yes"
            ax_boost_user_date_time_lib=""
        else
          want_boost="yes"
           ax_boost_user_date_time_lib="$withval"
      fi
        ],
        [want_boost="no"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

        AC_CACHE_CHECK(whether the Boost::Date_Time library is available,
                  ax_cv_boost_date_time,
        [AC_LANG_PUSH([C++])
       AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/date_time/gregorian/gregorian_types.hpp>]],
                                   [[using namespace boost::gregorian; date d(2002,Jan,10);
                                     return 0;
                                   ]]),
         ax_cv_boost_date_time=yes, ax_cv_boost_date_time=no)
         AC_LANG_POP([C++])
      ])
      if test "x$ax_cv_boost_date_time" = "xyes"; then
         AC_DEFINE(HAVE_BOOST_DATE_TIME,,[define if the Boost::Date_Time library is available])
         BN=boost_date_time
            if test "x$ax_boost_user_date_time_lib" = "x"; then
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                               lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                               $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
               AC_CHECK_LIB($ax_lib, main, [BOOST_DATE_TIME_LIB="-l$ax_lib" AC_SUBST(BOOST_DATE_TIME_LIB) link_date_time="yes" break],
                               [link_date_time="no"])
              done
            else
               for ax_lib in $ax_boost_user_date_time_lib $BN-$ax_boost_user_date_time_lib; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_DATE_TIME_LIB="-l$ax_lib" AC_SUBST(BOOST_DATE_TIME_LIB) link_date_time="yes" break],
                                   [link_date_time="no"])
                  done

            fi
         if test "x$link_date_time" = "xno"; then
            AC_MSG_ERROR(Could not link against $ax_lib !)
         fi
      fi

      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_filesystem.html
#
# SYNOPSIS
#
#   AX_BOOST_FILESYSTEM
#
# DESCRIPTION
#
#   Test for Filesystem library from the Boost C++ libraries. The macro
#   requires a preceding call to AX_BOOST_BASE. Further documentation
#   is available at <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_FILESYSTEM_LIB)
#
#   And sets:
#
#     HAVE_BOOST_FILESYSTEM
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#   Copyright (c) 2006 Michael Tindal <mtindal@paradoxpoint.com>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_FILESYSTEM],
[
   AC_ARG_WITH([boost-filesystem],
   AS_HELP_STRING([--with-boost-filesystem@<:@=special-lib@:>@],
                   [use the Filesystem library from boost - it is possible to specify a certain library for the linker
                        e.g. --with-boost-filesystem=boost_filesystem-gcc-mt ]),
        [
        if test "$withval" = "no"; then
         want_boost="no"
        elif test "$withval" = "yes"; then
            want_boost="yes"
            ax_boost_user_filesystem_lib=""
        else
          want_boost="yes"
           ax_boost_user_filesystem_lib="$withval"
      fi
        ],
        [want_boost="yes"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

        AC_CACHE_CHECK(whether the Boost::Filesystem library is available,
                  ax_cv_boost_filesystem,
        [AC_LANG_PUSH([C++])
         AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/filesystem/path.hpp>]],
                                   [[using namespace boost::filesystem;
                                   path my_path( "foo/bar/data.txt" );
                                   return 0;]]),
                               ax_cv_boost_filesystem=yes, ax_cv_boost_filesystem=no)
         AC_LANG_POP([C++])
      ])

      if test "x$ax_cv_boost_filesystem" = "xyes"; then
         AC_DEFINE(HAVE_BOOST_FILESYSTEM,,[define if the Boost::Filesystem library is available])
         BN=boost_filesystem
            if test "x$ax_boost_user_filesystem_lib" = "x"; then
             for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main,
                                 [BOOST_FILESYSTEM_LIB="-l$ax_lib" AC_SUBST(BOOST_FILESYSTEM_LIB) link_filesystem="yes" break],
                                 [link_filesystem="no"])
              done
            else
               for ax_lib in $ax_boost_user_filesystem_lib $BN-$ax_boost_user_filesystem_lib; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_FILESYSTEM_LIB="-l$ax_lib" AC_SUBST(BOOST_FILESYSTEM_LIB) link_filesystem="yes" break],
                                   [link_filesystem="no"])
                  done

            fi
         if test "x$link_filesystem" = "xno"; then
            AC_MSG_ERROR(Could not link against $ax_lib !)
         fi
      fi

      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_iostreams.html
#
# SYNOPSIS
#
#   AX_BOOST_IOSTREAMS
#
# DESCRIPTION
#
#   Test for IOStreams library from the Boost C++ libraries. The macro
#   requires a preceding call to AX_BOOST_BASE. Further documentation
#   is available at <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_IOSTREAMS_LIB)
#
#   And sets:
#
#     HAVE_BOOST_IOSTREAMS
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_IOSTREAMS],
[
   AC_ARG_WITH([boost-iostreams],
   AS_HELP_STRING([--with-boost-iostreams@<:@=special-lib@:>@],
                   [use the IOStreams library from boost - it is possible to specify a certain library for the linker
                        e.g. --with-boost-iostreams=boost_iostreams-gcc-mt-d-1_33_1 ]),
        [
        if test "$withval" = "no"; then
         want_boost="no"
        elif test "$withval" = "yes"; then
            want_boost="yes"
            ax_boost_user_iostreams_lib=""
        else
          want_boost="yes"
           ax_boost_user_iostreams_lib="$withval"
      fi
        ],
        [want_boost="no"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

        AC_CACHE_CHECK(whether the Boost::IOStreams library is available,
                  ax_cv_boost_iostreams,
        [AC_LANG_PUSH([C++])
       AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/iostreams/filtering_stream.hpp>
                                  @%:@include <boost/range/iterator_range.hpp>
                                 ]],
                                  [[std::string  input = "Hello World!";
                                namespace io = boost::iostreams;
                            io::filtering_istream  in(boost::make_iterator_range(input));
                            return 0;
                                   ]]),
                             ax_cv_boost_iostreams=yes, ax_cv_boost_iostreams=no)
         AC_LANG_POP([C++])
      ])
      if test "x$ax_cv_boost_iostreams" = "xyes"; then
         AC_DEFINE(HAVE_BOOST_IOSTREAMS,,[define if the Boost::IOStreams library is available])
         BN=boost_iostreams
            if test "x$ax_boost_user_iostreams_lib" = "x"; then
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main, [BOOST_IOSTREAMS_LIB="-l$ax_lib" AC_SUBST(BOOST_IOSTREAMS_LIB) link_thread="yes" break],
                                 [link_thread="no"])
              done
            else
               for ax_lib in $ax_boost_user_iostreams_lib $BN-$ax_boost_user_iostreams_lib; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_IOSTREAMS_LIB="-l$ax_lib" AC_SUBST(BOOST_IOSTREAMS_LIB) link_iostreams="yes" break],
                                   [link_iostreams="no"])
                  done

            fi
         if test "x$link_iostreams" = "xno"; then
            AC_MSG_ERROR(Could not link against $ax_lib !)
         fi
      fi

      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_program_options.html
#
# SYNOPSIS
#
#   AX_BOOST_PROGRAM_OPTIONS
#
# DESCRIPTION
#
#   Test for program options library from the Boost C++ libraries. The
#   macro requires a preceding call to AX_BOOST_BASE. Further
#   documentation is available at
#   <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_PROGRAM_OPTIONS_LIB)
#
#   And sets:
#
#     HAVE_BOOST_PROGRAM_OPTIONS
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_PROGRAM_OPTIONS],
[
   AC_ARG_WITH([boost-program-options],
      AS_HELP_STRING([--with-boost-program-options@<:@=special-lib@:>@],
                       [use the program options library from boost - it is possible to specify a certain library for the linker
                        e.g. --with-boost-program-options=boost_program_options-gcc-mt-1_33_1 ]),
        [
        if test "$withval" = "no"; then
         want_boost="no"
        elif test "$withval" = "yes"; then
            want_boost="yes"
            ax_boost_user_program_options_lib=""
        else
          want_boost="yes"
           ax_boost_user_program_options_lib="$withval"
      fi
        ],
        [want_boost="no"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
       export want_boost
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS
      AC_CACHE_CHECK([whether the Boost::Program_Options library is available],
                  ax_cv_boost_program_options,
                  [AC_LANG_PUSH(C++)
                       AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/program_options.hpp>
                                                          ]],
                                  [[boost::program_options::options_description generic("Generic options");
                                   return 0;]]),
                           ax_cv_boost_program_options=yes, ax_cv_boost_program_options=no)
                           AC_LANG_POP([C++])
      ])
      if test "$ax_cv_boost_program_options" = yes; then
            AC_DEFINE(HAVE_BOOST_PROGRAM_OPTIONS,,[define if the Boost::PROGRAM_OPTIONS library is available])
              BN=boost_program_options
                if test "x$ax_boost_user_program_options_lib" = "x"; then
              for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                                lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                                $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_PROGRAM_OPTIONS_LIB="-l$ax_lib" AC_SUBST(BOOST_PROGRAM_OPTIONS_LIB) link_program_options="yes" break],
                                   [link_program_options="no"])
                done
                else
                  for ax_lib in $ax_boost_user_program_options_lib $BN-$ax_boost_user_program_options_lib; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_PROGRAM_OPTIONS_LIB="-l$ax_lib" AC_SUBST(BOOST_PROGRAM_OPTIONS_LIB) link_program_options="yes" break],
                                   [link_program_options="no"])
                  done
                fi
            if test "x$link_program_options" = "xno"; then
               AC_MSG_ERROR([Could not link against [$ax_lib] !])
            fi
      fi
      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_python.html
#
# SYNOPSIS
#
#   AX_BOOST_PYTHON
#
# DESCRIPTION
#
#   This macro checks to see if the Boost.Python library is installed.
#   It also attempts to guess the currect library name using several
#   attempts. It tries to build the library name using a user supplied
#   name or suffix and then just the raw library.
#
#   If the library is found, HAVE_BOOST_PYTHON is defined and
#   BOOST_PYTHON_LIB is set to the name of the library.
#
#   This macro calls AC_SUBST(BOOST_PYTHON_LIB).
#
#   In order to ensure that the Python headers are specified on the
#   include path, this macro requires AX_PYTHON to be called.
#
# LAST MODIFICATION
#
#   2005-05-20
#
# COPYLEFT
#
#   Copyright (c) 2005 Michael Tindal <mtindal@paradoxpoint.com>
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
#   02111-1307, USA.
#
#   As a special exception, the respective Autoconf Macro's copyright
#   owner gives unlimited permission to copy, distribute and modify the
#   configure scripts that are the output of Autoconf when processing
#   the Macro. You need not follow the terms of the GNU General Public
#   License when using or distributing such scripts, even though
#   portions of the text of the Macro appear in them. The GNU General
#   Public License (GPL) does govern all other use of the material that
#   constitutes the Autoconf Macro.
#
#   This special exception to the GPL applies to versions of the
#   Autoconf Macro released by the Autoconf Macro Archive. When you
#   make and distribute a modified version of the Autoconf Macro, you
#   may extend this special exception to the GPL to apply to your
#   modified version as well.

AC_DEFUN([AX_BOOST_PYTHON],
[AC_REQUIRE([AX_PYTHON])dnl
AC_CACHE_CHECK(whether the Boost::Python library is available,
ac_cv_boost_python,
[AC_LANG_SAVE
 AC_LANG_CPLUSPLUS
 CPPFLAGS_SAVE=$CPPFLAGS
 if test x$PYTHON_INCLUDE_DIR != x; then
   CPPFLAGS=-I$PYTHON_INCLUDE_DIR $CPPFLAGS
 fi
 AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[
 #include <boost/python/module.hpp>
 using namespace boost::python;
 BOOST_PYTHON_MODULE(test) { throw "Boost::Python test."; }]],
             [[return 0;]]),
              ac_cv_boost_python=yes, ac_cv_boost_python=no)
 AC_LANG_RESTORE
 CPPFLAGS=$CPPFLAGS_SAVE
])
if test "$ac_cv_boost_python" = "yes"; then
  AC_DEFINE(HAVE_BOOST_PYTHON,,[define if the Boost::Python library is available])
  ax_python_lib=boost_python
  AC_ARG_WITH([boost-python],AS_HELP_STRING([--with-boost-python],[specify the boost python library or suffix to use]),
  [if test "x$with_boost_python" != "xno"; then
     ax_python_lib=$with_boost_python
     ax_boost_python_lib=boost_python-$with_boost_python
   fi])
  for ax_lib in $ax_python_lib $ax_boost_python_lib boost_python; do
    AC_CHECK_LIB($ax_lib, main, [BOOST_PYTHON_LIB=$ax_lib break])
  done
  AC_SUBST(BOOST_PYTHON_LIB)
fi
])dnl
##### http://autoconf-archive.cryp.to/ax_boost_regex.html
#
# SYNOPSIS
#
#   AX_BOOST_REGEX
#
# DESCRIPTION
#
#   Test for Regex library from the Boost C++ libraries. The macro
#   requires a preceding call to AX_BOOST_BASE. Further documentation
#   is available at <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_REGEX_LIB)
#
#   And sets:
#
#     HAVE_BOOST_REGEX
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#   Copyright (c) 2006 Michael Tindal <mtindal@paradoxpoint.com>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_REGEX],
[
   AC_ARG_WITH([boost-regex],
   AS_HELP_STRING([--with-boost-regex@<:@=special-lib@:>@],
                   [use the Regex library from boost - it is possible to specify a certain library for the linker
                        e.g. --with-boost-regex=boost_regex-gcc-mt-d-1_33_1 ]),
        [
        if test "$withval" = "no"; then
                        AC_MSG_NOTICE(1) 
         want_boost="no"
        elif test "$withval" = "yes"; then
            AC_MSG_NOTICE(2) 
            want_boost="yes"
            ax_boost_user_regex_lib=""
        else
                 AC_MSG_NOTICE(3) 
          want_boost="yes"
           ax_boost_user_regex_lib="$withval"
      fi
        ],
        [want_boost="no"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS
           AC_MSG_NOTICE(3) 
        AC_CACHE_CHECK(whether the Boost::Regex library is available,
                  ax_cv_boost_regex,
        [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/regex.hpp>
                                    ]],
                                   [[boost::regex r(); return 0;]]),
                   ax_cv_boost_regex=yes, ax_cv_boost_regex=no)
         AC_LANG_POP([C++])
      ])
      if test "x$ax_cv_boost_regex" = "xyes"; then
         AC_DEFINE(HAVE_BOOST_REGEX,,[define if the Boost::Regex library is available])
         BN=boost_regex
                        AC_MSG_NOTICE(3)        
            if test "x$ax_boost_user_regex_lib" = "x"; then
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main, [BOOST_REGEX_LIB="-l$ax_lib" AC_SUBST(BOOST_REGEX_LIB) link_regex="yes" break],
                                 [link_regex="no"])
              done
                                AC_MSG_NOTICE(3)
            else
               for ax_lib in $ax_boost_user_regex_lib $BN-$ax_boost_user_regex_lib; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_REGEX_LIB="-l$ax_lib" AC_SUBST(BOOST_REGEX_LIB) link_regex="yes" break],
                                   [link_regex="no"])
               done
               AC_MSG_NOTICE(3)
            fi
         if test "x$link_regex" = "xno"; then
            AC_MSG_ERROR(Could not link against $ax_lib !)
                         else
                                 AC_MSG_NOTICE(Linked against $ax_lib !) 
         fi
      fi
                AC_MSG_NOTICE(3)
      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
        AC_MSG_NOTICE(3)
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_serialization.html
#
# SYNOPSIS
#
#   AX_BOOST_SERIALIZATION
#
# DESCRIPTION
#
#   Test for Serialization library from the Boost C++ libraries. The
#   macro requires a preceding call to AX_BOOST_BASE. Further
#   documentation is available at
#   <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_SERIALIZATION_LIB)
#
#   And sets:
#
#     HAVE_BOOST_SERIALIZATION
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_SERIALIZATION],
[
   AC_ARG_WITH([boost-serialization],
   AS_HELP_STRING([--with-boost-serialization@<:@=special-lib@:>@],
                   [use the Serialization library from boost - it is possible to specify a certain library for the linker
                        e.g. --with-boost-serialization=boost_serialization-gcc-mt-d-1_33_1 ]),
        [
        if test "$withval" = "no"; then
         want_boost="no"
        elif test "$withval" = "yes"; then
            want_boost="yes"
            ax_boost_user_serialization_lib=""
        else
          want_boost="yes"
           ax_boost_user_serialization_lib="$withval"
      fi
        ],
        [want_boost="no"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

        AC_CACHE_CHECK(whether the Boost::Serialization library is available,
                  ax_cv_boost_serialization,
        [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <fstream>
                                     @%:@include <boost/archive/text_oarchive.hpp>
                                                 @%:@include <boost/archive/text_iarchive.hpp>
                                    ]],
                                   [[std::ofstream ofs("filename");
                           boost::archive::text_oarchive oa(ofs);
                            return 0;
                                   ]]),
                   ax_cv_boost_serialization=yes, ax_cv_boost_serialization=no)
         AC_LANG_POP([C++])
      ])
      if test "x$ax_cv_boost_serialization" = "xyes"; then
         AC_DEFINE(HAVE_BOOST_SERIALIZATION,,[define if the Boost::Serialization library is available])
         BN=boost_serialization
            if test "x$ax_boost_user_serialization_lib" = "x"; then
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main,
                                 [BOOST_SERIALIZATION_LIB="-l$ax_lib" AC_SUBST(BOOST_SERIALIZATION_LIB) link_serialization="yes" break],
                                 [link_serialization="no"])
              done
            else
               for ax_lib in $ax_boost_user_serialization_lib $BN-$ax_boost_user_serialization_lib; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_SERIALIZATION_LIB="-l$ax_lib" AC_SUBST(BOOST_SERIALIZATION_LIB) link_serialization="yes" break],
                                   [link_serialization="no"])
                  done

            fi
         if test "x$link_serialization" = "xno"; then
            AC_MSG_ERROR(Could not link against $ax_lib !)
         fi
      fi

      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_signals.html
#
# SYNOPSIS
#
#   AX_BOOST_SIGNALS
#
# DESCRIPTION
#
#   Test for Signals library from the Boost C++ libraries. The macro
#   requires a preceding call to AX_BOOST_BASE. Further documentation
#   is available at <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_SIGNALS_LIB)
#
#   And sets:
#
#     HAVE_BOOST_SIGNALS
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#   Copyright (c) 2006 Michael Tindal <mtindal@paradoxpoint.com>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_SIGNALS],
[
   AC_ARG_WITH([boost-signals],
   AS_HELP_STRING([--with-boost-signals@<:@=special-lib@:>@],
                   [use the Signals library from boost - it is possible to specify a certain library for the linker
                        e.g. --with-boost-signals=boost_signals-gcc-mt-d ]),
        [
        if test "$withval" = "no"; then
         want_boost="no"
        elif test "$withval" = "yes"; then
            want_boost="yes"
            ax_boost_user_signals_lib=""
        else
          want_boost="yes"
           ax_boost_user_signals_lib="$withval"
      fi
        ],
        [want_boost="no"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

        AC_CACHE_CHECK(whether the Boost::Signals library is available,
                  ax_cv_boost_signals,
        [AC_LANG_PUSH([C++])
       AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/signal.hpp>
                                 ]],
                                  [[boost::signal<void ()> sig;
                                    return 0;
                                  ]]),
                           ax_cv_boost_signals=yes, ax_cv_boost_signals=no)
         AC_LANG_POP([C++])
      ])
      if test "x$ax_cv_boost_signals" = "xyes"; then
         AC_DEFINE(HAVE_BOOST_SIGNALS,,[define if the Boost::Signals library is available])
         BN=boost_signals
            if test "x$ax_boost_user_signals_lib" = "x"; then
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main, [BOOST_SIGNALS_LIB="-l$ax_lib" AC_SUBST(BOOST_SIGNALS_LIB) link_signals="yes" break],
                                 [link_signals="no"])
              done
            else
               for ax_lib in $ax_boost_user_signals_lib $BN-$ax_boost_user_signals_lib; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_SIGNALS_LIB="-l$ax_lib" AC_SUBST(BOOST_SIGNALS_LIB) link_signals="yes" break],
                                   [link_signals="no"])
                  done

            fi
         if test "x$link_signals" = "xno"; then
            AC_MSG_ERROR(Could not link against $ax_lib !)
         fi
      fi

      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_thread.html
#
# SYNOPSIS
#
#   AX_BOOST_THREAD
#
# DESCRIPTION
#
#   Test for Thread library from the Boost C++ libraries. The macro
#   requires a preceding call to AX_BOOST_BASE. Further documentation
#   is available at <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_THREAD_LIB)
#
#   And sets:
#
#     HAVE_BOOST_THREAD
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#   Copyright (c) 2006 Michael Tindal <mtindal@paradoxpoint.com>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_THREAD],
[
   AC_ARG_WITH([boost-thread],
   AS_HELP_STRING([--with-boost-thread@<:@=special-lib@:>@],
                   [use the Thread library from boost - it is possible to specify a certain library for the linker
                        e.g. --with-boost-thread=boost_thread-gcc-mt ]),
        [
        if test "$withval" = "no"; then
         want_boost="no"
        elif test "$withval" = "yes"; then
            want_boost="yes"
            ax_boost_user_thread_lib=""
        else
          want_boost="yes"
           ax_boost_user_thread_lib="$withval"
      fi
        ],
        [want_boost="no"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

        AC_CACHE_CHECK(whether the Boost::Thread library is available,
                  ax_cv_boost_thread,
        [AC_LANG_PUSH([C++])
          CXXFLAGS_SAVE=$CXXFLAGS

          if test "x$build_os" = "xsolaris" ; then
               CXXFLAGS="-pthreads $CXXFLAGS"
          elif test "x$build_os" = "xming32" ; then
             CXXFLAGS="-mthreads $CXXFLAGS"
          elif test "x$build_os" = "xlinux-gnu" ; then
            CXXFLAGS="-pthread $CXXFLAGS"
          fi
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/thread/thread.hpp>]],
                                   [[boost::thread_group thrds;
                                   return 0;]]),
                   ax_cv_boost_thread=yes, ax_cv_boost_thread=no)
          CXXFLAGS=$CXXFLAGS_SAVE
             AC_LANG_POP([C++])
      ])
      if test "x$ax_cv_boost_thread" = "xyes"; then
           if test "x$build_os" = "xsolaris" ; then
           BOOST_CPPFLAGS="-pthreads $BOOST_CPPFLAGS"
         elif test "x$build_os" = "xming32" ; then
           BOOST_CPPFLAGS="-mthreads $BOOST_CPPFLAGS"
         elif test "x$build_os" = "xlinux-gnu" ; then
           BOOST_CPPFLAGS="-pthread $BOOST_CPPFLAGS"
         fi

         AC_SUBST(BOOST_CPPFLAGS)

         AC_DEFINE(HAVE_BOOST_THREAD,,[define if the Boost::Date_Time library is available])
         BN=boost_thread

            if test "x$ax_boost_user_thread_lib" = "x"; then
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main, [BOOST_THREAD_LIB="-l$ax_lib" AC_SUBST(BOOST_THREAD_LIB) link_thread="yes" break],
                                 [link_thread="no"])
              done
            else
               for ax_lib in $ax_boost_user_thread_lib $BN-$ax_boost_user_thread_lib; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_THREAD_LIB="-l$ax_lib" AC_SUBST(BOOST_THREAD_LIB) link_thread="yes" break],
                                   [link_thread="no"])
                  done

            fi
         if test "x$link_thread" = "xno"; then
            AC_MSG_ERROR(Could not link against $ax_lib !)
         fi
      fi

      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_unit_test_framework.html
#
# SYNOPSIS
#
#   AX_BOOST_UNIT_TEST_FRAMEWORK
#
# DESCRIPTION
#
#   Test for Unit_Test_Framework library from the Boost C++ libraries.
#   The macro requires a preceding call to AX_BOOST_BASE. Further
#   documentation is available at
#   <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_UNIT_TEST_FRAMEWORK_LIB)
#
#   And sets:
#
#     HAVE_BOOST_UNIT_TEST_FRAMEWORK
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_UNIT_TEST_FRAMEWORK],
[
   AC_ARG_WITH([boost-unit-test-framework],
   AS_HELP_STRING([--with-boost-unit-test-framework@<:@=special-lib@:>@],
                   [use the Unit_Test_Framework library from boost - it is possible to specify a certain library for the linker
                        e.g. --with-boost-unit-test-framework=boost_unit_test_framework-gcc ]),
        [
        if test "$withval" = "no"; then
         want_boost="no"
        elif test "$withval" = "yes"; then
            want_boost="yes"
            ax_boost_user_unit_test_framework_lib=""
        else
          want_boost="yes"
           ax_boost_user_unit_test_framework_lib="$withval"
      fi
        ],
        [want_boost="no"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

        AC_CACHE_CHECK(whether the Boost::Unit_Test_Framework library is available,
                  ax_cv_boost_unit_test_framework,
        [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/test/unit_test.hpp>]],
                                    [[using boost::unit_test::test_suite;
                                test_suite* test= BOOST_TEST_SUITE( "Unit test example 1" ); return 0;]]),
                   ax_cv_boost_unit_test_framework=yes, ax_cv_boost_unit_test_framework=no)
         AC_LANG_POP([C++])
      ])
      if test "x$ax_cv_boost_unit_test_framework" = "xyes"; then
         AC_DEFINE(HAVE_BOOST_UNIT_TEST_FRAMEWORK,,[define if the Boost::Unit_Test_Framework library is available])
         BN=boost_unit_test_framework
            if test "x$ax_boost_user_unit_test_framework_lib" = "x"; then
               saved_ldflags="${LDFLAGS}"
             for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                             lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                             $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                   LDFLAGS="${LDFLAGS} -l$ax_lib"
                AC_CACHE_CHECK(Boost::UnitTestFramework library linkage,
                               ax_cv_boost_unit_test_framework_link,
                    [AC_LANG_PUSH([C++])
                   AC_LINK_IFELSE([AC_LANG_PROGRAM([[@%:@include <boost/test/unit_test.hpp>
                                                     using boost::unit_test::test_suite;
                                                     test_suite* init_unit_test_suite( int argc, char * argv[] ) {
                                                     test_suite* test= BOOST_TEST_SUITE( "Unit test example 1" );
                                                     return test;
                                                     }
                                                   ]],
                                 [[ return 0;]])],
                                 link_unit_test_framework="yes",link_unit_test_framework="no")
               AC_LANG_POP([C++])
                  ])
                  LDFLAGS="${saved_ldflags}"

               if test "x$link_unit_test_framework" = "xyes"; then
                      BOOST_UNIT_TEST_FRAMEWORK_LIB="-l$ax_lib"
                      AC_SUBST(BOOST_UNIT_TEST_FRAMEWORK_LIB)
                 break
              fi
                done
            else
               saved_ldflags="${LDFLAGS}"
               for ax_lib in $ax_boost_user_unit_test_framework_lib $BN-$ax_boost_user_unit_test_framework_lib; do
                   LDFLAGS="${LDFLAGS} -l$ax_lib"
                          AC_CACHE_CHECK(Boost::UnitTestFramework library linkage,
                               ax_cv_boost_unit_test_framework_link,
                    [AC_LANG_PUSH([C++])
                           AC_LINK_IFELSE([AC_LANG_PROGRAM([[@%:@include <boost/test/unit_test.hpp>
                                                        using boost::unit_test::test_suite;
                                                        test_suite* init_unit_test_suite( int argc, char * argv[] ) {
                                                        test_suite* test= BOOST_TEST_SUITE( "Unit test example 1" );
                                                        return test;
                                                        }
                                                   ]],
                                 [[ return 0;]])],
                                 link_unit_test_framework="yes",link_unit_test_framework="no")
               AC_LANG_POP([C++])
                  ])
                  LDFLAGS="${saved_ldflags}"
               if test "x$link_unit_test_framework" = "xyes"; then
                      BOOST_UNIT_TEST_FRAMEWORK_LIB="-l$ax_lib"
                      AC_SUBST(BOOST_UNIT_TEST_FRAMEWORK_LIB)
                 break
              fi
               done
            fi
         if test "x$link_unit_test_framework" = "xno"; then
            AC_MSG_ERROR(Could not link against $ax_lib !)
         fi
      fi

      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
   fi
])
##### http://autoconf-archive.cryp.to/ax_boost_wserialization.html
#
# SYNOPSIS
#
#   AX_BOOST_WSERIALIZATION
#
# DESCRIPTION
#
#   Test for Serialization library from the Boost C++ libraries. The
#   macro requires a preceding call to AX_BOOST_BASE. Further
#   documentation is available at
#   <http://randspringer.de/boost/index.html>.
#
#   This macro calls:
#
#     AC_SUBST(BOOST_WSERIALIZATION_LIB)
#
#   And sets:
#
#     HAVE_BOOST_WSERIALIZATION
#
# LAST MODIFICATION
#
#   2006-06-15
#
# COPYLEFT
#
#   Copyright (c) 2006 Thomas Porschberg <thomas@randspringer.de>
#
#   Copying and distribution of this file, with or without
#   modification, are permitted in any medium without royalty provided
#   the copyright notice and this notice are preserved.

AC_DEFUN([AX_BOOST_WSERIALIZATION],
[
   AC_ARG_WITH([boost-wserialization],
   AS_HELP_STRING([--with-boost-wserialization@<:@=special-lib@:>@],
                   [use the WSerialization library from boost -  it is possible to specify a certain library for the linker
                        e.g. --with-boost-wserialization=boost_wserialization-gcc-mt-d-1_33_1 ]),
        [
        if test "$withval" = "no"; then
         want_boost="no"
        elif test "$withval" = "yes"; then
            want_boost="yes"
            ax_boost_user_wserialization_lib=""
        else
          want_boost="yes"
           ax_boost_user_wserialization_lib="$withval"
      fi
        ],
        [want_boost="no"]
   )

   if test "x$want_boost" = "xyes"; then
        AC_REQUIRE([AC_PROG_CC])
      CPPFLAGS_SAVED="$CPPFLAGS"
      CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
      export CPPFLAGS
       LDFLAGS_SAVED="$LDFLAGS"
      LDFLAGS="$LDFLAGS $BOOST_LDFLAGS"
      export LDFLAGS

        AC_CACHE_CHECK(whether the Boost::WSerialization library is available,
                  ax_cv_boost_wserialization,
        [AC_LANG_PUSH([C++])
          AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <fstream>
                                     @%:@include <boost/archive/text_oarchive.hpp>
                                                 @%:@include <boost/archive/text_iarchive.hpp>
                                    ]],
                                   [[std::ofstream ofs("filename");
                           boost::archive::text_oarchive oa(ofs);
                            return 0;
                                   ]]),
                   ax_cv_boost_wserialization=yes, ax_cv_boost_wserialization=no)
         AC_LANG_POP([C++])
      ])
      if test "x$ax_cv_boost_wserialization" = "xyes"; then
         AC_DEFINE(HAVE_BOOST_WSERIALIZATION,,[define if the Boost::WSerialization library is available])
         BN=boost_wserialization
            if test "x$ax_boost_user_wserialization_lib" = "x"; then
            for ax_lib in $BN $BN-$CC $BN-$CC-mt $BN-$CC-mt-s $BN-$CC-s \
                              lib$BN lib$BN-$CC lib$BN-$CC-mt lib$BN-$CC-mt-s lib$BN-$CC-s \
                              $BN-mgw $BN-mgw $BN-mgw-mt $BN-mgw-mt-s $BN-mgw-s ; do
                AC_CHECK_LIB($ax_lib, main,
                                 [BOOST_WSERIALIZATION_LIB="-l$ax_lib" AC_SUBST(BOOST_WSERIALIZATION_LIB) link_wserialization="yes" break],
                                 [link_wserialization="no"])
              done
            else
               for ax_lib in $ax_boost_user_wserialization_lib $BN-$ax_boost_user_wserialization_lib; do
                  AC_CHECK_LIB($ax_lib, main,
                                   [BOOST_WSERIALIZATION_LIB="-l$ax_lib" AC_SUBST(BOOST_WSERIALIZATION_LIB) link_wserialization="yes" break],
                                   [link_wserialization="no"])
                  done

            fi
         if test "x$link_wserialization" = "xno"; then
            AC_MSG_ERROR(Could not link against $ax_lib !)
         fi
      fi

      CPPFLAGS="$CPPFLAGS_SAVED"
       LDFLAGS="$LDFLAGS_SAVED"
   fi
])


AC_DEFUN([AX_BOOST_XPRESSIVE],
[
CPPFLAGS_SAVED="$CPPFLAGS"
CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
    AC_CACHE_CHECK(Boost::xpressive header,
             ax_cv_boost_xpressive,    [AC_LANG_PUSH([C++])
       AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/xpressive/xpressive.hpp>]],[[]]),
                   ax_cv_boost_xpressive=yes, ax_cv_boost_xpressive=no)
         AC_LANG_POP([C++])
    ])

    if test "ax_cv_boost_xpressive" = "xyes"; then
      AC_DEFINE(HAVE_BOOST_XPRESSIVE,,[define if the Boost::xpressive is available])
    fi

CPPFLAGS="$CPPFLAGS_SAVED"
])

AC_DEFUN([AX_BOOST_CIRCULARBUFFER],
[
CPPFLAGS_SAVED="$CPPFLAGS"
CPPFLAGS="$CPPFLAGS $BOOST_CPPFLAGS"
    AC_CACHE_CHECK(Boost::circular_buffer header,
             ax_cv_boost_circular_buffer,    [AC_LANG_PUSH([C++])
       AC_COMPILE_IFELSE(AC_LANG_PROGRAM([[@%:@include <boost/circular_buffer.hpp>]],[[]]),
                   ax_cv_boost_circular_buffer=yes, ax_cv_boost_circular_buffer=no)
         AC_LANG_POP([C++])
    ])

    if test "x$ax_cv_boost_circular_buffer" = "xyes"; then
      AC_DEFINE(HAVE_BOOST_CIRCULAR_BUFFER,,[define if the Boost::circular_buffer is available])
    fi
    AM_CONDITIONAL(HAVE_BOOST_CIRCULAR_BUFFER, test "x$ax_cv_boost_circular_buffer" = "xyes")

CPPFLAGS="$CPPFLAGS_SAVED"
])
# CPL_CHECK_CFITSIO(version)
#---------------------------
# Checks for the cfitsio library and header files.
AC_DEFUN([CPL_CHECK_CFITSIO],
[

    AC_REQUIRE([ESO_CHECK_THREADS_POSIX])
        
    cpl_cfitsio_check_version="$1"
    cpl_cfitsio_check_header="fitsio.h"
    cpl_cfitsio_check_lib="libcfitsio.a"

    cpl_cfitsio_incdirs=""
    cpl_cfitsio_libdirs=""
    cpl_cfitsio_includes=""
    cpl_cfitsio_libraries=""

    AC_ARG_WITH(cfitsio,
                AC_HELP_STRING([--with-cfitsio],
                               [location where cfitsio is installed]),
                [
                    cpl_with_cfitsio=$withval
                ])

    AC_ARG_WITH(cfitsio-includes,
                AC_HELP_STRING([--with-cfitsio-includes],
                               [location of the cfitsio header files]),
                cpl_with_cfitsio_includes=$withval)

    AC_ARG_WITH(cfitsio-libs,
                AC_HELP_STRING([--with-cfitsio-libs],
                               [location of the cfitsio library]),
                cpl_with_cfitsio_libs=$withval)

    AC_ARG_ENABLE(cfitsio-test,
                  AC_HELP_STRING([--disable-cfitsio-test],
                                 [disables checks for the cfitsio library and headers]),
                  cpl_enable_cfitsio_test=$enableval,
                  cpl_enable_cfitsio_test=yes)


    # We need libpthread for the folloing tests

    if test -z "$LIBPTHREAD"; then
        AC_MSG_ERROR([POSIX thread library was not found on your system! Please check!])
    fi

    
    AC_MSG_CHECKING([for cfitsio])

    if test "x$cpl_enable_cfitsio_test" = xyes; then

        # Check for the cfitsio includes

        if test -z "$cpl_with_cfitsio_includes"; then
        
            if test -z "$cpl_with_cfitsio"; then
            
                # Try some known system locations
                
                cpl_cfitsio_incdirs="$HOME/sph3rd/include"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /opt/cfitsio/include"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/local/include/libcfitsio0"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/local/include/cfitsio"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/local/include"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/include/libcfitsio0"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/include/cfitsio"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/include"

                test -n "$CFITSIODIR" && \ 
                    cpl_cfitsio_incdirs="$CFITSIODIR/include/libcfitsio0 \
                                         $CFITSIODIR/include/cfitsio \
                                         $CFITSIODIR/include \
                                         $cpl_cfitsio_incdirs"

                test -n "$CPLDIR" && \
                    cpl_cfitsio_incdirs="$CPLDIR/include/libcfitsio0 \
                                         $CPLDIR/include/cfitsio \
                                         $CPLDIR/include \
                                         $cpl_cfitsio_incdirs"

            else

                cpl_cfitsio_incdirs="$cpl_with_cfitsio/include/libcfitsio0"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs $cpl_with_cfitsio/include/cfitsio"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs $cpl_with_cfitsio/include"

            fi
            
        else
            cpl_cfitsio_incdirs="$cpl_with_cfitsio_includes"
        fi

        ESO_FIND_FILE($cpl_cfitsio_check_header, $cpl_cfitsio_incdirs,
                      cpl_cfitsio_includes)


        # Check for the cfitsio library

        if test -z "$cpl_with_cfitsio_libs"; then

            if test -z "$cpl_with_cfitsio"; then
            
                # Try some known system locations
                
                cpl_cfitsio_libdirs="$HOME/sph3rd/lib"
		cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs $HOME/sph3rd/lib64"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /opt/cfitsio/lib"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/local/lib64"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/local/lib"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/local/lib32"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/lib64"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/lib"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/lib32"

                test -n "$CFITSIODIR" && \
                    cpl_cfitsio_libdirs="$CFITSIODIR/lib64 $CFITSIODIR/lib \
                                         $CFITSIODIR/lib32 $cpl_cfitsio_libdirs"

                test -n "$CPLDIR" && \
                    cpl_cfitsio_libdirs="$CPLDIR/lib64 $CPLDIR/lib $CPLDIR/lib32 \
                                         $cpl_cfitsio_libdirs"

            else

                cpl_cfitsio_libdirs="$cpl_with_cfitsio/lib64"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs $cpl_with_cfitsio/lib"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs $cpl_with_cfitsio/lib32"

            fi
            
        else
            cpl_cfitsio_libdirs="$cpl_with_cfitsio_libs"
        fi

        ESO_FIND_FILE($cpl_cfitsio_check_lib, $cpl_cfitsio_libdirs,
                      cpl_cfitsio_libraries)


        if test x"$cpl_cfitsio_includes" = xno || \
            test x"$cpl_cfitsio_libraries" = xno; then
            cpl_cfitsio_notfound=""

            if test x"$cpl_cfitsio_includes" = xno; then
                if test x"$cpl_cfitsio_libraries" = xno; then
                    cpl_cfitsio_notfound="(headers and libraries)"
                else
                    cpl_cfitsio_notfound="(headers)"
                fi
            else
                cpl_cfitsio_notfound="(libraries)"
            fi

            AC_MSG_ERROR([cfitsio $cpl_cfitsio_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_cfitsio_libraries, headers $cpl_cfitsio_includes])
        fi

        # Set up the symbols

        # Add '-lz' to the static library symbol, as distributors apparently
        # remove the libz code from the cfitsio sources.
        
        CFITSIO_INCLUDES="-I$cpl_cfitsio_includes"
        CFITSIO_LDFLAGS="-L$cpl_cfitsio_libraries"
        LIBCFITSIO="-lcfitsio"
        LIBCFITSIO_STATIC="$cpl_cfitsio_libraries/$cpl_cfitsio_check_lib"
        
        # Do not add redundant libraries        
        echo $LIBS | grep -q -e '-lm' || LIBS="-lm $LIBS" 
        

        # Check whether cfitsio can be used

        AC_MSG_CHECKING([whether cfitsio can be used])
        AC_LANG_PUSH(C)
        
        cpl_cfitsio_cflags_save="$CFLAGS"
        cpl_cfitsio_ldflags_save="$LDFLAGS"
        cpl_cfitsio_libs_save="$LIBS"

        CFLAGS="$CFITSIO_INCLUDES $CFLAGS"
        LDFLAGS="$CFITSIO_LDFLAGS $LDFLAGS"
        LIBS="$LIBCFITSIO_STATIC $LIBPTHREAD -lm"
        
        AC_LINK_IFELSE([AC_LANG_PROGRAM(
                       [[
                       #include <fitsio.h>
                       ]],
                       [
                       float v;
                       fits_get_version(&v);
                       ])],
                       [cpl_cfitsio_is_usable="yes"],
                       [cpl_cfitsio_is_usable="no"])

        AC_MSG_RESULT([$cpl_cfitsio_is_usable])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cfitsio_cflags_save"
        LDFLAGS="$cpl_cfitsio_ldflags_save"
        LIBS="$cpl_cfitsio_libs_save"

        if test x"$cpl_cfitsio_is_usable" = xno; then
            AC_MSG_ERROR([Linking with cfitsio failed! Please check architecture!])
        fi
 
        
        # Check cfitsio version

        AC_MSG_CHECKING([for a cfitsio version >= $cpl_cfitsio_check_version])
        
        AC_LANG_PUSH(C)
        
        cpl_cfitsio_cflags_save="$CFLAGS"
        cpl_cfitsio_ldflags_save="$LDFLAGS"
        cpl_cfitsio_libs_save="$LIBS"

        CFLAGS="$CFITSIO_INCLUDES $CFLAGS"
        LDFLAGS="$CFITSIO_LDFLAGS $LDFLAGS"
        LIBS="$LIBCFITSIO_STATIC $LIBPTHREAD -lm"
        
        AC_RUN_IFELSE([AC_LANG_PROGRAM(
                      [[
                      #include <stdio.h>
                      #include <fitsio.h>
                      ]],
                      [
                      int vlib = 0;
                      int vmin = (int)(1000. * $cpl_cfitsio_check_version + 0.5);
                       
                      float v;

                      fits_get_version(&v);
                      vlib = (int)(v * 1000 + 0.5);
                                            
                      FILE* f = fopen("conftest.out", "w");
                      fprintf(f, "%5.3f\n", v);
                      fclose(f);
                      
                      if (vlib < vmin) {
                          return 1;
                      }

                      return 0;
                      ])],
                      [cpl_cfitsio_version="`cat conftest.out`"],
                      [
                       cpl_cfitsio_version="no";
                       cpl_cfitsio_version_found="`cat conftest.out`"
                      ])

        AC_MSG_RESULT([$cpl_cfitsio_version])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cfitsio_cflags_save"
        LDFLAGS="$cpl_cfitsio_ldflags_save"
        LIBS="$cpl_cfitsio_libs_save"

        if test x"$cpl_cfitsio_version" = xno; then
            AC_MSG_ERROR([Installed cfitsio ($cpl_cfitsio_version_found) is too old. Please update to version $cpl_cfitsio_check_version or newer.])
        fi
 
        
        # Check whether cfitsio has large file support
        
        AC_LANG_PUSH(C)
        
        cpl_cfitsio_cflags_save="$CFLAGS"
        cpl_cfitsio_ldflags_save="$LDFLAGS"
        cpl_cfitsio_libs_save="$LIBS"

        CFLAGS="$CFITSIO_INCLUDES $CFLAGS"
        LDFLAGS="$CFITSIO_LDFLAGS $LDFLAGS"
        LIBS="$LIBCFITSIO_STATIC -lm $LIBPTHREAD"
        
        AC_MSG_CHECKING([whether cfitsio provides fits_hdu_getoff()])

        AC_COMPILE_IFELSE([AC_LANG_PROGRAM(
                          [[
                          #include <fitsio.h>
                          ]],
                          [
                          fitsfile f;
                          int sts;
                          fits_get_hduoff(&f, NULL, NULL, NULL, &sts);
                          ])],
                          [cpl_cfitsio_have_fits_get_hduoff="yes"],
                          [cpl_cfitsio_have_fits_get_hduoff="no"])

        AC_MSG_RESULT([$cpl_cfitsio_have_fits_get_hduoff])
        
        AC_MSG_CHECKING([whether cfitsio provides fits_get_hduaddrll()])

        AC_COMPILE_IFELSE([AC_LANG_PROGRAM(
                          [[
                          #include <fitsio.h>
                          ]],
                          [
                          fitsfile f;
                          int sts;
                          fits_get_hduaddrll(&f, NULL, NULL, NULL, &sts);
                          ])],
                          [cpl_cfitsio_have_fits_get_hduaddrll="yes"],
                          [cpl_cfitsio_have_fits_get_hduaddrll="no"])

        AC_MSG_RESULT([$cpl_cfitsio_have_fits_get_hduaddrll])

        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cfitsio_cflags_save"
        LDFLAGS="$cpl_cfitsio_ldflags_save"
        LIBS="$cpl_cfitsio_libs_save"


        # Check whether cfitsio is thread-safe
        
        AC_MSG_CHECKING([whether cfitsio requires libpthread])
        AC_LANG_PUSH(C)
        
        cpl_cfitsio_cflags_save="$CFLAGS"
        cpl_cfitsio_ldflags_save="$LDFLAGS"
        cpl_cfitsio_libs_save="$LIBS"

        CFLAGS="$CFITSIO_INCLUDES $CFLAGS"
        LDFLAGS="$CFITSIO_LDFLAGS $LDFLAGS"
        LIBS="$LIBCFITSIO_STATIC -lm"
        
        
        cpl_cfitsio_provides_pthread="no"
        
        AC_LINK_IFELSE([AC_LANG_PROGRAM(
                       [[
                       #include <fitsio.h>
                       ]],
                       [
                       float v;
                       fits_get_version(&v);
                       ])],
                       [cpl_cfitsio_requires_pthread="no"],
                       [cpl_cfitsio_requires_pthread="undefined"])

		if test x"$cpl_cfitsio_requires_pthread" = xno; then
		
	    	# If libpthread is not required this means either cfitsio is
    		# not compiled with thread support, or the library dependencies
    		# are compiled into cfitsio.

			# Check whether shared library dependencies are present
			# Don't use pthread_mutex_init/destroy here, since glibc
			# provides these symbols too! Sigh!
		
	        AC_LINK_IFELSE([AC_LANG_PROGRAM(
		                   [[
	    	               #include <pthread.h>
    	    	           #include <fitsio.h>
        	    	       ]],
            	    	   [                	       
	                	   float v;
    	            	   pthread_mutexattr_t attrb;
    	            	   pthread_mutexattr_init(&attrb);
            	           fits_get_version(&v);
                		   pthread_mutexattr_destroy(&attrb);
	                	   ])],
	    	               [cpl_cfitsio_provides_pthread="yes"],
    	    	           [cpl_cfitsio_provides_pthread="no"])
            	           
        else
        
            LIBS="$LIBCFITSIO_STATIC -lm $LIBPTHREAD"
        
            AC_LINK_IFELSE([AC_LANG_PROGRAM(
                           [[
                           #include <fitsio.h>
                           ]],
                           [
                           float v;
                           fits_get_version(&v);
                           ])],
                           [cpl_cfitsio_requires_pthread="yes"],
                           AC_MSG_FAILURE([Cannot link with cfitsio! Please check!]))
          
        fi
                       
        AC_MSG_RESULT([$cpl_cfitsio_requires_pthread])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cfitsio_cflags_save"
        LDFLAGS="$cpl_cfitsio_ldflags_save"
        LIBS="$cpl_cfitsio_libs_save"


		AC_MSG_CHECKING([whether cfitsio was compiled with thread support])
		
		if test x"$cpl_cfitsio_requires_pthread" = xyes || \
			test x"$cpl_cfitsio_provides_pthread" = xyes; then
			cpl_cfitsio_is_thread_safe=yes
		else
			cpl_cfitsio_is_thread_safe=no
		fi
		
		AC_MSG_RESULT([$cpl_cfitsio_is_thread_safe])
		
		
        # Set compiler flags and libraries
        
        if test x"$cpl_cfitsio_have_fits_get_hduoff" = xyes || \
          test x"$cpl_cfitsio_have_fits_get_hduaddrll" = xyes; then

            CFLAGS="-D_LARGEFILE_SOURCE=1 -D_FILE_OFFSET_BITS=64 $CFLAGS"
            
            if test x"$cpl_cfitsio_have_fits_get_hduoff"; then
                AC_DEFINE([HAVE_FITS_GET_HDUOFF], [1],
                          [Define if you have the `fits_get_hduoff' function])
            else
                AC_DEFINE([HAVE_FITS_GET_HDUADDRLL],  [1],
                          [Define if you have the `fits_get_hduaddrll' function])
            fi
                    
        fi
                
        if test x"$cpl_cfitsio_requires_pthread" = xyes; then
            echo $LIBS | grep -q -e "$LIBPTHREAD" || LIBS="$LIBPTHREAD $LIBS"
        fi
        
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([cfitsio checks have been disabled! This package may not build!])
        CFITSIO_INCLUDES=""
        CFITSIO_LDFLAGS=""
        LIBCFITSIO=""
        
        cpl_cfitsio_is_thread_safe="undefined"
        cpl_cfitsio_requires_pthread="undefined"
    fi

	AC_CACHE_VAL(cpl_cv_cfitsio_requires_pthread,
	             cpl_cv_cfitsio_requires_pthread=$cpl_cfitsio_requires_pthread)
	AC_CACHE_VAL(cpl_cv_cfitsio_is_thread_safe,
	             cpl_cv_cfitsio_is_thread_safe=$cpl_cfitsio_is_thread_safe)
	             
    AC_SUBST(CFITSIO_INCLUDES)
    AC_SUBST(CFITSIO_LDFLAGS)
    AC_SUBST(LIBCFITSIO)

])


# CPL_CHECK_CEXT(incdirs=[], libdirs=[])
#---------------------------------------
# Checks for the C extension library and header files.
AC_DEFUN([CPL_CHECK_CEXT],
[

    AC_MSG_CHECKING([for libcext])

    cpl_cext_check_header="cxtypes.h"
    cpl_cext_check_lib="libcext.a"

    cpl_cext_incdirs=""
    cpl_cext_libdirs=""

    cpl_cext_includes=""
    cpl_cext_libraries=""


	# Initialize directory search paths with the arguments provided
	
	if test -n "$1"; then
		cpl_cext_incdirs="$1"
	fi

	if test -n "$2"; then
		cpl_cext_libdirs="$2"
	fi


    AC_ARG_WITH(cext,
                AC_HELP_STRING([--with-cext],
                               [location where libcext is installed]),
                [
                    cpl_with_cext=$withval
                ])

    AC_ARG_WITH(cext-includes,
                AC_HELP_STRING([--with-cext-includes],
                               [location of the libcext header files]),
                cpl_with_cext_includes=$withval)

    AC_ARG_WITH(cext-libs,
                AC_HELP_STRING([--with-cext-libs],
                               [location of the libcext library]),
                cpl_with_cext_libs=$withval)

    AC_ARG_ENABLE(cext-test,
                  AC_HELP_STRING([--disable-cext-test],
                                 [disables checks for the libcext library and headers]),
                  cpl_enable_cext_test=$enableval,
                  cpl_enable_cext_test=yes)


    if test "x$cpl_enable_cext_test" = xyes; then
    
        # Check for the libcext includes

        if test -z "$cpl_with_cext_includes"; then
        
            if test -z "$cpl_with_cext"; then
            
            	if test -z "$cpl_cext_incdirs"; then
            
	                # Try some known system locations
                
    	            cpl_cext_incdirs="/opt/cext/include"
        	        cpl_cext_incdirs="$cpl_cext_incdirs /usr/local/include/cext"
            	    cpl_cext_incdirs="$cpl_cext_incdirs /usr/local/include"
                	cpl_cext_incdirs="$cpl_cext_incdirs /usr/include/cext"
                	cpl_cext_incdirs="$cpl_cext_incdirs /usr/include"

                	test -n "$CPLDIR" && \
                    	cpl_cext_incdirs="$CPLDIR/include/cext \
                        	              $CPLDIR/include \
                            	          $cpl_cext_incdirs"

				fi
				
            else

                cpl_cext_incdirs="$cpl_with_cext/include/cext"
                cpl_cext_incdirs="$cpl_cext_incdirs $cpl_with_cext/include"

            fi
            
        else
            cpl_cext_incdirs="$cpl_with_cext_includes"
        fi

        ESO_FIND_FILE($cpl_cext_check_header, $cpl_cext_incdirs,
                      cpl_cext_includes)


        # Check for the libcext library

        if test -z "$cpl_with_cext_libs"; then

            if test -z "$cpl_with_cext"; then
            
            	if test -z "$cpl_cext_libdirs"; then

	                # Try some known system locations

    	            cpl_cext_libdirs="/opt/cext/lib"
                	cpl_cext_libdirs="$cpl_cext_libdirs /usr/local/lib64"
                	cpl_cext_libdirs="$cpl_cext_libdirs /usr/local/lib"
                	cpl_cext_libdirs="$cpl_cext_libdirs /usr/local/lib32"
                	cpl_cext_libdirs="$cpl_cext_libdirs /usr/lib64"
                	cpl_cext_libdirs="$cpl_cext_libdirs /usr/lib"
                	cpl_cext_libdirs="$cpl_cext_libdirs /usr/lib32"

                	test -n "$CPLDIR" && \
                    	cpl_cext_libdirs="$CPLDIR/lib64 \
                    					  $CPLDIR/lib \
                    					  $CPLDIR/lib32 \
                                          $cpl_cext_libdirs"
                                          
				fi
				
            else

                cpl_cext_libdirs="$cpl_with_cext/lib64"
                cpl_cext_libdirs="$cpl_cext_libdirs $cpl_with_cext/lib"
                cpl_cext_libdirs="$cpl_cext_libdirs $cpl_with_cext/lib32"

            fi
            
        else
            cpl_cext_libdirs="$cpl_with_cext_libs"
        fi

        ESO_FIND_FILE($cpl_cext_check_lib, $cpl_cext_libdirs,
                      cpl_cext_libraries)


        if test x"$cpl_cext_includes" = xno || \
            test x"$cpl_cext_libraries" = xno; then
            cpl_cext_notfound=""

            if test x"$cpl_cext_includes" = xno; then
                if test x"$cpl_cext_libraries" = xno; then
                    cpl_cext_notfound="(headers and libraries)"
                else
                    cpl_cext_notfound="(headers)"
                fi
            else
                cpl_cext_notfound="(libraries)"
            fi

            AC_MSG_ERROR([libcext $cpl_cext_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_cext_libraries, headers $cpl_cext_includes])
        fi


        # Set up the symbols

        CX_INCLUDES="-I$cpl_cext_includes"
        CX_LDFLAGS="-L$cpl_cext_libraries"
        LIBCEXT="-lcext"
        
        
        AC_MSG_CHECKING([whether libcext can be used])
        AC_LANG_PUSH(C)
        
        cpl_cext_cflags_save="$CFLAGS"
        cpl_cext_ldflags_save="$LDFLAGS"
        cpl_cext_libs_save="$LIBS"

        CFLAGS="$CX_INCLUDES $CFLAGS"
        LDFLAGS="$CX_LDFLAGS $LDFLAGS"
        LIBS="$LIBCEXT"
        
        AC_LINK_IFELSE([AC_LANG_PROGRAM(
                       [[
                       #include <cxutils.h>
                       ]],
                       [
                       cx_program_set_name("MyProgram");
                       ])],
                       [cpl_cext_is_usable="yes"],
                       [cpl_cext_is_usable="no"])

        AC_MSG_RESULT([$cpl_cext_is_usable])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cext_cflags_save"
        LDFLAGS="$cpl_cext_ldflags_save"
        LIBS="$cpl_cext_libs_save"

        if test x"$cpl_cext_is_usable" = xno; then
            AC_MSG_ERROR([Linking with libcext failed! Please check architecture!])
        fi
        
    else
    
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([libcext checks have been disabled! This package may not build!])
        CX_INCLUDES=""
        CX_LDFLAGS=""
        LIBCEXT=""
        
    fi

    AC_SUBST(CX_INCLUDES)
    AC_SUBST(CX_LDFLAGS)
    AC_SUBST(LIBCEXT)

])


# CPL_CHECK_WCS(version)
#-----------------------
# Checks for the wcs library and header files.
AC_DEFUN([CPL_CHECK_WCS],
[
    AC_MSG_CHECKING([for libwcs])


    cpl_wcs_check_version="$1"
    cpl_wcs_check_header="wcslib/wcslib.h"
    cpl_wcs_check_lib="libwcs.a"

    cpl_wcs_includes=""
    cpl_wcs_libraries=""


    AC_ARG_WITH(wcs,
                AC_HELP_STRING([--with-wcs],
                               [location where wcs is installed]),
                [
                    cpl_with_wcs=$withval
                ])

    AC_ARG_WITH(wcs-includes,
                AC_HELP_STRING([--with-wcs-includes],
                               [location of the libwcs header files]),
                cpl_with_wcs_includes=$withval)

    AC_ARG_WITH(wcs-libs,
                AC_HELP_STRING([--with-wcs-libs],
                               [location of the libwcs library]),
                cpl_with_wcs_libs=$withval)


    # Check for the wcs includes

	if test -z "$cpl_with_wcs_includes"; then    
    	
    	if test -z "$cpl_with_wcs"; then
    
    		# Try some known system locations
    	
    		cpl_wcs_incdirs="/opt/wcslib/include/wcslib"
    		cpl_wcs_incdirs="$cpl_wcs_incdirs /usr/local/include/wcslib"
    		cpl_wcs_incdirs="$cpl_wcs_incdirs /usr/local/include"
    		cpl_wcs_incdirs="$cpl_wcs_incdirs /usr/include/wcslib"
    		cpl_wcs_incdirs="$cpl_wcs_incdirs /usr/include"
    	
        	test -n "$WCSDIR" && \
        		cpl_wcs_incdirs="$WCSDIR/include \
        					     $cpl_wcs_incdirs"
        	
    	else
    	
        	cpl_wcs_incdirs="$cpl_with_wcs/include"
        	
    	fi
    
    else
    
    	cpl_wcs_incdirs="$cpl_with_wcs_includes"
    	
    fi
    
    ESO_FIND_FILE($cpl_wcs_check_header, $cpl_wcs_incdirs, cpl_wcs_includes)


    # Check for the wcs library
    
    if test -z "$cpl_with_wcs_libs"; then
    
    	if test -z "$cpl_with_wcs"; then
    	
    		# Try some known system locations
    		
    		cpl_wcs_libdirs="/opt/wcslib/lib64"
    		cpl_wcs_libdirs="$cpl_wcs_libdirs /opt/wcslib/lib"
    		cpl_wcs_libdirs="$cpl_wcs_libdirs /opt/wcslib/lib32"
    		cpl_wcs_libdirs="$cpl_wcs_libdirs /usr/local/lib64"
    		cpl_wcs_libdirs="$cpl_wcs_libdirs /usr/local/lib"
    		cpl_wcs_libdirs="$cpl_wcs_libdirs /usr/local/lib32"
    		cpl_wcs_libdirs="$cpl_wcs_libdirs /usr/lib64"
    		cpl_wcs_libdirs="$cpl_wcs_libdirs /usr/lib"
    		cpl_wcs_libdirs="$cpl_wcs_libdirs /usr/lib32"
    		
	        test -n "$WCSDIR" && \
    	    	cpl_wcs_libdirs="$WCSDIR/lib64 \
    	    	                 $WCSDIR/lib \
    	    	                 $WCSDIR/lib32 \
    	    	                 $cpl_wcs_libdirs"

        else

        	cpl_wcs_libdirs="$cpl_with_wcs/lib64"
        	cpl_wcs_libdirs="$cpl_wcs_libdirs $cpl_with_wcs/lib"
        	cpl_wcs_libdirs="$cpl_wcs_libdirs $cpl_with_wcs/lib32"

        fi
        	
    else

        cpl_wcs_libdirs="$cpl_with_wcs_libs"

    fi
    
    ESO_FIND_FILE($cpl_wcs_check_lib, $cpl_wcs_libdirs, cpl_wcs_libraries)


    if test x"$cpl_wcs_includes" = xno || \
    	test x"$cpl_wcs_libraries" = xno; then
    	cpl_wcs_notfound=""
    	
        if test x"$cpl_wcs_includes" = xno; then
            if test x"$cpl_wcs_libraries" = xno; then
                cpl_wcs_notfound="(headers and libraries)"
            else
                cpl_wcs_notfound="(headers)"
            fi
        else
            cpl_wcs_notfound="(libraries)"
        fi

        AC_MSG_WARN([libwcs $cpl_wcs_notfound was not found on your system. WCS support will be disabled!])
        
    else
    
        AC_MSG_RESULT([libraries $cpl_wcs_libraries, headers $cpl_wcs_includes])
        
        
        # Setup the symbols

        WCS_INCLUDES="-I$cpl_wcs_includes/wcslib"
        WCS_LDFLAGS="-L$cpl_wcs_libraries"
        LIBWCS="-lwcs -lm"
        LIBWCS_STATIC="$cpl_wcs_libraries/$cpl_wcs_check_lib -lm"

        
        # Check wcs library version

        AC_MSG_CHECKING([for a libwcs version >= $cpl_wcs_check_version])
        
        AC_LANG_PUSH(C)
        
        cpl_wcs_cflags_save="$CFLAGS"
        cpl_wcs_ldflags_save="$LDFLAGS"
        cpl_wcs_libs_save="$LIBS"

        CFLAGS="$WCS_INCLUDES $CFLAGS"
        LDFLAGS="$WCS_LDFLAGS $LDFLAGS"
        LIBS="$LIBWCS_STATIC"
        
        AC_RUN_IFELSE([AC_LANG_PROGRAM(
                      [[
                      #include <stdio.h>
                      #include <wcslib.h>
                      
                      #define stringify(v) stringify_arg(v)
                      #define stringify_arg(v) #v
                      ]],
                      [
                      char vmin[[]] = "$cpl_wcs_check_version";
                      char vlib[[]] = stringify(WCSLIB_VERSION);

                      int min_major = 0;
                      int min_minor = 0;
                      int min_micro = 0;

                      int lib_major = 0;
                      int lib_minor = 0;
                      int lib_micro = 0;

                      sscanf(vmin, "%d.%d.%d", &min_major, &min_minor, &min_micro);
                      sscanf(vlib, "%d.%d.%d", &lib_major, &lib_minor, &lib_micro);
                     
                      FILE* f = fopen("conftest.out", "w");
                      fprintf(f, "%s\n", vlib);
                      fclose(f);
                                          
                      if (lib_major < min_major) {
                          return 1;
                      }
                      else {
                          if (lib_major == min_major) {
                              if (lib_minor <  min_minor) {
                                  return 1;
                              }
                              else {
                                  if (lib_minor == min_minor) {
                                      if (lib_micro < min_micro) {
                                          return 1;
                                      }
                                  }
                              }
                          }
                      }

                      return 0;                      
                      ])],
                      [cpl_wcs_version="`cat conftest.out`"],
                      [
                       cpl_wcs_version="no";
                       cpl_wcs_version_found="`cat conftest.out`"
                      ])

        AC_MSG_RESULT([$cpl_wcs_version])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_wcs_cflags_save"
        LDFLAGS="$cpl_wcs_ldflags_save"
        LIBS="$cpl_wcs_libs_save"


        if test x"$cpl_wcs_version" = xno; then
        
            AC_MSG_WARN([Installed libwcs ($cpl_wcs_version_found) is too old. WCS support will not be available!])
            
            WCS_INCLUDES=""
            WCS_LDFLAGS=""
            LIBWCS=""
            
        else	         
    	    AC_DEFINE_UNQUOTED(CPL_WCS_INSTALLED, 1, [Defined if WCS is available])
 		fi
 
        AC_SUBST(WCS_INCLUDES)
        AC_SUBST(WCS_LDFLAGS)
        AC_SUBST(LIBWCS)
 
    fi

])


# CPL_CHECK_FFTW(version)
#------------------------
# Checks for the wcs library and header files.
AC_DEFUN([CPL_CHECK_FFTW],
[

	cpl_fftw_check_version="$1"
    cpl_fftw_check_header="fftw3.h"
    cpl_fftw_check_lib="libfftw3.a"
    cpl_fftwf_check_lib="libfftw3f.a"

    cpl_fftw_includes=""
    cpl_fftw_libraries=""
    cpl_fftwf_libraries=""

    AC_ARG_WITH(fftw,
                AC_HELP_STRING([--with-fftw],
                               [location where fftw is installed]),
                [
                    cpl_with_fftw=$withval
                ])

    AC_ARG_WITH(fftw-includes,
                AC_HELP_STRING([--with-fftw-includes],
                               [location of the fftw header files]),
                cpl_with_fftw_includes=$withval)

    AC_ARG_WITH(fftw-libs,
                AC_HELP_STRING([--with-fftw-libs],
                               [location of the fftw libraries]),
                cpl_with_fftw_libs=$withval)


    AC_MSG_CHECKING([for fftw (normal-precision)])


    # Check for the fftw includes

	if test -z "$cpl_with_fftw_includes"; then    
    	
    	if test -z "$cpl_with_fftw"; then
    
    		# Try some known system locations
    		cpl_fftw_incdirs="$HOME/sph3rd/include"
    		cpl_fftw_incdirs="$cpl_fftw_incdirs /opt/fftw/include"
    		cpl_fftw_incdirs="$cpl_fftw_incdirs /usr/local/include"
    		cpl_fftw_incdirs="$cpl_fftw_incdirs /usr/include"
    	
        	test -n "$FFTWDIR" && \
        		cpl_fftw_incdirs="$FFTWDIR/include \
        					     $cpl_fftw_incdirs"
        	
    	else
    	
        	cpl_fftw_incdirs="$cpl_with_fftw/include"
        	
    	fi
    
    else
    
    	cpl_fftw_incdirs="$cpl_with_fftw_includes"
    	
    fi
    
    ESO_FIND_FILE($cpl_fftw_check_header, $cpl_fftw_incdirs, cpl_fftw_includes)


    # Check for normal-precision fftw library
    
    if test -z "$cpl_with_fftw_libs"; then
    
    	if test -z "$cpl_with_fftw"; then
    	
    		# Try some known system locations
    		
    		cpl_fftw_libdirs="$HOME/sph3rd/lib"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs $HOME/sph3rd/lib64"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs /opt/fftw/lib64"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs /opt/fftw/lib"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs /opt/fftw/lib32"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs /usr/local/lib64"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs /usr/local/lib"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs /usr/local/lib32"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs /usr/lib64"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs /usr/lib"
    		cpl_fftw_libdirs="$cpl_fftw_libdirs /usr/lib32"
    		
	        test -n "$FFTWDIR" && \
    	    	cpl_fftw_libdirs="$FFTWDIR/lib64 \
    	    	                  $FFTWDIR/lib \
    	    	                  $FFTWDIR/lib32 \
    	    	                  $cpl_fftw_libdirs"

        else

        	cpl_fftw_libdirs="$cpl_with_fftw/lib64"
        	cpl_fftw_libdirs="$cpl_fftw_libdirs $cpl_with_fftw/lib"
        	cpl_fftw_libdirs="$cpl_fftw_libdirs $cpl_with_fftw/lib32"
        fi
        	
    else

        cpl_fftw_libdirs="$cpl_with_fftw_libs"

    fi
    
    ESO_FIND_FILE($cpl_fftw_check_lib, $cpl_fftw_libdirs, cpl_fftw_libraries)

    if test x"$cpl_fftw_includes" = xno || \
    	test x"$cpl_fftw_libraries" = xno; then
    	cpl_fftw_notfound=""
    	
        if test x"$cpl_fftw_includes" = xno; then
            if test x"$cpl_fftw_libraries" = xno; then
                cpl_fftw_notfound="(headers and libraries)"
            else
                cpl_fftw_notfound="(headers)"
            fi
        else
            cpl_fftw_notfound="(libraries)"
        fi

		AC_MSG_RESULT([no])
        AC_MSG_WARN([fftw (normal-precision) $cpl_fftw_notfound was not found on your system.])
        
    else
    
        AC_MSG_RESULT([libraries $cpl_fftw_libraries, headers $cpl_fftw_includes])

	
		# Set up the symbols
	
        FFTW_INCLUDES="-I$cpl_fftw_includes"
        FFTW_LDFLAGS="-L$cpl_fftw_libraries"
        LIBFFTW="-lfftw3 -lm"
        LIBFFTW_STATIC="$cpl_fftw_libraries/$cpl_fftw_check_lib -lm"


        # Check fftw (normal-precision) library version

        AC_MSG_CHECKING([for a fftw (normal-precision) version >= $cpl_fftw_check_version])
        
        AC_LANG_PUSH(C)
        
        cpl_fftw_cflags_save="$CFLAGS"
        cpl_fftw_ldflags_save="$LDFLAGS"
        cpl_fftw_libs_save="$LIBS"

        CFLAGS="$FFTW_INCLUDES $CFLAGS"
        LDFLAGS="$FFTW_LDFLAGS $LDFLAGS"
        LIBS="$LIBFFTW_STATIC"
        
        AC_RUN_IFELSE([AC_LANG_PROGRAM(
                      [[
                      #include <stdio.h>
                      #include <stdlib.h>
                      #include <string.h>
                      #include <fftw3.h>
                      ]],
                      [
                      char vmin[[]] = "$cpl_fftw_check_version";
                      char *vstr    = strdup(fftw_version);
                      char *vlib    = vstr;
                      char *suffix  = NULL;

                      int min_major = 0;
                      int min_minor = 0;
                      int min_micro = 0;

                      int lib_major = 0;
                      int lib_minor = 0;
                      int lib_micro = 0;

                      vlib = strchr(vstr, '-') + 1;
                      suffix = strrchr(vlib, '-');
					  
					  if (suffix) {
                          *suffix = '\0';
					  }
					  
                      sscanf(vmin, "%d.%d.%d", &min_major, &min_minor, &min_micro);
                      sscanf(vlib, "%d.%d.%d", &lib_major, &lib_minor, &lib_micro);
                     
                      FILE* f = fopen("conftest.out", "w");
                      fprintf(f, "%s\n", vlib);
                      fclose(f);
                      
                      free(vstr);
                                          
                      if (lib_major < min_major) {
                          return 1;
                      }
                      else {
                          if (lib_major == min_major) {
                              if (lib_minor <  min_minor) {
                                  return 1;
                              }
                              else {
                                  if (lib_minor == min_minor) {
                                      if (lib_micro < min_micro) {
                                          return 1;
                                      }
                                  }
                              }
                          }
                      }

                      return 0;                      
                      ])],
                      [cpl_fftw_version="`cat conftest.out`"],
                      [
                       cpl_fftw_version="no";
                       cpl_fftw_version_found="`cat conftest.out`"
                      ])

        AC_MSG_RESULT([$cpl_fftw_version])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_fftw_cflags_save"
        LDFLAGS="$cpl_fftw_ldflags_save"
        LIBS="$cpl_fftw_libs_save"

        if test x"$cpl_fftw_version" = xno; then
        
            AC_MSG_WARN([Installed normal-precision fftw ($cpl_fftw_version_found) is too old])
            
            FFTW_INCLUDES=""
            FFTW_LDFLAGS=""
            LIBFFTW=""
            
        else	         
	        AC_DEFINE_UNQUOTED(CPL_FFTW_INSTALLED, 1, [Defined if FFTW (normal-precision) is available])
 		fi
 
        AC_SUBST(FFTW_INCLUDES)
        AC_SUBST(FFTW_LDFLAGS)
        AC_SUBST(LIBFFTW)
			
	fi
	
	
	# Check for single-precision fftw
	
    AC_MSG_CHECKING([for fftw (single-precision)])

    ESO_FIND_FILE($cpl_fftwf_check_lib, $cpl_fftw_libdirs, cpl_fftwf_libraries)

    if test x"$cpl_fftw_includes" = xno || \
    	test x"$cpl_fftwf_libraries" = xno; then
    	cpl_fftw_notfound=""
    	
        if test x"$cpl_fftw_includes" = xno; then
            if test x"$cpl_fftwf_libraries" = xno; then
                cpl_fftw_notfound="(headers and libraries)"
            else
                cpl_fftw_notfound="(headers)"
            fi
        else
            cpl_fftw_notfound="(libraries)"
        fi

		AC_MSG_RESULT([no])
        AC_MSG_WARN([fftw (single-precision) $cpl_fftw_notfound was not found on your system.])
        
    else
    
        AC_MSG_RESULT([libraries $cpl_fftwf_libraries, headers $cpl_fftw_includes])

	
		# Set up the symbols
	
        FFTWF_INCLUDES="-I$cpl_fftw_includes"
        FFTWF_LDFLAGS="-L$cpl_fftwf_libraries"
        LIBFFTWF="-lfftw3f -lm"
        LIBFFTWF_STATIC="$cpl_fftwf_libraries/$cpl_fftwf_check_lib -lm"


        # Check fftw (single-precision) library version

        AC_MSG_CHECKING([for a fftw (single-precision) version >= $cpl_fftw_check_version])
        
        AC_LANG_PUSH(C)
        
        cpl_fftw_cflags_save="$CFLAGS"
        cpl_fftw_ldflags_save="$LDFLAGS"
        cpl_fftw_libs_save="$LIBS"

        CFLAGS="$FFTWF_INCLUDES $CFLAGS"
        LDFLAGS="$FFTWF_LDFLAGS $LDFLAGS"
        LIBS="$LIBFFTWF"
        
        AC_RUN_IFELSE([AC_LANG_PROGRAM(
                      [[
                      #include <stdio.h>
                      #include <stdlib.h>
                      #include <string.h>
                      #include <fftw3.h>
                      ]],
                      [
                      char vmin[[]] = "$cpl_fftw_check_version";
                      char *vstr    = strdup(fftwf_version);
                      char *vlib    = vstr;
                      char *suffix  = NULL;

                      int min_major = 0;
                      int min_minor = 0;
                      int min_micro = 0;

                      int lib_major = 0;
                      int lib_minor = 0;
                      int lib_micro = 0;

                      vlib = strchr(vstr, '-') + 1;
                      suffix = strrchr(vlib, '-');
					  
					  if (suffix) {
                          *suffix = '\0';
					  }
					  
                      sscanf(vmin, "%d.%d.%d", &min_major, &min_minor, &min_micro);
                      sscanf(vlib, "%d.%d.%d", &lib_major, &lib_minor, &lib_micro);
                     
                      FILE* f = fopen("conftest.out", "w");
                      fprintf(f, "%s\n", vlib);
                      fclose(f);
                      
                      free(vstr);
                                          
                      if (lib_major < min_major) {
                          return 1;
                      }
                      else {
                          if (lib_major == min_major) {
                              if (lib_minor <  min_minor) {
                                  return 1;
                              }
                              else {
                                  if (lib_minor == min_minor) {
                                      if (lib_micro < min_micro) {
                                          return 1;
                                      }
                                  }
                              }
                          }
                      }

                      return 0;                      
                      ])],
                      [cpl_fftwf_version="`cat conftest.out`"],
                      [
                       cpl_fftwf_version="no";
                       cpl_fftwf_version_found="`cat conftest.out`"
                      ])

        AC_MSG_RESULT([$cpl_fftwf_version])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_fftw_cflags_save"
        LDFLAGS="$cpl_fftw_ldflags_save"
        LIBS="$cpl_fftw_libs_save"

        if test x"$cpl_fftwf_version" = xno; then
        
            AC_MSG_WARN([Installed single-precision fftw ($cpl_fftw_version_found) is too old!])
            
            FFTWF_INCLUDES=""
            FFTWF_LDFLAGS=""
            LIBFFTWF=""
            
        else	         
	        AC_DEFINE_UNQUOTED(CPL_FFTWF_INSTALLED, 1, [Defined if FFTW (single-precision) is available])
 		fi
 
        AC_SUBST(FFTWF_INCLUDES)
        AC_SUBST(FFTWF_LDFLAGS)
        AC_SUBST(LIBFFTWF)
			
	fi

])


#
# CPL_CREATE_SYMBOLS(build=[])
#-----------------------------
# Sets the Makefile symbols for the CPL libraries. If an argument is
# provided the symbols are setup for building CPL, if no argument is
# given (default) the symbols are set for using the libraries
# for external package development.
AC_DEFUN([CPL_CREATE_SYMBOLS],
[

    if test -z "$1"; then
        LIBCPLCORE='-lcplcore'
        LIBCPLDRS='-lcpldrs'
        LIBCPLUI='-lcplui'
        LIBCPLDFS='-lcpldfs'
    else
        LIBCPLCORE='$(top_builddir)/cplcore/libcplcore.la'
        LIBCPLDRS='$(top_builddir)/cpldrs/libcpldrs.la'
        LIBCPLUI='$(top_builddir)/cplui/libcplui.la'
        LIBCPLDFS='$(top_builddir)/cpldfs/libcpldfs.la'
    fi

   AC_SUBST(LIBCPLCORE)
   AC_SUBST(LIBCPLDRS)
   AC_SUBST(LIBCPLUI)
   AC_SUBST(LIBCPLDFS)

])


# CPL_CHECK_LIBS
#---------------
# Checks for the CPL libraries and header files.
AC_DEFUN([CPL_CHECK_LIBS],
[

    AC_MSG_CHECKING([for CPL])

    cpl_check_cpl_header="cpl.h"
    cpl_check_cpl_lib="libcplcore.a"

    cpl_incdirs=""
    cpl_libdirs=""
    cpl_includes=""
    cpl_libraries=""

    AC_ARG_WITH(cpl,
                AC_HELP_STRING([--with-cpl],
                               [location where CPL is installed]),
                [
                    cpl_with_cpl=$withval
                ])

    AC_ARG_WITH(cpl-includes,
                AC_HELP_STRING([--with-cpl-includes],
                               [location of the CPL header files]),
                cpl_with_cpl_includes=$withval)

    AC_ARG_WITH(cpl-libs,
                AC_HELP_STRING([--with-cpl-libs],
                               [location of the CPL library]),
                cpl_with_cpl_libs=$withval)

    AC_ARG_ENABLE(cpl-test,
                  AC_HELP_STRING([--disable-cpl-test],
                                 [disables checks for the CPL library and headers]),
                  cpl_enable_cpl_test=$enableval,
                  cpl_enable_cpl_test=yes)


    if test "x$cpl_enable_cpl_test" = xyes; then
    
        # Check for the CPL includes

        if test -z "$cpl_with_cpl_includes"; then
        
            if test -z "$cpl_with_cpl"; then
            
                # Try some known system locations
                
                cpl_incdirs="$HOME/sph3rd/include"
                cpl_incdirs="$cpl_incdirs /opt/cpl/include"
                cpl_incdirs="$cpl_incdirs /usr/local/include"
                cpl_incdirs="$cpl_incdirs /usr/include"

                test -n "$CPLDIR" && \
                    cpl_incdirs="$CPLDIR/include \
                                 $cpl_incdirs"

            else

                cpl_incdirs="$cpl_with_cpl/include"

            fi
            
        else
            cpl_incdirs="$cpl_with_cpl_includes"
        fi

        ESO_FIND_FILE($cpl_check_cpl_header, $cpl_incdirs, cpl_includes)


        # Check for the CPL libraries

        if test -z "$cpl_with_cpl_libs"; then

            if test -z "$cpl_with_cpl"; then
            
                # Try some known system locations

                cpl_libdirs="$HOME/sph3rd/lib64"
                cpl_libdirs="$cpl_libdirs /opt/cpl/lib"
                cpl_libdirs="$cpl_libdirs /usr/local/lib64"
                cpl_libdirs="$cpl_libdirs /usr/local/lib"
                cpl_libdirs="$cpl_libdirs /usr/local/lib32"
                cpl_libdirs="$cpl_libdirs /usr/lib64"
                cpl_libdirs="$cpl_libdirs /usr/lib"
                cpl_libdirs="$cpl_libdirs /usr/lib32"

                test -n "$CPLDIR" && \
                    cpl_libdirs="$CPLDIR/lib64 $CPLDIR/lib $CPLDIR/lib32 \
                                 $cpl_libdirs"

            else

                cpl_libdirs="$cpl_with_cpl/lib64"
                cpl_libdirs="$cpl_libdirs $cpl_with_cpl/lib"
                cpl_libdirs="$cpl_libdirs $cpl_with_cpl/lib32"

            fi
            
        else
            cpl_libdirs="$cpl_with_cpl_libs"
        fi

        ESO_FIND_FILE($cpl_check_cpl_lib, $cpl_libdirs, cpl_libraries)


        if test x"$cpl_includes" = xno || test x"$cpl_libraries" = xno; then
            cpl_notfound=""

            if test x"$cpl_includes" = xno; then
                if test x"$cpl_libraries" = xno; then
                    cpl_notfound="(headers and libraries)"
                else
                    cpl_notfound="(headers)"
                fi
            else
                cpl_notfound="(libraries)"
            fi

            AC_MSG_ERROR([CPL $cpl_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_libraries, headers $cpl_includes])
        fi

    	# The libcext headers are required. By default it part of the
    	# CPL installation and it is assumed that the cext headers are
    	# present in the same location as the CPL headers
   	      	   
   	    CPL_CHECK_CEXT($cpl_includes, $cpl_libraries) 


        # Set up the symbols

        CPL_INCLUDES="-I$cpl_includes $CX_INCLUDES $CFITSIO_INCLUDES"
        CPL_LDFLAGS="-L$cpl_libraries $CX_LDFLAGS $CFITSIO_LDFLAGS"
        CPL_CREATE_SYMBOLS
 
 
        AC_MSG_CHECKING([whether CPL can be used])
        AC_LANG_PUSH(C)
        
        cpl_cflags_save="$CFLAGS"
        cpl_ldflags_save="$LDFLAGS"
        cpl_libs_save="$LIBS"

        CFLAGS="$CPL_INCLUDES $CFLAGS"
        LDFLAGS="$CPL_LDFLAGS $LDFLAGS"
        LIBS="$LIBCPLCORE $LIBCEXT $LIBS"
        
        AC_LINK_IFELSE([AC_LANG_PROGRAM(
                       [[
                       #include <cpl.h>
                       ]],
                       [
                       cpl_init(CPL_INIT_DEFAULT);
                       ])],
                       [cpl_is_usable="yes"],
                       [cpl_is_usable="no"])

        AC_MSG_RESULT([$cpl_is_usable])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cflags_save"
        LDFLAGS="$cpl_ldflags_save"
        LIBS="$cpl_libs_save"

        if test x"$cpl_is_usable" = xno; then
            AC_MSG_ERROR([Linking with CPL failed! Please check architecture!])
        fi
        
    else
    
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([CPL checks have been disabled! This package may not build!])
        CPL_INCLUDES=""
        CPL_LDFLAGS=""
        LIBCPLCORE=""
        LIBCPLDRS=""
        LIBCPLUI=""
        LIBCPLDFS=""
        
    fi

    AC_SUBST(CPL_INCLUDES)
    AC_SUBST(CPL_LDFLAGS)
    AC_SUBST(LIBCPLCORE)
    AC_SUBST(LIBCPLDRS)
    AC_SUBST(LIBCPLUI)
    AC_SUBST(LIBCPLDFS)

])
#
# CUNIT_CREATE_SYMBOLS(build=[])
#-----------------------------
# Sets the Makefile symbols for the CUNIT and C extension libraries. If an
# argument is provided the symbols are setup for building CUNIT, if no
# argument is given (default) the symbols are set for using the libraries
# for external package development.
AC_DEFUN([AC_CUNIT_CREATE_SYMBOLS],
[

    if test -z "$1"; then
        LIBCUNIT='-lcunit'
    else
        LIBCUNIT='$(top_builddir)/cunit/libcunit.la'
    fi

   AC_SUBST(LIBCUNIT)
])


# CUNIT_CHECK_LIBS
#---------------
# Checks for the CUNIT libraries and header files.
AC_DEFUN([AC_SEARCH_CUNIT],
[

    AC_MSG_CHECKING([for CUNIT])

    cunit_check_cunit_header="CUnit/Basic.h"
    cunit_check_cunit_lib="libcunit.la"

    cunit_includes=""
    ccunt_libraries=""

    AC_ARG_WITH(cunit,
                AC_HELP_STRING([--with-cunit],
                               [location where CUNIT is installed]),
                [
                    cunit_with_cunit_includes=$withval/include
                    cunit_with_cunit_libs=$withval/lib
                ])

    AC_ARG_WITH(cunit-includes,
                AC_HELP_STRING([--with-cunit-includes],
                               [location of the CUNIT header files]),
                cunit_with_cunit_includes=$withval)

    AC_ARG_WITH(cunit-libs,
                AC_HELP_STRING([--with-cunit-libs],
                               [location of the CUNIT library]),
                cunit_with_cunit_libs=$withval)

    AC_ARG_ENABLE(cunit-test,
                  AC_HELP_STRING([--disable-cunit-test],
                                 [disables checks for the CUNIT library and headers]),
                  cunit_enable_cunit_test=$enableval,
                  cunit_enable_cunit_test=yes)


    if test "x$cunit_enable_cunit_test" = xyes; then

        # Check for the CUNIT includes

        if test -z "$cunit_with_cunit_includes"; then
            cunit_incdirs="/opt/cunit/include \
                         /usr/local/include \
                         /usr/local/cunit/include \
                         /usr/local/include/cunit \
                         /usr/include/cunit \
                         /usr/include \
                         $HOME/sph3rd/include \
                         $HOME/sphereP/include \
                         $CUNIT_SEARCH_LOCAL/include"

            test -n "$CUNITDIR" && cunit_incdirs="$CUNITDIR/include $cunit_incdirs"
        else
            cunit_incdirs="$cunit_with_cunit_includes"
        fi

        ESO_FIND_FILE($cunit_check_cunit_header, $cunit_incdirs, cunit_includes)


        # Check for the CUNIT libraries

        if test -z "$cunit_with_cunit_libs"; then
            cunit_libdirs="/opt/cunit/lib \
                         /usr/local/lib \
                         /usr/local/cunit/lib \
                         /usr/lib \
                         $HOME/sph3rd/lib \
                         $HOME/sphereP/lib \
                         $HOME/sph3rd/lib64 \
                         $CUNIT_SEARCH_LOCAL/lib"

            test -n "$CUNITDIR" && cunit_libdirs="$CUNITDIR/lib $cunit_libdirs"
        else
            cunit_libdirs="$cunit_with_cunit_libs"
        fi

        ESO_FIND_FILE($cunit_check_cunit_lib, $cunit_libdirs, cunit_libraries)


        if test x"$cunit_includes" = xno || test x"$cunit_libraries" = xno; then
            cunit_notfound=""

            if test x"$cunit_includes" = xno; then
                if test x"$cunit_libraries" = xno; then
                    cunit_notfound="(headers and libraries)"
                else            
                    cunit_notfound="(headers)"
                fi
            else
                cunit_notfound="(libraries)"
            fi

            AC_MSG_ERROR([CUNIT $cunit_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cunit_libraries, headers $cunit_includes])
        fi

        # Set up the symbols

        CUNIT_INCLUDES="-I$cunit_includes"
        CUNIT_LDFLAGS="-L$cunit_libraries"
        CUNIT_CFLAGS="-I$cunit_includes"

        CUNIT_CREATE_SYMBOLS
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([CUNIT checks have been disabled! This package may not build!])
        CUNIT_INCLUDES=""
        CUNIT_LDFLAGS=""
        LIBCUNIT=""
    fi

    AC_SUBST(CUNIT_INCLUDES)
    AC_SUBST(CUNIT_LDFLAGS)
    AC_SUBST(LIBCUNIT)

])
AC_DEFUN([AB_DOCU],
[

AC_ARG_ENABLE(
        Documentation,
        AC_HELP_STRING([--enable-Documentation], [Build the documentation (default off)]),
        ac_request_docu=$enableval,
        ac_request_docu=no)

if test "x$ac_request_docu" = xyes; then

	AC_CHECK_PROGS([ac_doxygen],doxygen,no)
	if test "x$ac_doxygen" != xno; then
      		AC_SUBST(DOXYGEN, "$ac_doxygen")
	else
	      	AC_WARN(doxygen not (properly) installed.)
	fi

        if test "x$ac_doxygen" = xyes; then
                ac_compile_html=yes
        fi

	AC_CHECK_PROGS([ac_lyx],lyx, no)
	if test "x$ac_lyx" != xno; then
      		AC_SUBST(LYX, "$ac_lyx")
	else
      		AC_WARN(lyx not (properly) installed.)
	fi


fi

AM_CONDITIONAL(COND_HTMLDOC, [test "x$ac_compile_html" = xyes])
AM_CONDITIONAL(HAVE_LYX, test x$ac_lyx = xtrue)
AM_CONDITIONAL(HAVE_DOXYGEN, test x$ac_doxygen = xtrue)

])

# ESO_PROG_CC_FLAG(FLAG, [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
#-----------------------------------------------------------------
AC_DEFUN([ESO_PROG_CC_FLAG],
[
    AC_REQUIRE([AC_PROG_CC])

    flag=`echo $1 | sed 'y%.=/+-%___p_%'`
    AC_CACHE_CHECK([whether $CC supports -$1],
                   [eso_cv_prog_cc_$flag],
                   [
                       eval "eso_cv_prog_cc_$flag=no"
                       AC_LANG_PUSH(C)

                       echo 'int main() { return 0; }' >conftest.$ac_ext

                       try_compile="`$CC -$1 -c conftest.$ac_ext 2>&1`"
                       if test -z "$try_compile"; then
                           try_link="`$CC -$1 -o conftest$ac_exeext \
                                    conftest.$ac_ext 2>&1`"
                           if test -z "$try_link"; then
                               eval "eso_cv_prog_cc_$flag=yes"
                           fi
                       fi
                       rm -f conftest*

                       AC_LANG_POP(C)
                   ])

    if eval "test \"`echo '$eso_cv_prog_cc_'$flag`\" = yes"; then
        :
        $2
    else
        :
        $3
    fi
])


# ESO_ENABLE_DEBUG(debug=no)
#---------------------------
AC_DEFUN([ESO_ENABLE_DEBUG],
[
    AC_REQUIRE([AC_PROG_CC])

    AC_ARG_ENABLE(debug,
                  AC_HELP_STRING([--enable-debug],
                                 [creates debugging code [[default=$1]]]),
                  eso_enable_debug=$enableval, eso_enable_debug=$1)

    AC_CACHE_CHECK([whether debugging code should be created],
                   eso_cv_enable_debug,
                   eso_cv_enable_debug=$eso_enable_debug)

    if test x"$eso_cv_enable_debug" = xyes; then

        eso_clean_CFLAGS="`echo $CFLAGS | sed -e 's/-O[[0-9]]//g' \
                                              -e 's/-g[[0-9]]//g' \
                                              -e 's/-g[[a-z,A-Z]]* / /g' \
                                              -e 's/-[[Og]]//g'`"

        ESO_PROG_CC_FLAG([g3], [CFLAGS="$CFLAGS -g3"])

        if test x"$eso_cv_prog_cc_g3" = xyes; then
            CFLAGS="-g3"
        else
            if test x"$ac_cv_prog_cc_g" = xyes; then
                CFLAGS="-g"
            else
                CFLAGS=""
            fi
        fi

        ESO_PROG_CC_FLAG([ggdb], [CFLAGS="$CFLAGS -ggdb"])
        ESO_PROG_CC_FLAG([O0], [CFLAGS="$CFLAGS -O0"])
        ESO_PROG_CC_FLAG([rdynamic], [CFLAGS="$CFLAGS -rdynamic"])
        ESO_PROG_CC_FLAG([Wall], [CFLAGS="$CFLAGS -Wall"])
        ESO_PROG_CC_FLAG([W], [CFLAGS="$CFLAGS -W"])

        CFLAGS="$CFLAGS $eso_clean_CFLAGS"
        ESO_DEBUG_FLAGS="-DESO_ENABLE_DEBUG"
    else
        ESO_DEBUG_FLAGS="-DNDEBUG"
    fi

    AC_SUBST(ESO_DEBUG_FLAGS)
])


# ESO_ENABLE_STRICT(strict=no)
#-----------------------------
AC_DEFUN([ESO_ENABLE_STRICT],
[
	AC_REQUIRE([AC_PROG_EGREP])
    AC_REQUIRE([AC_PROG_CC])

    AC_ARG_ENABLE(strict,
                  AC_HELP_STRING([--enable-strict],
                                 [compiles with strict compiler options (may not work!) [[default=$1]]]),
                  eso_enable_strict=$enableval, eso_enable_strict=$1)

    AC_CACHE_CHECK([whether strict compiler options should be used],
                   eso_cv_enable_strict,
                   eso_cv_enable_strict=$eso_enable_strict)


    if test x"$eso_cv_enable_strict" = xyes; then
    
    	eso_enable_strict_std_set=no
    	
        if test -n "$CFLAGS"; then
            echo $CFLAGS | $EGREP '(\-std=|-ansi)' >/dev/null 2>&1
            if test x"$?" = x0; then
            	eso_enable_strict_std_set=yes
            fi
        fi
        
        if test x"$eso_enable_strict_std_set" = xno; then
        	ESO_PROG_CC_FLAG([std=c99], [CFLAGS="$CFLAGS -std=c99"])
        fi
        
        ESO_PROG_CC_FLAG([pedantic], [CFLAGS="$CFLAGS -pedantic"])
        
    fi
])


# ESO_ENABLE_PROFILE(profile=no)
#-----------------------------
AC_DEFUN([ESO_ENABLE_PROFILE],
[
    AC_REQUIRE([AC_PROG_CC])

    AC_ARG_ENABLE(profile,
                  AC_HELP_STRING([--enable-profile],
                                 [compiles with compiler options necessary for profiling (may not work!) [[default=$1]]]),
                  eso_enable_profile=$enableval, eso_enable_profile=$1)

    AC_CACHE_CHECK([whether profiling compiler options should be used],
                   eso_cv_enable_profile,
                   eso_cv_enable_profile=$eso_enable_profile)


    if test x"$eso_cv_enable_profile" = xyes; then
        ESO_PROG_CC_FLAG([pg], [CFLAGS="$CFLAGS -pg"])
        ESO_PROG_CC_FLAG([g], [CFLAGS="$CFLAGS -g"])
        ESO_PROG_CC_FLAG([static-libgcc], [CFLAGS="$CFLAGS -static-libgcc"])

        AC_ENABLE_SHARED(no)
        AC_ENABLE_STATIC(yes)
    fi
])


# ESO_CHECK_DOCTOOLS
#-------------------
AC_DEFUN([ESO_CHECK_DOCTOOLS],
[
    AC_ARG_VAR([DOXYGEN], [doxygen command])
    AC_PATH_PROG([DOXYGEN], [doxygen])

    AC_ARG_VAR([LATEX], [latex command])
    AC_PATH_PROG([LATEX], [latex])


    if test -z "${DOXYGEN}"; then
        DOXYGEN=":"
    fi

    if test -z "${LATEX}"; then
        LATEX=":"
    fi

])


# ESO_PROG_AR
#------------
# Checks if ar is in the path
AC_DEFUN([ESO_PROG_AR],
[
    AC_CHECK_PROG(AR, ar, ar, NONE)

    if test x"$AR" = xNONE; then
        AC_MSG_ERROR([Cannot find \'ar\'])
    fi

])


# ESO_CHECK_EXTRA_LIBS
#---------------------
# Check for non-standard headers and libraries
AC_DEFUN([ESO_CHECK_EXTRA_LIBS],
[

    AC_ARG_WITH(extra-includes,
                AC_HELP_STRING([--with-extra-includes=DIR],
                               [adds non standard include paths]),
                eso_with_extra_includes=$withval, eso_with_extra_includes=NONE)

    AC_ARG_WITH(extra-libs,
                AC_HELP_STRING([--with-extra-libs=DIR],
                              [adds non standard library paths]),
                eso_with_extra_libs=$withval, eso_with_extra_libs=NONE)

    AC_MSG_CHECKING([for extra includes])
    AC_CACHE_VAL([eso_cv_with_extra_includes],
                 [
                     eso_cv_with_extra_includes=$eso_with_extra_includes
                 ])

    if test x"$eso_cv_with_extra_includes" != xNONE; then
        eso_save_IFS=$IFS
        IFS=':'

        for dir in $eso_cv_with_extra_includes; do
            EXTRA_INCLUDES="$EXTRA_INCLUDES -I$dir"
        done

        IFS=$eso_save_IFS
        AC_MSG_RESULT(added)
    else
        AC_MSG_RESULT(no)
    fi


    AC_MSG_CHECKING([for extra libs])
    AC_CACHE_VAL([eso_cv_with_extra_libs],
                 [
                     eso_cv_with_extra_libs=$eso_with_extra_libs
                 ])

    if test x"$eso_cv_with_extra_libs" != xNONE; then
        eso_save_IFS=$IFS
        IFS=':'

        for dir in $eso_cv_with_extra_libs; do
            EXTRA_LDFLAGS="$EXTRA_LDFLAGS -L$dir"
        done

        IFS=$eso_save_IFS
        AC_MSG_RESULT(added)
    else
        AC_MSG_RESULT(no)
    fi

])


# ESO_CHECK_THREADS_POSIX
#------------------------
# Check whether the POSIX threads are available. The cached result is
# set to 'yes' if either the compiler supports the '-pthread' flag, or linking
# with the POSIX thread library works, and the header file defining the POSIX
# threads symbols is present. If POSIX threads are not supported, the
# result is set to 'no'. Whether the compiler supports POSIX threads,
# or whether the library, and the header are available is stored in cache
# variables.  
AC_DEFUN([ESO_CHECK_THREADS_POSIX],
[
    AC_REQUIRE([AC_PROG_CC])

    ESO_PROG_CC_FLAG([pthread], [], [])
    
    AC_CHECK_LIB([pthread], [pthread_create],
                 [eso_threads_have_libpthread=yes],
                 [eso_threads_have_libpthread=no])

    AC_CHECK_HEADER([pthread.h],
                    [eso_threads_have_pthread_h=yes],
                    [eso_threads_have_pthread_h=no])

    if test x"$eso_threads_have_pthread_h" != xyes; then
        eso_threads_posix=no
    else
        if test x"$eso_threads_have_libpthread" != xyes && \
          test x"$eso_cv_prog_cc_pthread" != xyes; then
            eso_threads_posix=no
        else
            eso_threads_posix=yes
        fi
    fi
    
    
    # Setup the POSIX thread symbols

    if test x"$eso_threads_have_pthread_h" = xyes; then
        AC_DEFINE([HAVE_PTHREAD_H], [1],
                  [Define to 1 if you have <pthread.h>.])
    fi
    
    if test x"$eso_threads_posix" = xyes; then
    
        if test x"$eso_cv_prog_cc_pthread" = xyes; then
            PTHREAD_CFLAGS="-pthread"
        else
            PTHREAD_CFLAGS=""
        fi
        
        if test x"$eso_threads_have_libpthread" = xyes; then
            LIBPTHREAD="-lpthread"
        else
            LIBPTHREAD=""
        fi
        
    fi  

    AC_CACHE_VAL(eso_cv_threads_posix_header,
                 eso_cv_threads_posix_header=$eso_threads_have_pthread_h)          
    AC_CACHE_VAL(eso_cv_threads_posix_lib,
                 eso_cv_threads_posix_lib=$eso_threads_have_libpthread)          
    AC_CACHE_VAL(eso_cv_threads_posix_flags,
                 eso_cv_threads_posix_flags=$eso_cv_prog_cc_pthread)          
    AC_CACHE_VAL(eso_cv_threads_posix,
                 eso_cv_threads_posix=$eso_threads_posix)

    AC_SUBST(PTHREAD_CFLAGS)
    AC_SUBST(LIBPTHREAD)
    
])


# ESO_CHECK_FUNC(FUNCTION, INCLUDES, SYMBOL)
#-------------------------------------------
# Checks whether a function is available and declared.
AC_DEFUN([ESO_CHECK_FUNC],
[

    AC_LANG_PUSH(C)

    AC_CHECK_DECL($1, [], [], [$2])

    eso_save_CFLAGS="$CFLAGS"

    if test x"$GCC" = xyes; then
        CFLAGS="$CFLAGS -pedantic-errors"
    fi

    AC_CHECK_FUNC($1)

    CFLAGS="$eso_save_CFLAGS"

    AC_LANG_POP(C)

    if test x"$ac_cv_have_decl_$1" = xyes &&
       test x"$ac_cv_func_$1" = xyes; then
        AC_DEFINE($3)
    fi

])


# ESO_FUNC_VSNPRINTF_C99
#-----------------------
# Check whether vsnprintf() has C99 semantics.
AC_DEFUN([ESO_FUNC_VSNPRINTF_C99],
[

    AH_TEMPLATE([HAVE_VSNPRINTF_C99],
                [Define if you have the C99 `vsnprintf' function.])

    AC_CACHE_CHECK([whether vsnprintf has C99 semantics],
                   [eso_cv_func_vsnprintf_c99],
                   [
                       AC_LANG_PUSH(C)

                       eso_cppflags_save="$CPPFLAGS"
                       eso_cflags_save="$CFLAGS"
                       eso_ldflags_save="$LDFLAGS"
                       eso_libs_save="$LIBS"

                      if test x$GCC = xyes; then
                          CFLAGS="$CFLAGS -pedantic-errors"
                          CPPFLAGS="$CPPFLAGS $CFLAGS"
                      fi

                       AC_RUN_IFELSE([[
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

int
doit(char * s, ...)
{
    char buffer[32];
    va_list args;
    int q, r;

    va_start(args, s);
    q = vsnprintf(NULL, 0, s, args);
    r = vsnprintf(buffer, 5, s, args);
    va_end(args);

    if (q != 7 || r != 7)
      exit(1);

    exit(0);
}

int
main(void)
{
    doit((char*)"1234567");
    exit(1);
}
                                     ]],
                                     eso_cv_func_vsnprintf_c99=yes,
                                     eso_cv_func_vsnprintf_c99=no,
                                     eso_cv_func_vsnprintf_c99=no)

                       CPPFLAGS="$eso_cppflags_save"
                       CFLAGS="$eso_cflags_save"
                       LDFLAGS="$eso_ldflags_save"
                       LIBS="$eso_libs_save"

                       AC_LANG_POP(C)
                   ])

# Note that the default is to be pessimistic in the case of cross compilation.
# If you know that the target has a C99 vsnprintf(), you can get around this
# by setting eso_func_vsnprintf_c99 to yes, as described in the Autoconf
# manual.

    if test x$eso_cv_func_vsnprintf_c99 = xyes; then
        AC_DEFINE(HAVE_VSNPRINTF_C99)
    fi

])


# ESO_CHECK_PRINTF_FORMATS
#-------------------------
# Checks for printf() format peculiarities.
AC_DEFUN([ESO_CHECK_PRINTF_FORMATS],
[

    # Check if string format for NULL is `(null)'

    AH_TEMPLATE([HAVE_PRINTF_STR_FMT_NULL],
                [Define if printf outputs `(null)' when printing NULL using
                 `%s'])

    AC_RUN_IFELSE([[
#include <stdio.h>
#include <string.h>

int main()
{
    char s[128];

    sprintf(s, "%s", NULL);
    return strncmp(s, "(null)", 6) ? 1 : 0;
}
                  ]],
                  eso_have_printf_str_format_null=yes,
                  eso_have_printf_str_format_null=no,
                  eso_have_printf_str_format_null=no
                 )

    if test x$eso_have_printf_str_format_null = xyes; then
        AC_DEFINE(HAVE_PRINTF_STR_FMT_NULL)
    fi


    # Check if pointer format for NULL is `(nil)'

    AH_TEMPLATE([HAVE_PRINTF_PTR_FMT_NIL],
                [Define if printf outputs `(nil)' when printing NULL using
                 `%p'])

    AC_RUN_IFELSE([[
#include <stdio.h>
#include <string.h>

int main()
{
    char s[128];

    sprintf(s, "%p", NULL);
    return strncmp(s, "(nil)", 5) ? 1 : 0;
}
                  ]],
                  eso_have_printf_ptr_format_nil=yes,
                  eso_have_printf_ptr_format_nil=no,
                  eso_have_printf_ptr_format_nil=no
                 )

    if test x$eso_have_printf_ptr_format_nil = xyes; then
        AC_DEFINE(HAVE_PRINTF_PTR_FMT_NIL)
    fi


    # Check if output for `%p' is the same as `%#x'

    AH_TEMPLATE([HAVE_PRINTF_PTR_FMT_ALTERNATE],
                [Define if printf format `%p' produces the same output as
                 `%#x' or `%#lx'])

    AC_RUN_IFELSE([[
#include <stdio.h>
#include <string.h>

int main()
{
    char s1[128], s2[128];

    sprintf(s1, "%p", s1);
    sprintf(s2, "%#x", s1);
    return strncmp(s1, s2, 3) ? 1 : 0;
}
                  ]],
                  eso_have_printf_ptr_format_alternate=yes,
                  eso_have_printf_ptr_format_alternate=no,
                  eso_have_printf_ptr_format_alternate=no
                 )

    if test x$eso_have_printf_ptr_format_alternate = xyes; then
        AC_DEFINE(HAVE_PRINTF_PTR_FMT_ALTERNATE)
    fi


    # Check if pointers are treated as signed

    AH_TEMPLATE([HAVE_PRINTF_PTR_FMT_SIGNED],
                [Define if printf treats pointers as signed when using a sign
                 flag])

    AC_RUN_IFELSE([[
#include <stdio.h>

int main()
{
    char s[128];

    sprintf(s, "%+p", s);
    return s[0] == '+' ? 0 : 1;
}
                  ]],
                  eso_have_printf_ptr_format_signed=yes,
                  eso_have_printf_ptr_format_signed=no,
                  eso_have_printf_ptr_format_signed=no
                 )

    if test x$eso_have_printf_ptr_format_signed = xyes; then
        AC_DEFINE(HAVE_PRINTF_PTR_FMT_SIGNED)
    fi


    # Check if default precision for conversion specifier `g' is 1 (as
    # required by ISO C) or 6.

    AH_TEMPLATE([HAVE_PRINTF_FLT_FMT_G_STD],
                [Define if printf default precision for format `g' is 1
                 (ISO C standard) or 6])

    AC_RUN_IFELSE([[
#include <stdio.h>

int main()
{
    char s1[128], s2[128];
    int n1, n2;

    sprintf(s1, "%g%n", 1.123456, &n1);
    sprintf(s2, "%.1g%n", 1.123456, &n2);
    return n1 > n2 ? 1 : 0;
}
                  ]],
                  eso_have_printf_flt_format_g_std=yes,
                  eso_have_printf_flt_format_g_std=no,
                  eso_have_printf_flt_format_g_std=no
                 )

    if test x$eso_have_printf_flt_format_g_std = xyes; then
        AC_DEFINE(HAVE_PRINTF_FLT_FMT_G_STD)
    fi

])


# ESO_FUNC_VSNPRINTF
#-------------------
# Checks for vsnprintf and snprintf declaration and function.
AC_DEFUN([ESO_FUNC_VSNPRINTF],
[

    eso_compile_snprintf=no

    AH_TEMPLATE([HAVE_VSNPRINTF],
                [Define if you have the `vsnprintf' function])
    ESO_CHECK_FUNC(vsnprintf, [
#include <stdio.h>
#include <stdarg.h>
                              ], HAVE_VSNPRINTF)

    if test x$ac_cv_func_vsnprintf = xyes &&
       test x$ac_cv_have_decl_vsnprintf = xyes; then

        ESO_FUNC_VSNPRINTF_C99

        if test x$eso_cv_func_vsnprintf_c99 != xyes; then
            eso_compile_snprintf=yes
        fi

    else
        eso_compile_snprintf=yes
    fi

    if test x$eso_compile_snprintf = xyes; then
        if test -n "$LIBTOOL"; then
            SNPRINTF=snprintf.lo
        else
            SNPRINTF=snprintf.$ac_objext
        fi
    fi

    AC_SUBST(SNPRINTF)

    # The symbols defined by the following macro are only needed to setup the
    # vsnprintf() replacement. May be useless if the vsnprintf implementation
    # changes.
    ESO_CHECK_PRINTF_FORMATS

    AH_TEMPLATE([HAVE_SNPRINTF],
                [Define if you have the `snprintf' function])
    ESO_CHECK_FUNC(snprintf, [#include <stdio.h>], HAVE_SNPRINTF)

])


# ESO_FUNC_VASPRINTF
#-------------------
# Checks for vasprintf declaration and function.
AC_DEFUN([ESO_FUNC_VASPRINTF],
[

    AH_TEMPLATE([HAVE_VASPRINTF],
                [Define if you have the `vasprintf' function])
    ESO_CHECK_FUNC(vasprintf, [
#include <stdio.h>
#include <stdarg.h>
                              ], HAVE_VASPRINTF)

])


# ESO_FUNC_FPATHCONF
#-------------------
# Checks for fpathconf declaration and function.
AC_DEFUN([ESO_FUNC_FPATHCONF],
[

    AH_TEMPLATE([HAVE_FPATHCONF],
                [Define if you have the `fpathconf' function])
    ESO_CHECK_FUNC(fpathconf, [#include <unistd.h>], HAVE_FPATHCONF)

    # If we have fpathconf we should also have pathconf, but who knows.
    AH_TEMPLATE([HAVE_PATHCONF],
                [Define if you have the `pathconf' function])
    ESO_CHECK_FUNC(pathconf, [#include <unistd.h>], HAVE_PATHCONF)

])


# ESO_FUNC_SYSCONF
#-----------------
# Checks for sysconf declaration and function.
AC_DEFUN([ESO_FUNC_SYSCONF],
[

    AH_TEMPLATE([HAVE_SYSCONF],
                [Define if you have the `sysconf' function])
    ESO_CHECK_FUNC(sysconf, [#include <unistd.h>], HAVE_SYSCONF)

])


# ESO_FUNC_GETOPT
#----------------
# Checks for GNU getopt_long declaration and function.
AC_DEFUN([ESO_FUNC_GETOPT],
[

    AH_TEMPLATE([HAVE_GETOPT_LONG],
                [Define if you have the `getopt_long' function])

    ESO_CHECK_FUNC(getopt_long, [#include <getopt.h>], HAVE_GETOPT_LONG)

    if test x"$ac_cv_func_getopt_long" = xno ||
       test x"$eso_cv_have_decl_getopt_long" = xno; then
        if test -n "$LIBTOOL"; then
            GETOPT="getopt.lo getopt1.lo"
        else
            GETOPT="getopt.$ac_objext getopt1.$ac_objext"
        fi
    fi

    AC_SUBST(GETOPT)


])


# ESO_FUNC_GETPWUID
#------------------
# Checks for getpwuid declaration and function.
AC_DEFUN([ESO_FUNC_GETPWUID],
[

    AH_TEMPLATE([HAVE_GETPWUID],
                [Define if you have the `getpwuid' function])

    ESO_CHECK_FUNC(getpwuid, [#include <pwd.h>], HAVE_GETPWUID)

])


# ESO_FUNC_GETUID
#----------------
AC_DEFUN([ESO_FUNC_GETUID],
[

    AH_TEMPLATE([HAVE_GETUID],
                [Define if you have the `getuid' function])

    ESO_CHECK_FUNC(getuid, [#include <unistd.h>], HAVE_GETUID)

])


# ESO_FUNC_LSTAT
#---------------
AC_DEFUN([ESO_FUNC_LSTAT],
[

    AH_TEMPLATE([HAVE_LSTAT],
                [Define if you have the `lstat' function])

    ESO_CHECK_FUNC(lstat, [#include <sys/stat.h>], HAVE_LSTAT)

])


# ESO_FUNC_STRDUP
#----------------
AC_DEFUN([ESO_FUNC_STRDUP],
[

    AH_TEMPLATE([HAVE_STRDUP],
                [Define if you have the `strdup' function])

    ESO_CHECK_FUNC(strdup, [#include <string.h>], HAVE_STRDUP)

    AH_BOTTOM([
#ifndef HAVE_STRDUP
#  define strdup  cx_strdup
#endif
              ])
])


# ESO_FUNC_STPCPY
#----------------
AC_DEFUN([ESO_FUNC_STPCPY],
[

    AH_TEMPLATE([HAVE_STPCPY],
                [Define if you have the `stpcpy' function])

    ESO_CHECK_FUNC(stpcpy, [#include <stpcpy.h>], HAVE_STPCPY)

])


# ESO_FUNC_SYMLINK
#-----------------
AC_DEFUN([ESO_FUNC_SYMLINK],
[

    AH_TEMPLATE([HAVE_SYMLINK],
                [Define if you have the `symlink' function])

    ESO_CHECK_FUNC(symlink, [#include <unistd.h>], HAVE_SYMLINK)

])


# ESO_FUNC_WORDEXP
#-----------------
AC_DEFUN([ESO_FUNC_WORDEXP],
[

    AH_TEMPLATE([HAVE_WORDEXP],
                [Define if you have the `wordexp' function])

    ESO_CHECK_FUNC(wordexp, [#include <wordexp.h>], HAVE_WORDEXP)

])


# ESO_FUNC_GETTIMEOFDAY
#----------------------
AC_DEFUN([ESO_FUNC_GETTIMEOFDAY],
[

    AH_TEMPLATE([HAVE_GETTIMEOFDAY],
                [Define if you have the `gettimeofday' function])

    ESO_CHECK_FUNC(gettimeofday,
                   [
                       #include <unistd.h>
                       #include <sys/time.h>
                   ],
                   HAVE_GETTIMEOFDAY)
])


# ESO_FUNC_VA_COPY(symbol)
#-------------------------
# Check for an implementation of va_copy(). The argument which must be
# given is the preprocessor symbol that is defined to be either va_copy
# or __va_copy depending on the available function, provided that an
# implementation of va_copy is available at all.
AC_DEFUN([ESO_FUNC_VA_COPY],
[

    # Check for all three va_copy possibilities, so we get
    # all results in config.log for bug reports.

    # Check for availability of va_copy(). This is ISO C. Available with
    # gcc since version 3.0.
    AC_CACHE_CHECK([for an implementation of va_copy()], eso_cv_have_va_copy,
                   [
                       AC_RUN_IFELSE([
#include <stdarg.h>

void f(int i, ...)
{
    va_list args1, args2;
    va_start (args1, i);
    va_copy (args2, args1);

    if (va_arg (args2, int) != 42 || va_arg (args1, int) != 42)
        exit (1);

    va_end (args1);
    va_end (args2);
}

int main() {
  f(0, 42);
  return 0;
}
                                     ],
                                     eso_cv_have_va_copy=yes,
                                     eso_cv_have_va_copy=no,
                                     eso_cv_have_va_copy=no)
                   ])


    # Check for availability of __va_copy(). Some compilers provide
    # this. Available with gcc since version 2.8.1.
    AC_CACHE_CHECK([for an implementation of __va_copy()],
                   eso_cv_have__va_copy,
                   [
                       AC_RUN_IFELSE([
#include <stdarg.h>

void f(int i, ...)
{
    va_list args1, args2;

    va_start (args1, i);
    __va_copy (args2, args1);

    if (va_arg (args2, int) != 42 || va_arg (args1, int) != 42)
        exit (1);

    va_end (args1);
    va_end (args2);
}

int main()
{
    f(0, 42);
    return 0;
}
                                     ],
                                     eso_cv_have__va_copy=yes,
                                     eso_cv_have__va_copy=no,
                                     eso_cv_have__va_copy=no)

                   ])

    if test "x$eso_cv_have_va_copy" = "xyes"; then
        eso_func_va_copy=va_copy
    else
        if test "x$eso_cv_have__va_copy" = "xyes"; then
            eso_func_va_copy=__va_copy
        fi
    fi

    AH_TEMPLATE([HAVE_VA_COPY],
                [Define if you have an implementation of a `va_copy()' style
                 function.])
    AH_TEMPLATE([$1], [A `va_copy()' style function])

    if test -n "$eso_func_va_copy"; then
        AC_DEFINE_UNQUOTED([$1], $eso_func_va_copy)
        AC_DEFINE(HAVE_VA_COPY)
    fi

    # Check whether va_lists can be copied by value
    AC_CACHE_CHECK([whether va_lists can be copied by value],
                   eso_cv_have_va_value_copy,
                   [
                       AC_RUN_IFELSE([
#include <stdarg.h>

void f(int i, ...)
{
    va_list args1, args2;
    va_start (args1, i);
    args2 = args1;

    if (va_arg (args2, int) != 42 || va_arg (args1, int) != 42)
        exit (1);

    va_end (args1);
    va_end (args2);
}

int main()
{
  f(0, 42);
  return 0;
}
                                     ],
                                     eso_cv_have_va_value_copy=yes,
                                     eso_cv_have_va_val_copy=no,
                                     eso_cv_have_va_val_copy=no)
                   ])

    AH_TEMPLATE([HAVE_VA_LIST_COPY_BY_VALUE],
                [Define if `va_lists' can be copied by value])
    if test "x$eso_cv_have_va_value_copy" = "xyes"; then
        AC_DEFINE(HAVE_VA_LIST_COPY_BY_VALUE)
    fi

])


# ESO_FUNC_REALLOC_SANITY
#-------------------------
# Check whether realloc(NULL,) works.
AC_DEFUN([ESO_FUNC_REALLOC_SANITY],
[
    AC_CACHE_CHECK([whether realloc(NULL,) works],
                   eso_cv_have_sane_realloc,
                   [
                       AC_RUN_IFELSE([
#include <stdlib.h>

int main()
{
    return realloc (0, sizeof (int)) == 0;
}
                                     ],
                                     eso_cv_have_sane_realloc=yes,
                                     eso_cv_have_sane_realloc=no,
                                     eso_cv_have_sane_realloc=no)
                   ])

    AH_TEMPLATE([HAVE_WORKING_REALLOC],
                [Define if realloc(NULL,) works])

    if test x$eso_cv_have_sane_realloc = xyes; then
        AC_DEFINE(HAVE_WORKING_REALLOC)
    fi

])


# ESO_FIND_FILE(file, directories, variable)
#------------------------------------------
# Search for file in directories. Set variable to the first location
# where file was found, if file is not found at all variable is set to NO.
AC_DEFUN([ESO_FIND_FILE],
[
    $3=no

    for i in $2; do
        for j in $1; do

            echo "configure: __oline__: $i/$j" >&AC_FD_CC

            if test -r "$i/$j"; then
                echo "taking that" >&AC_FD_CC
                $3=$i
                break 2
            fi
        done
    done
])


# ESO_SET_LIBRARY_VERSION([CURRENT], [REVISION], [AGE])
#------------------------------------------------------
# Sets the libtool versioning symbols LT_CURRENT, LT_REVISION, LT_AGE.
AC_DEFUN([ESO_SET_LIBRARY_VERSION],
[

    if test -z "$1"; then
        LT_CURRENT=0
    else
        LT_CURRENT="$1"
    fi

    if test -z "$2"; then
        LT_REVISION=0
    else
        LT_REVISION="$2"
    fi

    if test -z "$3"; then
        LT_AGE=0
    else
        LT_AGE="$3"
    fi

    AC_SUBST(LT_CURRENT)
    AC_SUBST(LT_REVISION)
    AC_SUBST(LT_AGE)
])
AC_DEFUN([AB_EXTRA_PATH],
[
   AC_ARG_WITH([extra-include-dir],
      AC_HELP_STRING([--with-extra-include-dir=DIR], [extra headers are in DIR]),
      ac_with_extra=$withval
      if test "x${ac_with_extra}" != xyes && test "x${ac_with_extra}" != xno; then
            CPPFLAGS="-I$withval $CPPFLAGS"
      fi
   )

   AC_ARG_WITH([extra-lib-dir],
      AC_HELP_STRING([--with-extra-lib-dir=DIR], [extra libraries are in DIR]),
      ac_with_extra=$withval
      if test "x${ac_with_extra}" != xyes && test "x${ac_with_extra}" != xno; then
            LDFLAGS="-L$withval $LDFLAGS"
      fi
   )
])

AC_DEFUN([AB_DEBUG],
[
AC_ARG_ENABLE(debug,
              AC_HELP_STRING([--disable-debug], 
                             [Build without -DDEBUG -O2 -Wall (default on)]),
              [ac_request_debug=$enableval], 
              [ac_request_debug=yes])

if test "x$ac_request_debug" = xyes; then
   AC_MSG_NOTICE(Build with -DDEBUG -Wall)
   CPPFLAGS="$CPPFLAGS -DDEBUG -Wall"
   CFLAGS="-g $CFLAGS"
   CXXLAGS="-g"
fi
])
#
# FFTW_CREATE_SYMBOLS(build=[])
#-----------------------------
# Sets the Makefile symbols for the FFTW and C extension libraries. If an
# argument is provided the symbols are setup for building FFTW, if no
# argument is given (default) the symbols are set for using the libraries
# for external package development.
AC_DEFUN([FFTW_CREATE_SYMBOLS],
[

    if test -z "$1"; then
        LIBFFTW='-lfftw3 -lm'
    else
        LIBFFTW='$(top_builddir)/fftw3/libfftw3.la -lm'
    fi

   AC_SUBST(LIBFFTW)
])


# FFTW_CHECK_LIBS
#---------------
# Checks for the FFTW libraries and header files.
AC_DEFUN([AC_SEARCH_FFTW],
[

    AC_MSG_CHECKING([for FFTW])

    fftw_check_fftw_header="fftw3.h"
    fftw_check_fftw_lib="libfftw3.a"
    fftw_includes=""
    ccunt_libraries=""
    AC_ARG_WITH(fftw,
                AC_HELP_STRING([--with-fftw],
                               [location where FFTW is installed]),
                [
                    fftw_with_fftw_includes="$withval/include $withval"
                    fftw_with_fftw_libs="$withval/lib64 $withval/lib32 $withval/lib $withval"
                ])

    AC_ARG_WITH(fftw-includes,
                AC_HELP_STRING([--with-fftw-includes],
                               [location of the FFTW header files]),
                fftw_with_fftw_includes=$withval)

    AC_ARG_WITH(fftw-libs,
                AC_HELP_STRING([--with-fftw-libs],
                               [location of the FFTW library]),
                fftw_with_fftw_libs=$withval)

    AC_ARG_ENABLE(fftw-test,
                  AC_HELP_STRING([--disable-fftw-test],
                                 [disables checks for the FFTW library and headers]),
                  fftw_enable_fftw_test=$enableval,
                  fftw_enable_fftw_test=yes)


    if test "x$fftw_enable_fftw_test" = xyes; then

        # Check for the FFTW includes

        if test -z "$fftw_with_fftw_includes"; then
            fftw_incdirs="/opt/fftw/include \
                         /usr/local/include \
                         /usr/local/include \
                         /usr/include \
                         $HOME/sph3rd/include \
                         $FFTW_SEARCH_LOCAL/include \
                         /usr/local/include \
                         $FFTW_SEARCH_LOCAL/include/fftw"
        else
            fftw_incdirs="$fftw_with_fftw_includes"
        fi
        test -n "$FFTWDIR" && fftw_incdirs="$FFTWDIR/include $FFTWDIR $fftw_incdirs"
        ESO_FIND_FILE($fftw_check_fftw_header, $fftw_incdirs, fftw_includes)


        # Check for the FFTW libraries

        if test -z "$fftw_with_fftw_libs"; then
            fftw_libdirs="/opt/fftw/lib \
                         /usr/local/lib \
                         /usr/local/fftw/lib \
                         /usr/lib \
                         /usr/lib64 \
                         $HOME/sph3rd/lib \
                         $HOME/sph3rd/lib64 \
                         $FFTW_SEARCH_LOCAL/lib"
        else
            fftw_libdirs="$fftw_with_fftw_libs \
                         /opt/fftw/lib \
                         /usr/local/lib \
                         /usr/local/fftw/lib \
                         /usr/lib \
                         /usr/lib64 \
                         $HOME/sph3rd/lib \
                         $HOME/sph3rd/lib64 \
                         $FFTW_SEARCH_LOCAL/lib"
        fi
	test -n "$FFTWDIR" && fftw_libdirs="$FFTWDIR/lib $FFTWDIR $fftw_libdirs"
        ESO_FIND_FILE($fftw_check_fftw_lib, $fftw_libdirs, fftw_libraries)


        if test x"$fftw_includes" = xno || test x"$fftw_libraries" = xno; then
            fftw_notfound=""

            if test x"$fftw_includes" = xno; then
                if test x"$fftw_libraries" = xno; then
                    fftw_notfound="(headers and libraries)"
                else            
                    fftw_notfound="(headers)"
                fi
            else
                fftw_notfound="(libraries)"
            fi

            AC_MSG_ERROR([FFTW $fftw_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $fftw_libraries, headers $fftw_includes])
        fi

        # Set up the symbols

        FFTW_INCLUDES="-I$fftw_includes"
        FFTW_LDFLAGS="-L$fftw_libraries"
        FFTW_CFLAGS="-I$fftw_includes"
	FFTW_CREATE_SYMBOLS
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([FFTW checks have been disabled! This package may not build!])
        FFTW_INCLUDES=""
        FFTW_LDFLAGS=""
        FFTW_CFLAGS=""
        LIBFFTW=""
    fi

    AC_SUBST(FFTW_INCLUDES)
    AC_SUBST(FFTW_LDFLAGS)
    AC_SUBST(FFTW_CFLAGS)
    AC_SUBST(LIBFFTW)

])

#
# GSL_CREATE_SYMBOLS(build=[])
#-----------------------------
# Sets the Makefile symbols for the GSL and C extension libraries. If an
# argument is provided the symbols are setup for building GSL, if no
# argument is given (default) the symbols are set for using the libraries
# for external package development.
AC_DEFUN([GSL_CREATE_SYMBOLS],
[

    if test -z "$1"; then
        LIBGSL='-lgsl -lgslcblas'
    else
        LIBGSL='$(top_builddir)/gsl/libgsl.la'
    fi

   AC_SUBST(LIBGSL)
])


# GSL_CHECK_LIBS
#---------------
# Checks for the GSL libraries and header files.
AC_DEFUN([AC_SEARCH_GSL],
[

    AC_MSG_CHECKING([for GSL])

    gsl_check_gsl_header="gsl/gsl_version.h"
    gsl_check_gsl_lib="libgsl.a"
    gsl_includes=""
    ccunt_libraries=""
    AC_ARG_WITH(gsl,
                AC_HELP_STRING([--with-gsl],
                               [location where GSL is installed]),
                [
                    gsl_with_gsl_includes="$withval/include $withval/include/gsl $withval"
                    gsl_with_gsl_libs="$withval/lib64 $withval/lib32 $withval/lib $withval"
                ])

    AC_ARG_WITH(gsl-includes,
                AC_HELP_STRING([--with-gsl-includes],
                               [location of the GSL header files]),
                gsl_with_gsl_includes=$withval)

    AC_ARG_WITH(gsl-libs,
                AC_HELP_STRING([--with-gsl-libs],
                               [location of the GSL library]),
                gsl_with_gsl_libs=$withval)

    AC_ARG_WITH(extra-includes,
                AC_HELP_STRING([--with-extra-includes],
                               [location of the extra header files]),
                gsl_with_gsl_includes=$withval)

    AC_ARG_WITH(extra-libs,
                AC_HELP_STRING([--with-extra-libs],
                               [location of the extra library]),
                gsl_with_gsl_libs=$withval)

    AC_ARG_ENABLE(gsl-test,
                  AC_HELP_STRING([--disable-gsl-test],
                                 [disables checks for the GSL library and headers]),
                  gsl_enable_gsl_test=$enableval,
                  gsl_enable_gsl_test=yes)


    if test "x$gsl_enable_gsl_test" = xyes; then

        # Check for the GSL includes

        if test -z "$gsl_with_gsl_includes"; then
            gsl_incdirs="/opt/gsl/include \
                         /usr/local/include \
                         /usr/local/gsl/include \
                         /usr/local/include \
                         /usr/include \
                         /usr/include \
                         $HOME/sph3rd/include \
                         $HOME/gslP/include \
                         $GSL_SEARCH_LOCAL/include \
                         /opt/gsl/include/gsl \
                         /usr/local/include/gsl \
                         /usr/local/gsl/include/gsl \
                         /usr/local/include/gsl \
                         /usr/include/gsl \
                         /usr/include/gsl \
                         $HOME/sph3rd/include/gsl \
                         $HOME/gslP/include/gsl \
                         $GSL_SEARCH_LOCAL/include/gsl"
        else
            gsl_incdirs="$gsl_with_gsl_includes"
        fi

        test -n "$GSLDIR" && gsl_incdirs="$GSLDIR/include $GSLDIR/gsl/ $gsl_incdirs"
        ESO_FIND_FILE($gsl_check_gsl_header, $gsl_incdirs, gsl_includes)


        # Check for the GSL libraries

        if test -z "$gsl_with_gsl_libs"; then
            gsl_libdirs="/opt/gsl/lib \
        		 /opt/gsl/lib64 \
                         /usr/local/lib \
                         /usr/local/lib64 \
                         /usr/local/gsl/lib \
                         /usr/local/gsl/lib64 \
                         /usr/lib \
                         /usr/lib64 \
                         $HOME/sph3rd/lib \
                         $HOME/sph3rd/lib64 \
                         $HOME/gslP/lib \
                         $HOME/gslP/lib64 \
                         $GSL_SEARCH_LOCAL/lib \
                         $GSL_SEARCH_LOCAL/lib64"
        else
            gsl_libdirs="$gsl_with_gsl_libs \
                         /opt/gsl/lib \
                         /opt/gsl/lib64 \
                         /usr/local/lib \
                         /usr/local/lib64 \
                         /usr/local/gsl/lib \
                         /usr/local/gsl/lib64 \
                         /usr/lib \
                         /usr/lib64 \
                         $HOME/sph3rd/lib \
                         $HOME/sph3rd/lib64 \
                         $HOME/gslP/lib \
                         $HOME/gslP/lib64 \
                         $GSL_SEARCH_LOCAL/lib \
                         $GSL_SEARCH_LOCAL/lib64"
        fi
	test -n "$GSLDIR" && gsl_libdirs="$GSLDIR/lib $gsl_libdirs"
        ESO_FIND_FILE($gsl_check_gsl_lib, $gsl_libdirs, gsl_libraries)


        if test x"$gsl_includes" = xno || test x"$gsl_libraries" = xno; then
            gsl_notfound=""

            if test x"$gsl_includes" = xno; then
                if test x"$gsl_libraries" = xno; then
                    gsl_notfound="(headers and libraries)"
                else            
                    gsl_notfound="(headers)"
                fi
            else
                gsl_notfound="(libraries)"
            fi

            AC_MSG_ERROR([GSL $gsl_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $gsl_libraries, headers $gsl_includes])
        fi

        # Set up the symbols

        GSL_INCLUDES="-I$gsl_includes -I$gsl_includes/gsl"
        GSL_LDFLAGS="-L$gsl_libraries"
        GSL_CFLAGS="-I$gsl_includes -I$gsl_includes/gsl"
	GSL_CREATE_SYMBOLS
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([GSL checks have been disabled! This package may not build!])
        GSL_INCLUDES=""
        GSL_LDFLAGS=""
        GSL_CFLAGS=""
        LIBGSL=""
    fi

    AC_SUBST(GSL_INCLUDES)
    AC_SUBST(GSL_LDFLAGS)
    AC_SUBST(GSL_CFLAGS)
    AC_SUBST(LIBGSL)

])

# HDRL_CHECK([location])
# ----------------------
# Sets build variables required to build and link hdrl
# the argument defines the relative location of the hdrl external in the source
# tree
AC_DEFUN([HDRL_CHECK],
[
    HDRL_LOCATION="$1"
    AC_MSG_CHECKING([HDRL in $HDRL_LOCATION])

    # visibility hidden to not expose any hdrl symbols in the pipeline shared
    # libraries
    HDRL_INTERNAL_CFLAGS="-std=c99 -fvisibility=hidden"
    # requires cpl and libm as it is a static library
    HDRL_LIBS="-lhdrl -lcplcore -lcpldrs -lcplui -lcpldfs -lcext -lm"
    # la file to be added to DEPENDENCIES libtool doesn't track static libs
    LIBHDRL="\$(top_builddir)/src/$HDRL_LOCATION/libhdrl.la"
    HDRL_LDFLAGS="-L\$(top_builddir)/src/$HDRL_LOCATION"
    HDRL_INCLUDES="-I\$(top_srcdir)/src/$HDRL_LOCATION"

    AC_SUBST(HDRL_LIBS)
    AC_SUBST(LIBHDRL)
    AC_SUBST(HDRL_LDFLAGS)
    AC_SUBST(HDRL_INTERNAL_CFLAGS)
    AC_SUBST(HDRL_INCLUDES)

    AC_MSG_RESULT([ok])
])
AC_DEFUN([AC_SEARCH_ICE],
[
  ac_with_Ice=no
dnl        if test -z "$ICEDIR"; then
dnl           AC_SUBST(ICEDIR, "/usr")
dnl        fi
dnl  ac_Ice_include=$ICEDIR/include
dnl        if test "${build_cpu}" = "x86_64"; then
dnl    ac_Ice_libdir=$ICEDIR/lib64
dnl        else
dnl          ac_Ice_libdir=$ICEDIR/lib
dnl        fi
dnl  ac_Ice_bindir=$ICEDIR/bin

  ac_Ice_libs="-lIce -lIceUtil"

  ac_search_path="$INSROOT $ICEDIR /usr /usr/local /opt /opt/local"
  ac_search_path_slice="$ICEDIR/slice /usr/slice /usr/share/slice /opt/local/share/ice/slice"

  AC_ARG_WITH(Ice,
    AC_HELP_STRING([--with-Ice=DIR], [root directory of Ice installation]),
    ac_with_Ice=$withval
    if test "x${ac_with_Ice}" != xyes && test "x${ac_with_Ice}" != xno; then
      ac_Ice_include="$withval/include"
      ac_Ice_libdir="$withval/lib"
      ac_Ice_bindir="$withval/bin"
    fi
  )

  AC_ARG_WITH([Ice-include-dir],
    AC_HELP_STRING([--with-Ice-include-dir=DIR], [Ice header files are in DIR]),
    ac_with_Ice=$withval
    if test "x${ac_with_Ice}" != xyes && test "x${ac_with_Ice}" != xno; then
      ac_Ice_include="$withval"
    fi
  )
  AC_ARG_WITH([Ice-bin-dir],
    AC_HELP_STRING([--with-Ice-bin-dir=DIR], [Ice utilities such as moc and uic are in DIR]),
    ac_with_Ice=$withval
    if test "x${ac_with_Ice}" != xyes && test "x${ac_with_Ice}" != xno; then
      ac_Ice_bindir="$withval"
    fi
  )
  AC_ARG_WITH([Ice-lib-dir],
    AC_HELP_STRING([--with-Ice-lib-dir=DIR], [The Ice library is in DIR]),
    ac_with_Ice=$withval
    if test "x${ac_with_Ice}" != xyes && test "x${ac_with_Ice}" != xno; then
      ac_Ice_libdir="$withval"
    fi
  )
  AC_ARG_WITH([Ice-slice-dir],
    AC_HELP_STRING([--with-Ice-slice-dir=DIR], [The slIce files are in DIR]),
    ac_with_Ice=$withval
    if test "x${ac_with_Ice}" != xyes && test "x${ac_with_Ice}" != xno; then
      ac_Ice_slice="$withval"
    fi
  )

  if test "x${ac_Ice_libdir}" == "x"; then
    for ac_ice_path_test in $ac_search_path ; do
dnl      AC_MSG_NOTICE(Check for lib path $ac_ice_path_test/lib)
      if test -r "$ac_ice_path_test/lib"; then
         if test "${build_cpu}" = "x86_64"; then
            ls $ac_ice_path_test/lib64/libIce*  >/dev/null 2>&1
            if test $? -eq 0; then
               ac_Ice_libdir="$ac_ice_path_test/lib64"
               break;
            fi
         else
            ls $ac_ice_path_test/lib/libIce*  >/dev/null 2>&1
            if test $? -eq 0; then
dnl                 AC_MSG_NOTICE(Found in $ac_ice_path_test/lib)
                 ac_Ice_libdir="$ac_ice_path_test/lib"
                 break;
            fi
         fi
      fi
    done
  fi

   if test "x${ac_Ice_include}" == "x"; then
    x=`echo $build_os |sed "s/darwin.*/darwin/"`
    for ac_ice_path_test in $ac_search_path ; do
dnl      AC_MSG_NOTICE(Check for include path $ac_ice_path_test/include/Ice/Ice.h)
      if test x$x = "xdarwin" ; then 
	 if test -r "$ac_ice_path_test/include/ice/Ice/Ice.h"; then
         	ac_Ice_include="$ac_ice_path_test/include/ice"
         	break;
      	 fi
      else
         if test -r "$ac_ice_path_test/include/Ice/Ice.h"; then
         	ac_Ice_include="$ac_ice_path_test/include"
         	break;
      	 fi
      fi
    done
   fi

   if test "x${ac_Ice_bindir}" == "x"; then
    for ac_ice_path_test in $ac_search_path ; do
dnl      AC_MSG_NOTICE(Check for bin path $ac_ice_path_test)
      if test -r "$ac_ice_path_test/bin/slice2cpp"; then
         ac_Ice_bindir="$ac_ice_path_test/bin"
         break;
      fi
    done
   fi

   if test "x${ac_Ice_slicedir}" == "x"; then
    for ac_ice_path_test in $ac_search_path_slice; do
dnl      AC_MSG_NOTICE(Check for lib path $ac_ice_path_test)
      if test -d "$ac_ice_path_test"; then
         ac_Ice_slice="$ac_ice_path_test"
         break;
      fi
    done
   fi

  AC_SEARCH_ICE_LOCAL($ac_Ice_libdir, $ac_Ice_include, $ac_Ice_bindir, $ac_Ice_libs, $ac_Ice_slice)
])


# subfunction for easier reuse
AC_DEFUN([AC_SEARCH_ICE_LOCAL],
[
  OLD_LDFLAGS=$LDFLAGS
  OLD_CXXFLAGS=$CXXFLAGS
  OLD_PATH=$PATH

  if test "$1"; then
    LDFLAGS="$LDFLAGS -L$1 $4"
  fi
  if test "$2"; then
    CXXFLAGS="$CXXFLAGS -I$2"
  fi
  if test "$3"; then
    PATH="$3:$PATH"
  fi

  AC_LANG_PUSH(C++)
  AC_MSG_CHECKING(for Ice)
  AC_TRY_LINK([
    #include <Ice/Ice.h>
  ],[
    ;
  ],[
  success=yes
  ],[
  success=no
  ])
  AC_MSG_RESULT($success)
  AC_LANG_POP()

  LDFLAGS=$OLD_LDFLAGS
  CXXFLAGS=$OLD_CXXFLAGS

  if test "x$success" != xyes; then
    AC_MSG_ERROR(Ice not (properly) installed. Get a (free) recent version from: http://www.zeroc.com)
  else
    if test "$1"; then
      AC_SUBST(ICE_LDFLAGS, "-L$1")
    fi
    if test "$2"; then
      AC_SUBST(ICE_CXXFLAGS, "-ftemplate-depth-128 -fPIC -D_REENTRANT -I$2")
    fi
    if test "$4"; then
      AC_SUBST(ICE_LIBS, "$4")
    fi
  fi


# slice2cpp test
  # AC_CHECK_PROG(ac_icebindir, slice2cpp, yes)
  AC_PATH_PROG(ac_icebindir, slice2cpp, no)
  if test "x$ac_icebindir" != xno; then
    AC_SUBST(ICE_SLICE2CPP, "$ac_icebindir")
  else
    AC_MSG_ERROR(Ice not (properly) installed. Get a (free) recent version from: http://www.zeroc.com)
  fi


# slice2py test
  AC_PATH_PROG(ac_icepybindir, slice2py, no)
  if test "x$ac_icebindir" != xno; then
    AC_SUBST(ICE_SLICE2PY, "$ac_icepybindir")
  else
    AC_MSG_ERROR(Ice not (properly) installed. Get a (free) recent version from: http://www.zeroc.com)
  fi

  AC_SUBST(ICE_SLICEFILES, "-I$ac_Ice_slice")
])

AC_DEFUN([AC_SEARCH_LATEX_MK],
[
  success=no
  AC_CHECK_PROGS([ac_latex_mk],latex-mk, no)
  if test "x$ac_latex_mk" != xno; then
        if test -n "$LATEX_MK_DIR"; then
            AC_SUBST(LATEX_MK, "$ac_latex_mk")
            success=yes
        else 
            AC_WARN(latex-mk environment variable not set.)
        fi
  else
        AC_WARN(latex_mk not (properly) installed.)
  fi
  if test "x$success" != xyes; then
     AM_CONDITIONAL(HAVE_LATEX_MK, false)
  else
     AM_CONDITIONAL(HAVE_LATEX_MK, true)
  fi
])


AC_DEFUN([AC_SEARCH_NICE],
[

  AX_BOOST_BASE([1.35])
  AX_BOOST_PROGRAM_OPTIONS
  AC_SEARCH_ICE

  #Adding ICE libs and include paths
  if test "$ICE_LDFLAGS"; then
     ac_nice_libdir="$ICE_LDFLAGS" 
  fi
  if test "$ICE_CXXFLAGS"; then
     ac_nice_incdir="$ICE_CXXFLAGS"
  fi
  if test "$ICE_LIBS"; then
     ac_nice_libs="$ICE_LIBS"
  fi
  if test "$ICE_SLICEFILES"; then
     ac_nice_incdir="$ac_nice_incdir $ICE_SLICEFILES" 
  fi

  #Adding BOOST libs and include paths
  if test "$BOOST_LDFLAGS"; then
     ac_nice_libdir="$ac_nice_libdir $BOOST_LDFLAGS"
  fi
  if test "$BOOST_CPPFLAGS"; then
     ac_nice_incdir="$ac_nice_incdir $BOOST_CPPFLAGS"
  fi

  AC_SUBST(NICE_LDFLAGS, "$ac_nice_libdir")
  AC_SUBST(NICE_CXXFLAGS, "$ac_nice_incdir")
  AC_SUBST(NICE_LIBS, "$ac_nice_libs -lNice")
])



AC_DEFUN([NICE_CHECK_COMPILER_FLAG],
[
#AC_REQUIRE([AC_CHECK_COMPILERS])
AC_MSG_CHECKING(whether $CXX supports -$1)
cache=`echo $1 | sed 'y%.=/+-%___p_%'`
AC_CACHE_VAL(cv_prog_cxx_$cache,
[
echo 'void f(){}' >conftest.cc
if test -z "`$CXX -$1 -c conftest.cc 2>&1`"; then
  eval "cv_prog_cxx_$cache=yes"
else
  eval "cv_prog_cxx_$cache=no"
fi
rm -f conftest*
])
if eval "test \"`echo '$cv_prog_cxx_'$cache`\" = yes"; then
 AC_MSG_RESULT(yes)
 :
 $2
else
 AC_MSG_RESULT(no)
 :
 $3
fi
])


AC_DEFUN([NICE_CHECKLIB_PTHREAD],
[
AC_CHECK_LIB(pthread, pthread_create, [LIBPTHREAD="-lpthread"] )
AC_SUBST(LIBPTHREAD)
])

AC_DEFUN([NICE_CHECK_PTHREAD_OPTION],
[
    AC_ARG_ENABLE(kernel-threads, [  --enable-kernel-threads Enable the use of the LinuxThreads port on FreeBSD/i386 only.],
   use_kernthreads=$enableval, use_kernthreads=no)

    if test "$use_kernthreads" = "yes"; then
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_CFLAGS="$CFLAGS"
      CXXFLAGS="-I/usr/local/include/pthread/linuxthreads $CXXFLAGS"
      CFLAGS="-I/usr/local/include/pthread/linuxthreads $CFLAGS"
      AC_CHECK_HEADERS(pthread/linuxthreads/pthread.h)
      CXXFLAGS="$ac_save_CXXFLAGS"
      CFLAGS="$ac_save_CFLAGS"
      if test "$ac_cv_header_pthread_linuxthreads_pthread_h" = "no"; then
        use_kernthreads=no
      else
        dnl Add proper -I and -l statements
        AC_CHECK_LIB(lthread, pthread_join, [LIBPTHREAD="-llthread -llgcc_r"]) dnl for FreeBSD
        if test "x$LIBPTHREAD" = "x"; then
          use_kernthreads=no
        else
          USE_THREADS="-D_THREAD_SAFE -I/usr/local/include/pthread/linuxthreads"
        fi
      fi
    else
      USE_THREADS=""
      if test -z "$LIBPTHREAD"; then
        NICE_CHECK_COMPILER_FLAG(pthread, [USE_THREADS="-pthread"] )
      fi
    fi

    AH_VERBATIM(__svr_define, [
#if defined(__SVR4) && !defined(__svr4__)
#define __svr4__ 1
#endif
])
    case $host_os in
        solaris*)
                CPPFLAGS="$CPPFLAGS -D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS -DUSE_SOLARIS -DSVR4"
                ;;
        freebsd*)
                CPPFLAGS="$CPPFLAGS -D_THREAD_SAFE"
                ;;
        aix*)
                CPPFLAGS="$CPPFLAGS -D_THREAD_SAFE"
                LIBPTHREAD="$LIBPTHREAD -lc_r"
                ;;
        linux*) CPPFLAGS="$CPPFLAGS -D_REENTRANT"
                if test "$CXX" = "KCC"; then
                    CXXFLAGS="$CXXFLAGS --thread_safe"
                    NOOPT_CXXFLAGS="$NOOPT_CXXFLAGS --thread_safe"
                fi
                ;;
   *)
      ;;
    esac
    AC_SUBST(USE_THREADS)
    AC_SUBST(LIBPTHREAD)
])

AC_DEFUN([NICE_CHECK_THREADING],
[
  AC_REQUIRE([NICE_CHECKLIB_PTHREAD])
  AC_REQUIRE([NICE_CHECK_PTHREAD_OPTION])
  dnl default is yes if libpthread is found and no if no libpthread is available
  if test -z "$LIBPTHREAD"; then
    if test -z "$USE_THREADS"; then
      check_threading_default=no
    else
      check_threading_default=yes
    fi
  else
    check_threading_default=yes
  fi
  AC_ARG_ENABLE(threading, [  --disable-threading     disables threading even if libpthread found ],
   use_threading=$enableval, use_threading=$check_threading_default)
  if test "x$use_threading" = "xyes"; then
    AC_DEFINE(HAVE_LIBPTHREAD, 1, [Define if you have a working libpthread (will enable threaded code)])
  fi
])
AC_DEFUN([AC_SEARCH_QT4],
[
  ac_with_Qt4=no
  ac_search_path="$INSROOT $QT4DIR /usr /usr/local /opt /opt/local"
  ac_search_include_path="$INSROOT/include $QT4DIR/include /usr/include /usr/local/include/qt4  /opt/local/include/qt4 /usr/include/qt4"

  ac_Qt4_modules="$1"
 
  AC_ARG_WITH(Qt4,
    AC_HELP_STRING([--with-Qt4=DIR], [root directory of Qt4 installation]),
    ac_with_Qt4=$withval
    if test "x${ac_with_Qt4}" != xyes && test "x${ac_with_Qt4}" != xno; then
      ac_Qt4_include="$withval/include"
      ac_Qt4_libdir="$withval/lib"
      ac_Qt4_bindir="$withval/bin"
    fi
  )

  AC_ARG_WITH([Qt4-include-dir],
    AC_HELP_STRING([--with-Qt4-include-dir=DIR], [Qt header files are in DIR]),
    ac_with_Qt4=$withval
    if test "x${ac_with_Qt4}" != xyes && test "x${ac_with_Qt4}" != xno; then
      ac_Qt4_include="$withval"
    fi
  )
  AC_ARG_WITH([Qt4-bin-dir],
    AC_HELP_STRING([--with-Qt4-bin-dir=DIR], [Qt utilities such as moc and uic are in DIR]),
    ac_with_Qt4=$withval
    if test "x${ac_with_Qt4}" != xyes && test "x${ac_with_Qt4}" != xno; then
      ac_Qt4_bindir="$withval"
    fi
  )
  AC_ARG_WITH([Qt4-lib-dir],
    AC_HELP_STRING([--with-Qt4-lib-dir=DIR], [The Qt library is in DIR]),
    ac_with_Qt4=$withval
    if test "x${ac_with_Qt4}" != xyes && test "x${ac_with_Qt4}" != xno; then
      ac_Qt4_libdir="$withval"
    fi
  )

  AC_ARG_WITH([Qt4-plugin-dir],
    AC_HELP_STRING([--with-Qt4-plugin-dir=DIR], [The Qt designer plugin is in DIR]),
    ac_with_Qt4=$withval
    if test "x${ac_with_Qt4}" != xyes && test "x${ac_with_Qt4}" != xno; then
      ac_Qt4_plugdir="$withval"
    fi
  )

  if test "x${ac_Qt4_libdir}" == "x"; then
    for ac_qt4_path_test in $ac_search_path ; do
dnl      AC_MSG_NOTICE(Check for lib path $ac_qt4_path_test/lib)
      if test -r "$ac_qt4_path_test/lib"; then
         ls $ac_qt4_path_test/lib/libQtCore*  >/dev/null 2>&1
         if test $? -eq 0; then
            if test "${build_cpu}" = "x86_64"; then
               ac_Qt4_libdir="$ac_qt4_path_test/lib64"
            else
               ac_Qt4_libdir="$ac_qt4_path_test/lib"
            fi
            break;
         fi
      fi
    done
    ls $ac_Qt4_libdir/libQt*.la >/dev/null 2>&1
    if test $? -eq 0 && test x`echo $build_os |sed "s/darwin.*/darwin/"` = "xdarwin"; then
	AC_WARN(Please remove all *.la files from directory $ac_Qt4_libdir. Otherwise QT application will not work!)
    fi
  fi

   if test "x${ac_Qt4_include}" == "x"; then
    for ac_qt4_path_test in $ac_search_include_path ; do
dnl      AC_MSG_NOTICE(Check for include path $ac_qt4_path_test/qapplication.h)
      if test -r "$ac_qt4_path_test/Qt/qapplication.h"; then
         ac_Qt4_include="$ac_qt4_path_test"
         break;
      fi
    done
   fi

   if test "x${ac_Qt4_bindir}" == "x"; then
    for ac_qt4_path_test in $ac_search_path ; do
dnl      AC_MSG_NOTICE(Check for bin path $ac_qt4_path_test/bin)
      if test -r "$ac_qt4_path_test/bin/moc"; then
         ac_Qt4_bindir="$ac_qt4_path_test/bin"
         break;
      fi
    done
   fi


  AC_SEARCH_QT4_LOCAL($ac_Qt4_libdir, $ac_Qt4_include, $ac_Qt4_bindir, $ac_Qt4_modules)
])



# subfunction for easier reuse
AC_DEFUN([AC_SEARCH_QT4_LOCAL],
[
  OLD_LDFLAGS=$LDFLAGS
  OLD_CXXFLAGS=$CXXFLAGS
  OLD_PATH=$PATH

  if test "$1"; then
    LDFLAGS="$LDFLAGS -L$1 -lQtGui -lQtCore"
  fi
  if test "$2"; then
    CXXFLAGS="$CXXFLAGS -I$2 -I$2/QtCore -I$2/QtGui"
  fi
  if test "$3"; then
    PATH="$3:$PATH"
  fi

  AC_LANG_PUSH(C++)
  AC_MSG_CHECKING(for Qt4)
  AC_TRY_LINK([
    #include <qapplication.h>
  ],[
    ;
  ],[
  success=yes
  ],[
  success=no
  ])
  AC_MSG_RESULT($success)
  AC_LANG_POP()


  LDFLAGS=$OLD_LDFLAGS
  CXXFLAGS=$OLD_CXXFLAGS
  AM_CONDITIONAL(HAVE_QT4, true)

  if test "x$success" != xyes; then
    AC_WARN(Qt4 not (properly) installed. Get a (free) recent version from: http://www.trolltech.com)
    AM_CONDITIONAL(HAVE_QT4, false)
  else
    if test "$1"; then
      AC_SUBST(QT4_LDFLAGS, "-L$1")
      AC_SUBST(QT4_LIBS, "-lQtGui -lQtCore")
      AC_SUBST(QT4_LIBS_ALL, "$QT4_LIBS")
    fi
    if test "$2"; then
       AC_SUBST(QT4_CXXFLAGS, "-DQT_CORE_LIB -DQT_GUI_LIB -DQT_SHARED -I$2 -I$2/Qt  -I$2/QtCore -I$2/QtGui")
    fi
  fi



# moc test
  AC_PATH_PROG(ac_Qt4_moc, moc, no)
  if test "x$ac_Qt4_moc" != xno; then
    AC_SUBST(QT4_MOC, "$ac_Qt4_moc")
  else
    AC_WARN(Qt4 moc not found!)
    AM_CONDITIONAL(HAVE_QT4, false)
  fi

# uic test
  AC_PATH_PROG(ac_Qt4_uic, uic, no)
  if test "x$ac_Qt4_uic" != xno; then
    AC_SUBST(QT4_UIC, "$ac_Qt4_uic")
  else
    AC_WARN(Qt4 uic not found!)
    AM_CONDITIONAL(HAVE_QT4, false)
  fi

# rcc test
  AC_PATH_PROG(ac_Qt4_rcc, rcc, no)
  if test "x$ac_Qt4_rcc" != xno; then
    AC_SUBST(QT4_RCC, "$ac_Qt4_rcc")
  else
    AC_WARN(Qt4 rcc not found!)
    AM_CONDITIONAL(HAVE_QT4, false)
  fi

  
PATH=$OLD_PATH
])


# qt4 designer search
AC_DEFUN([AC_SEARCH_QT4_DESIGNER],
[
  AC_ARG_ENABLE(
        QT4-Designer-Plugin,
        AC_HELP_STRING([--enable-QT4-Designer-Plugin], [Build the QT4 designer plugin (default off)]),
        ac_request_plugin=$enableval,
        ac_request_plugin=no)

  ac_search_path_plugin="$QT4DIR/plugin /usr/lib/qt4/plugin /opt/local/lib/qt4-plugins"

   if test "x${ac_request_plugin}" = "xyes"; then
        if test "x${ac_Qt4_plugdir}" == "x"; then
                for ac_qt4_path_test in $ac_search_path_plugin ; do
dnl      AC_MSG_NOTICE(Check for plugin path $ac_qt4_path_test)
                        if test -d "$ac_qt4_path_test"; then
                                ac_Qt4_plugdir="$ac_qt4_path_test/designer"
                                break;
                        fi
                done
        fi
	   AC_SEARCH_QT4_EXTRA(QtDesigner)
   else
	   AM_CONDITIONAL(HAVE_QT4_QtDesigner, false)
   fi
])


AC_DEFUN([AC_SEARCH_QT4_EXTRA],
[
  OLD_LDFLAGS=$LDFLAGS
  OLD_CXXFLAGS=$CXXFLAGS
  OLD_PATH=$PATH

dnl  AC_MSG_NOTICE(Check for module $ac_Qt4_want_module)

  if test "$1"; then
    LDFLAGS="$LDFLAGS $QT4_LDFLAGS $QT4_LIBS -l$1"
    CXXFLAGS="$CXXFLAGS $QT4_CXXFLAGS"
  fi

  AC_LANG_PUSH(C++)
  AC_MSG_CHECKING(for Qt4 $1 )
  AC_TRY_LINK([
    #include <QtGui/qapplication.h>
  ],[
    ;
  ],[
  success=yes
  ],[
  success=no
  ])
  AC_MSG_RESULT($success)
  AC_LANG_POP()

  LDFLAGS=$OLD_LDFLAGS
  CXXFLAGS=$OLD_CXXFLAGS

  if test "x$success" != xyes; then
      AC_WARN(Qt4 $1 not (properly) installed.)
      AM_CONDITIONAL(HAVE_QT4_$1, false)
  else
      AC_SUBST(QT4_LIBS_$1, "-l$1")
      AC_SUBST(QT4_CXXFLAGS_$1, "-I$1")
      AC_SUBST(QT4_LIBS_ALL, "-l$1 $QT4_LIBS_ALL")
      AC_SUBST(QT4_CXXFLAGS_ALL, "-I$1 $QT4_CXXFLAGS_ALL")
      AM_CONDITIONAL(HAVE_QT4_$1, true)
  fi
])
AC_DEFUN([AC_SEARCH_QT],
[
     ac_with_Qt=no
     ac_Qt_search="$INSROOT $QTDIR /usr /usr/local /opt /opt/local"
     ac_Qt_search_plugin="$INSROOT $QTDIR/plugin /usr/lib/qt3/plugin"
     ac_Qt_lib=qt-mt

dnl        if test -z "$QTDIR"; then
dnl           AC_SUBST(QTDIR, "/usr/lib/qt3")
dnl        fi
dnl     ac_Qt_include=$QTDIR/include
dnl        if test "${build_cpu}" = "x86_64"; then
dnl          ac_Qt_libdir=$QTDIR/lib64;
dnl        else
dnl          ac_Qt_libdir=$QTDIR/lib;
dnl        fi
dnl     ac_Qt_bindir=$QTDIR/bin
dnl     ac_Qt_lib=qt-mt


     AC_ARG_WITH(Qt,
          AC_HELP_STRING([--with-Qt=DIR], [root directory of Qt installation]),
          ac_with_Qt=$withval
          if test "x${ac_with_Qt}" != xyes && test "x${ac_with_Qt}" != xno; then
               ac_Qt_include="$withval/include"
               ac_Qt_libdir="$withval/lib"
               ac_Qt_bindir="$withval/bin"
          fi
     )

     AC_ARG_WITH([Qt-include-dir],
          AC_HELP_STRING([--with-Qt-include-dir=DIR], [Qt header files are in DIR]),
          ac_with_Qt=$withval
          if test "x${ac_with_Qt}" != xyes && test "x${ac_with_Qt}" != xno; then
               ac_Qt_include="$withval"
          fi
     )
     AC_ARG_WITH([Qt-bin-dir],
          AC_HELP_STRING([--with-Qt-bin-dir=DIR], [Qt utilities such as moc and uic are in DIR]),
          ac_with_Qt=$withval
          if test "x${ac_with_Qt}" != xyes && test "x${ac_with_Qt}" != xno; then
               ac_Qt_bindir="$withval"
          fi
     )
     AC_ARG_WITH([Qt-lib-dir],
          AC_HELP_STRING([--with-Qt-lib-dir=DIR], [The Qt library is in DIR]),
          ac_with_Qt=$withval
          if test "x${ac_with_Qt}" != xyes && test "x${ac_with_Qt}" != xno; then
               ac_Qt_libdir="$withval"
          fi
     )

  if test "x${ac_Qt_libdir}" == "x"; then
    for ac_qt_path_test in $ac_Qt_search ; do
dnl      AC_MSG_NOTICE(Check for lib path $ac_qt_path_test/lib)
      if test -r "$ac_qt_path_test/lib"; then
         ls $ac_qt_path_test/lib/libqt-mt*  >/dev/null 2>&1
         if test $? -eq 0; then
            if test "${build_cpu}" = "x86_64"; then
               ac_Qt_libdir="$ac_qt_path_test/lib64"
            else
               ac_Qt_libdir="$ac_qt_path_test/lib"
            fi
            break;
         fi
      fi
    done
  fi

   if test "x${ac_Qt_include}" == "x"; then
    for ac_qt_path_test in $ac_Qt_search ; do
dnl      AC_MSG_NOTICE(Check for include $ac_qt_path_test/include/qapplication.h)
      if test -r "$ac_qt_path_test/include/qapplication.h"; then
         ac_Qt_include="$ac_qt_path_test/include"
         break;
      fi
    done
   fi

   if test "x${ac_Qt_bindir}" == "x"; then
    for ac_qt_path_test in $ac_Qt_search ; do
dnl      AC_MSG_NOTICE(Check for bin path $ac_qt_path_test/bin)
      if test -r "$ac_qt_path_test/bin/moc"; then
         ac_Qt_bindir="$ac_qt_path_test/bin"
         break;
      fi
    done
   fi

   if test "x${ac_Qt_plugdir}" == "x"; then
    for ac_qt_path_test in $ac_Qt_search_plugin ; do
dnl      AC_MSG_NOTICE(Check for plugin path $ac_qt_path_test)
      if test -d "$ac_qt_path_test"; then
         ac_Qt_plugdir="$ac_qt_path_test"
         break;
      fi
    done
   fi

    AC_SEARCH_QT_LOCAL($ac_Qt_libdir, $ac_Qt_include, $ac_Qt_bindir, $ac_Qt_plugdir, $ac_Qt_lib)
])


# subfunction for easier reuse
AC_DEFUN([AC_SEARCH_QT_LOCAL],
[
     OLD_LDFLAGS=$LDFLAGS
     OLD_CXXFLAGS=$CXXFLAGS
     OLD_PATH=$PATH

     if test "$1"; then
          LDFLAGS="$LDFLAGS -L$1 -l$5"
     fi
     if test "$2"; then
          CXXFLAGS="$CXXFLAGS -I$2"
     fi
     if test "$3"; then
          PATH="$3:$PATH"
     fi

     AC_LANG_PUSH(C++)
     AC_MSG_CHECKING(for Qt)
     AC_TRY_LINK([
          #include <qapplication.h>
     ],[
          ;
     ],[
     success=yes
     ],[
     success=no
     ])
     AC_MSG_RESULT($success)
     AC_LANG_POP()


     LDFLAGS=$OLD_LDFLAGS
     CXXFLAGS=$OLD_CXXFLAGS
     AM_CONDITIONAL(HAVE_QT3, true)

     if test "x$success" != xyes; then
          AC_MSG_ERROR(Qt not (properly) installed. Get a (free) recent version from: http://www.trolltech.com)
     else
          if test "$1"; then
               AC_SUBST(QT_LDFLAGS, "-L$1")
          fi
          if test "$2"; then
               AC_SUBST(QT_CXXFLAGS, "-DQT_THREAD_SUPPORT -I$2")
          fi
          if test "$5"; then
               AC_SUBST(QT_LIBS, "-l$5")
          fi
     fi


# moc test
     AC_PATH_PROG(ac_Qt_moc, moc, yes)
     if test "x$ac_Qt_moc" != xno; then
          AC_SUBST(QT_MOC, "$ac_Qt_moc")
     else
          AC_WARN(Qt3 moc not found!)
          AM_CONDITIONAL(HAVE_QT3, false)
     fi

# uic test
     AC_PATH_PROG(ac_Qt_uic, uic, no)
     if test "x$ac_Qt_uic" != xno; then
          AC_SUBST(QT_UIC, "$ac_Qt_uic")
     else
          AC_WARN(Qt3 uic not found!)
          AM_CONDITIONAL(HAVE_QT3, false)
     fi

        PATH=$OLD_PATH
])
AC_DEFUN([AC_SEARCH_QWT4],
[
     ac_with_Qwt4=no
     ac_search_path="$INSROOT $QT4DIR /usr /usr/local /opt /opt/local"
     ac_search_libs="Qwt qwt qwt-qt4"
     ac_search_include="$INSROOT $QT4DIR /usr/include /usr/local/include /opt/include /opt/local/include  /usr/include/qwt-qt4"

     AC_ARG_WITH(Qwt4,
          AC_HELP_STRING([--with-Qwt4=DIR], [root directory of Qwt4 installation]),
          ac_with_Qwt4=$withval
          if test "x${ac_with_Qwt4}" != xyes && test "x${ac_with_Qwt4}" != xno; then
               ac_Qwt4_include="$withval/include"
               ac_Qwt4_libdir="$withval/lib"
          fi
     )

     AC_ARG_WITH([Qwt4-include-dir],
          AC_HELP_STRING([--with-Qwt4-include-dir=DIR], [Qwt4 header files are in DIR]),
          ac_with_Qwt4=$withval
          if test "x${ac_with_Qwt4}" != xyes && test "x${ac_with_Qwt4}" != xno; then
               ac_Qwt4_include="$withval"
          fi
     )

     AC_ARG_WITH([Qwt4-lib-dir],
          AC_HELP_STRING([--with-Qwt4-lib-dir=DIR], [The Qwt4 library is in DIR]),
          ac_with_Qwt4=$withval
          if test "x${ac_with_Qwt4}" != xyes && test "x${ac_with_Qwt4}" != xno; then
               ac_Qwt4_libdir="$withval"
          fi
     )

  if test "x${ac_Qwt4_libdir}" == "x"; then
    for ac_qt_path_test in $ac_search_path ; do
      if test -r "$ac_qt_path_test/lib"; then
	 for ac_qt_lib_test in $ac_search_libs ; do
         	ls $ac_qt_path_test/lib/lib$ac_qt_lib_test.*  >/dev/null 2>&1
       		if test $? -eq 0; then
            		if test "${build_cpu}" = "x86_64"; then
               			ac_Qwt4_libdir="$ac_qt_path_test/lib64"
            		else
               			ac_Qwt4_libdir="$ac_qt_path_test/lib"
            		fi
			ac_Qwt4_libs=$ac_qt_lib_test
            		break;
         	fi			
	 done
         if test "$ac_Qwt4_libs"; then
            break;
         fi
       fi
    done
  fi

   if test "x${ac_Qwt4_include}" == "x"; then
    for ac_qt_path_test in $ac_search_include ; do
dnl      AC_MSG_NOTICE(Check for lib path $ac_qt_path_test/include/qwt.h)
      if test -r "$ac_qt_path_test/qwt.h"; then
         ac_Qwt4_include="$ac_qt_path_test"
         break;
      fi
    done
   fi

    AC_SEARCH_QWT4_LOCAL($ac_Qwt4_libdir, $ac_Qwt4_include, $ac_Qwt4_libs)
])


# subfunction for easier reuse
AC_DEFUN([AC_SEARCH_QWT4_LOCAL],
[
     OLD_LDFLAGS=$LDFLAGS
     OLD_CXXFLAGS=$CXXFLAGS

     if test "x$QT4_LIBS" == "x"; then
         AC_SEARCH_QT4
     fi

     if test "$1" && test "$3"; then
          LDFLAGS="$LDFLAGS -L$1 -L$QT4_LDFLAGS -l$3 $QT4_LIBS"
     fi
     if test "$2"; then
          CXXFLAGS="$CXXFLAGS -I$2 $QT4_CXXFLAGS"
     fi
     if test "$3"; then
          LDFLAGS="$LDFLAGS -l$3"
     fi

     AC_LANG_PUSH(C++)
     AC_MSG_CHECKING(for Qwt for Qt4 in $1)
     AC_TRY_LINK([
          #include <qwt.h>
     ],[
          ;
     ],[
     success=yes
     ],[
     success=no
     ])
     AC_MSG_RESULT($success)
     AC_LANG_POP()


     LDFLAGS=$OLD_LDFLAGS
     CXXFLAGS=$OLD_CXXFLAGS
     AM_CONDITIONAL(HAVE_QWT4, true)

     if test "x$success" != xyes; then
          AC_MSG_ERROR(Qwt not (properly) installed. Get a (free) recent version from: http://qwt.sourceforge.net/)
     else
          if test "$1"; then
               AC_SUBST(QWT4_LDFLAGS, "-L$1")
          fi
          if test "$2"; then
               AC_SUBST(QWT4_CXXFLAGS, "-I$2")
          fi
	  if test "$3"; then
	       AC_SUBST(QWT4_LIBS, "-l$3")
	  fi
     fi

])
#DAs ist ein Test
AC_DEFUN([AC_SEARCH_QWT],
[
     ac_with_Qwt=no
     ac_search_path="$QTDIR /usr/local /usr/local/qwt /opt /opt/local"

     AC_ARG_WITH(Qwt,
          AC_HELP_STRING([--with-Qwt=DIR], [root directory of Qwt installation]),
          ac_with_Qwt=$withval
          if test "x${ac_with_Qwt}" != xyes && test "x${ac_with_Qwt}" != xno; then
               ac_Qwt_include="$withval/include"
               ac_Qwt_libdir="$withval/lib"
          fi
     )

     AC_ARG_WITH([Qwt-include-dir],
          AC_HELP_STRING([--with-Qwt-include-dir=DIR], [Qwt header files are in DIR]),
          ac_with_Qwt=$withval
          if test "x${ac_with_Qwt}" != xyes && test "x${ac_with_Qwt}" != xno; then
               ac_Qwt_include="$withval"
          fi
     )

     AC_ARG_WITH([Qwt-lib-dir],
          AC_HELP_STRING([--with-Qwt-lib-dir=DIR], [The Qwt library is in DIR]),
          ac_with_Qwt=$withval
          if test "x${ac_with_Qwt}" != xyes && test "x${ac_with_Qwt}" != xno; then
               ac_Qwt_libdir="$withval"
          fi
     )

  if test "x${ac_Qwt_libdir}" == "x"; then
    for ac_qt_path_test in $ac_search_path ; do
dnl      AC_MSG_NOTICE(Check for lib path $ac_qt_path_test/lib)
      if test -r "$ac_qt_path_test/lib"; then
         ls $ac_qt_path_test/lib/libqwt*  >/dev/null 2>&1
         if test $? -eq 0; then
            if test "${build_cpu}" = "x86_64"; then
               ac_Qwt_libdir="$ac_qt_path_test/lib64"
            else
               ac_Qwt_libdir="$ac_qt_path_test/lib"
            fi
            break;
         fi
      fi
    done
  fi

   if test "x${ac_Qwt_include}" == "x"; then
    for ac_qt_path_test in $ac_search_path ; do
dnl      AC_MSG_NOTICE(Check for lib path $ac_qt_path_test/include/qwt.h)
      if test -r "$ac_qt_path_test/include/qwt.h"; then
         ac_Qwt_include="$ac_qt_path_test/include"
         break;
      fi
    done
   fi

    AC_SEARCH_QWT_LOCAL($ac_Qwt_libdir, $ac_Qwt_include)
])


# subfunction for easier reuse
AC_DEFUN([AC_SEARCH_QWT_LOCAL],
[
     OLD_LDFLAGS=$LDFLAGS
     OLD_CXXFLAGS=$CXXFLAGS

     if test "x$QT_LIBS" == "x"; then
         AC_SEARCH_QT
     fi

     if test "$1"; then
          LDFLAGS="$LDFLAGS -L$1 -L$QT_LDFLAGS -lqwt $QT_LIBS"
     fi
     if test "$2"; then
          CXXFLAGS="$CXXFLAGS -I$2 $QT_CXXFLAGS"
     fi

     AC_LANG_PUSH(C++)
     AC_MSG_CHECKING(for Qwt for Qt3 in $1)
     AC_TRY_LINK([
          #include <qwt.h>
     ],[
          ;
     ],[
     success=yes
     ],[
     success=no
     ])
     AC_MSG_RESULT($success)
     AC_LANG_POP()

     LDFLAGS=$OLD_LDFLAGS
     CXXFLAGS=$OLD_CXXFLAGS
     AM_CONDITIONAL(HAVE_QWT3, true)

     if test "x$success" != xyes; then
          AC_MSG_ERROR(Qwt not (properly) installed. Get a (free) recent version from: http://qwt.sourceforge.net/)
     else
          if test "$1"; then
               AC_SUBST(QWT_LDFLAGS, "-L$1")
               AC_SUBST(QWT_LIBS, "-lqwt")
          fi
          if test "$2"; then
               AC_SUBST(QWT_CXXFLAGS, "-I$2")
          fi
     fi

])
AC_DEFUN([AB_RPM],
[
   case "${build_vendor}" in
       redhat) ac_rpm_root_dir=/usr/src/redhat ;;
       suse) ac_rpm_root_dir=/usr/src/packages  ;;
       *) AC_MSG_WARN("no rpm support for ${build_vendor}") ;;
    esac

    AC_SUBST(RPM_ROOTDIR, $ac_rpm_root_dir)
])
#------------------
# Define auxiliary directories of the installed directory tree.
AC_DEFUN([SPHERE_SET_PATHS],
[

    if test -z "$plugindir"; then
        plugindir='${libdir}/esopipes-plugins/${PACKAGE}-${VERSION}'
    fi

    if test -z "$htmldir"; then
        htmldir='${datadir}/doc/${PACKAGE}/html'
    fi

    if test -z "$configdir"; then
       configdir='${datadir}/${PACKAGE}/config'
    fi

    AC_SUBST(plugindir)
    AC_SUBST(htmldir)
    AC_SUBST(configdir)


    # Define a preprocesor symbol for the plugin search paths

    AC_DEFINE_UNQUOTED(SPHERE_PLUGIN_DIR, "esopipes-plugins",
                       [Plugin directory tree prefix])

    eval plugin_dir="$plugindir"
    plugin_path=`eval echo $plugin_dir | \
                sed -e "s/\/${PACKAGE}-${VERSION}.*$//"`

    AC_DEFINE_UNQUOTED(SPHERE_PLUGIN_PATH, "$plugin_path",
                       [Absolute path to the plugin directory tree])

])

dnl @synopsis SPLIT_VERSION
dnl
dnl Splits a version number in the format MAJOR.MINOR.POINT into it's
dnl separeate components.
dnl 
dnl Sets the variables.
dnl

AC_DEFUN([AB_SPLIT_VERSION],[
    ac_major_version=`echo "$VERSION" | $SED 's/\([[^.]][[^.]]*\).*/\1/'`
    ac_minor_version=`echo "$VERSION" | $SED 's/[[^.]][[^.]]*.\([[^.]][[^.]]*\).*/\1/'`
    ac_point_version=`echo "$VERSION" | $SED 's/[[^.]][[^.]]*.[[^.]][[^.]]*.\(.*\)/\1/'`

    AC_SUBST(MAJOR_VERSION, $ac_major_version)
    AC_SUBST(MINOR_VERSION, $ac_minor_version)
    AC_SUBST(POINT_VERSION, $ac_point_version)
    AC_SUBST(LIBRARY_VERSION, `echo $MAJOR_VERSION + $MINOR_VERSION | bc`:`expr $POINT_VERSION + 0`:`expr $MINOR_VERSION + 0`)

    AC_MSG_CHECKING([Major version])
    AC_MSG_RESULT([$MAJOR_VERSION])
    AC_MSG_CHECKING([Minor version])
    AC_MSG_RESULT([$MINOR_VERSION])
    AC_MSG_CHECKING([Point version])
    AC_MSG_RESULT([$POINT_VERSION])
    AC_MSG_CHECKING([Library version])
    AC_MSG_RESULT([$LIBRARY_VERSION])
])
# Configure paths for termcap lib
# Yoshihiro OKUMURA
#

dnl AM_PATH_TERMCAP()
dnl test for termcap, and define TERMCAP_LIBS
dnl
AC_DEFUN([AM_PATH_TERMCAP],
[dnl
  TERMCAP_LIBS=""

# checking headers
  AC_CHECK_HEADERS([ncurses.h termcap.h curses.h term.h])
  if test $ac_cv_header_ncurses_h = no && test $ac_cv_header_termcap_h = no && test $ac_cv_header_curses_h = no && test $ac_cv_header_term_h = no ; then
    AC_MSG_ERROR([*** curses or termcap header not installed - please install first ***])
  fi

# checking libs
  AC_CHECK_LIB(ncurses, tgetent, have_ncurses=yes, have_ncurses=no)
  AC_CHECK_LIB(curses, tgetent, have_curses=yes, have_curses=no)
  AC_CHECK_LIB(termcap, tgetent, have_termcap=yes, have_termcap=no)

  LDFLAGS=$OLD_LDFLAGS

  if test x$have_ncurses = xyes; then
    TERMCAP_LIBS="-lncurses"
  elif test x$have_curses = xyes; then
    TERMCAP_LIBS="-lcurses"
  fi
  if test x$have_termcap = xyes; then
    TERMCAP_LIBS="-ltermcap"
  else
    if test -r "/usr/lib/termcap/libtermcap.so"; then
      AC_MSG_NOTICE(Found termcap lib /usr/lib/termcap/libtermcap.so)
      TERMCAP_LIBS="-ltermcap"
    else
      AC_MSG_ERROR([*** curses or termcap lib not installed - please install first ***])
    fi
  fi

  AC_SUBST(TERMCAP_LIBS)
])

AC_DEFUN([AC_CHECK_XM],
[
  AC_MSG_CHECKING(for openmotif-devel)
    if test -r "/usr/lib/libXm.so"; then
      AC_MSG_RESULT(Found lib /usr/lib/libXm.so)
    else
      AC_MSG_RESULT(no)
      AC_MSG_ERROR(Couldn't find libXm.so. Please install openmotif-devel first.)
    fi

])

AC_DEFUN([AC_CHECK_XPM],
[
  AC_MSG_CHECKING(for libXpm-devel)
    if test -r "/usr/lib/libXpm.so"; then
      AC_MSG_RESULT(Found lib /usr/lib/libXpm.so)
    else
      AC_MSG_RESULT(no)
      AC_MSG_ERROR(Couldn't find libXpm.so. Please install xorg-x11-libXpm-devel first.)
    fi

])

